<?php

namespace App\Http\Controllers\accounts;

use App\Http\Controllers\Controller;
use App\Http\Traits\CommonTrait;
use App\Models\Account\Account;
use Illuminate\Http\Request;
use App\Http\Traits\EditLogTrait;
use App\Http\Traits\PermissionsTrait;
use App\Models\Account\Balances;
use App\Models\Account\Group;
use App\Models\Account\VoucherAccDetails;
use App\Models\Account\VoucherDesc;
use App\Models\Booking\BookGr;
use App\Models\FleetOwner\DieselAllotment;
use App\Models\FleetOwner\DieselReceipt;
use App\Models\FleetOwner\LoadingSlip;
use App\Models\Inventory\PurchaseDesc;
use App\Models\Master\Vehicle;
use App\Models\Pod\Pod;
use Illuminate\Support\Facades\DB;
use Carbon\Carbon;

class accountController extends Controller
{

    use PermissionsTrait;
    use EditLogTrait;
    use CommonTrait;

    public function index(Request $request)
    {
        $test = $this->is_read_permitted($request->user->id, 'Accounts');
        if (!$test) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        // Fetch and send all accounts
        $currentYear = $request->current_year;
        $accounts = Account::select('id', 'group_id', 'account_name', 'created_by')
                    ->with(['group:id,group_name'])
                    ->with('balances', function ($query) use ($currentYear) {
                        $query->select('id','account_id','opening_bal_dr','opening_bal_cr','fin_year')->where('fin_year', $currentYear)->first();
                    })
                    ->get();
        return response()->json($accounts);
    }

    public function show(Request $request, $pk)
    {
        // Fetch and send the account
        $currentYear = $request->current_year;
        $account = Account::select('id', 'group_id', 'account_name', 'created_by')
                    ->with(['group:id,group_name', 'balances'=> function ($query) use ($currentYear) {
                        $query->select('id','account_id','opening_bal_dr','opening_bal_cr')->where('fin_year', $currentYear)->first();
                    }])
                    ->find($pk);
          
        $data = [
            'id' => $account->id,
            'group' => $account->group,
            'account_name' => $account->account_name,
            'opening_bal_dr' => count($account->balances) ? $account->balances[0]->opening_bal_dr : 0,
            'opening_bal_cr' => count($account->balances) ? $account->balances[0]->opening_bal_cr : 0,
        ];

        return response()->json(['account' => $data]); 
    }

    public function store(Request $request)
    {
        $test = $this->is_new_permitted($request->user->id, 'Accounts');
        if (!$test) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        $validatedData = $request->validate([
            'group' => 'required',
            'account_name' => 'required|unique:accounts,account_name',
        ], [
            'group.required' => 'Required',
            'account_name.required' => 'Required',
            'account_name.unique' => 'Already Exists!',
        ]);

        try {
            // Transaction begins
            DB::beginTransaction();

            // Save a new Account
            $account = new Account();
            $account->group_id = $request->group['id'];
            $account->account_name = $request->account_name;
            $account->account_type = $request->group['group_name'];
            $account->created_by = $request->user->id;
            $account->save();

            $bal = new Balances();
            $bal->account_id = $account->id;
            $bal->opening_bal_dr = $request->opening_bal_dr ? $request->opening_bal_dr : 0.00;
            $bal->opening_bal_cr = $request->opening_bal_cr ? $request->opening_bal_cr : 0.00;
            $bal->opening_bal_date = Carbon::now();
            $bal->fin_year = $request->current_year;
            $bal->save();

            // Save the edit log
            $newVal = $this->get_data_for_obj($account->id,$request->current_year);
            $this->createEditLog('created', 'accounts', $account->id, $account->account_name, null, $newVal, $request->user->id, $request->current_year); // Create edit log


            // Transaction committed
            DB::commit();

            
            return response()->json("Success", 200); // Returned response

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function update(Request $request, $pk){   
        $test = $this->is_edit_permitted($request->user->id, 'Accounts');
        $test_edit = $test->edit;
        $edit_valid_till_hours = $test->edit_valid_till;

        if( !$test_edit ){
            return response()->json(['Permission Denied For The Action'],401);
        }

        $validatedData = $request->validate([
            'group' => 'required',
            'account_name' => 'required',
        ], [
            'group.required' => 'Required',
            'account_name.required' => 'Required',
        ]);
        
        $account = Account::find($pk);

        // FETCHING OLD VALUE FOR LOG
        $oldValue = $this->get_data_for_obj($pk,$request->current_year);

        if( $edit_valid_till_hours!=null ){
            $validity = Carbon::parse($account->created_at)->addHours($edit_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }
          

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();

            // SAVING A Acount
            $account->group_id = $request->group['id'];
            $account->account_name = $request->account_name;
            if($account->account_type!='Party' && $account->account_type!='Staff' && $account->account_type!='Driver' && $account->account_type!='Vehicle')
            $account->account_type = $request->group['group_name'];
            $account->updated_by = $request->user->id;
            $account->save();
            // Acount SAVED

            $bal = Balances::where('account_id',$account->id)->where('fin_year',$request->current_year)->first();
            $bal->opening_bal_dr = $request->opening_bal_dr ? $request->opening_bal_dr : 0.00;
            $bal->opening_bal_cr = $request->opening_bal_cr ? $request->opening_bal_cr : 0.00;
            $bal->save();

            $newVal = $this->get_data_for_obj($account->id,$request->current_year);
            $this->createEditLog('updated', 'accounts', $account->id, $account->account_name, $oldValue, $newVal, $request->user->id,$request->current_year); // CREATE EDIT LOG

            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function destroy(Request $request, $pk)
    {
        $test = $this->is_delete_permitted($request->user->id, 'Accounts');
        $test_delete = $test->delete;
        $delete_valid_till_hours = $test->delete_valid_till;

        if (!$test_delete) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();
            

        $account = Account::find($pk);

        if(!$account->created_by){
            return response()->json(['Pre Set accounts Cannot Be Removed'], 401);
        }

        if ($delete_valid_till_hours != null) {
            $validity = Carbon::parse($account->created_at)->addHours($delete_valid_till_hours);
            if (!$validity->isFuture()) {
                return response()->json(['Permission Timed Out'], 500);
            }
        }

        // Fetch the old value for log
        $oldValue = $this->get_data_for_obj($pk, $request->current_year);

        $balances = Balances::where('account_id', $pk)->delete();

        // Find and delete the account
        $account->delete();

        $this->createEditLog('deleted', 'accounts', $account->id, $account->account_name, $oldValue, null, $request->user->id,$request->current_year); // Create edit log

            // TRANSACTIONS COMMITTED
            DB::commit();
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function autocomplete(Request $request){
        // Initialize the query builder
        $fields = ['id', 'account_name'];
        
        if ($request->fields) {
            $fields = array_merge($fields, $request->fields);
        }

        $query = Account::select($fields);

        // Apply any additional filters based on other columns as needed
        if ($request->has('filterby')) {
            $orWhereFilters = isset($request->filterby['orWhere']) ? $request->filterby['orWhere'] : [];
            $whereFilters = isset($request->filterby['where']) ? $request->filterby['where'] : [];
            $notInFilters = isset($request->filterby['notInFilters']) ? $request->filterby['notInFilters'] : [];
            $whereGroup = isset($request->filterby['whereGroup']) ? $request->filterby['whereGroup'] : [];
            $excludeGroup = isset($request->filterby['excludeGroup']) ? $request->filterby['excludeGroup'] : [];
            $whereInFilters = isset($request->filterby['whereIn']) ? $request->filterby['whereIn'] : [];


           // Loop through the whereFilters array
           foreach ($whereFilters as $whereFilter) {
               // Apply the filter to the query
               $query->where($whereFilter['key'], $whereFilter['value']);
           }

           // Loop through the orWhereFilters array
           foreach ($orWhereFilters as $orWhereFilter) {
               // Apply the filter to the query
               $query->orWhere($orWhereFilter['key'], $orWhereFilter['value']);
           }

           foreach ($notInFilters as $notInFilter) {
               if(isset($notInFilter['key']) && isset($notInFilter['values']))
               $query->whereNotIn($notInFilter['key'], $notInFilter['values']);
           }

           foreach ($whereInFilters as $whereInFilter) {
                
                if(isset($whereInFilter['key']) && isset($whereInFilter['values']))
                $query->whereIn($whereInFilter['key'], $whereInFilter['values']);
            
           }  

           if ($whereGroup) {
                $query->where(function ($query) use ($whereGroup, $excludeGroup) {
                    $query->whereDoesntHave('group', function ($query) use ($excludeGroup) {
                        $query->whereIn('group_name', $excludeGroup);
                    })
                    ->where(function ($query) use ($whereGroup, $excludeGroup) {
                        $query->whereHas('group', function ($query) use ($whereGroup) {
                            $query->whereIn('group_name', $whereGroup);
                        })
                        ->orWhereHas('group.under_grp', function ($query) use ($whereGroup) {
                            $query->whereIn('group_name', $whereGroup);
                        });
                    });
                });
            }
        
        }
        
        
        // Check if searchVariable is provided and apply the filter
        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('account_name', 'like', '%'. $searchVariable . '%');
        }

        

        if($request->has('with_relations')){
            $query->with($request->with_relations); 
        }

        // Limit the number of results
        $query->orderBy('account_name','asc')->take($request->limit);

        // Retrieve the filtered 
        $accounts = $query->get();
        return response(['accounts' => $accounts, 'draw' => $request->draw]);
    }

    public function datatable(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'Accounts');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $currentYear = $request->current_year; // Set the current year based on your requirements

        $query = Account::select('id', 'account_name', 'account_type', 'group_id','no_delete', 'created_by')
            ->with(['group:id,group_name', 'balances'=> function ($query) use ($currentYear) {
                $query->select('id','account_id','opening_bal_dr','opening_bal_cr','fin_year')->where('fin_year', $currentYear);
            }])
            ->orderBy($column, $dir);

        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('account_name', 'like', $searchValue . '%');
            });
        }

        $data = $query->paginate($length);

        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    public function check_unique(Request $request)
    {
        $key = $request->key;
        $value = $request->value;

        if (!$value) {
            return response()->json(['exists' => false]);
        }

        $exists = Account::where($key, $value)->exists();
        if ($exists) {
            return response()->json(['errors' => [$key  => ['Already Exists !']]], 422);
        }

        return response()->json(['exists' => false]);
    }



    public function get_ledger(Request $request, $pk) {
        $currentYear = $request->current_year;
        $min_max_dates = $this->get_min_max_date($currentYear);

        $branch_id = $request->branch_id;

        $min_date = $request->min_date ? $request->min_date : $min_max_dates['min_date'] ;
        $from_date = $request->from_date ? $request->from_date : $min_max_dates['min_date'] ;
        $to_date = $request->to_date ? $request->to_date : $min_max_dates['max_date'];
        $print_type = $request->print_type ? $request->print_type : 'Monthly';

        
        
        $account = Account::select('id', 'account_name', 'account_type', 'group_id')
            ->with(['group:id,group_name,nature', 
            'balances'=> function ($query) use ($currentYear) {
                $query->select('id','account_id','opening_bal_dr','opening_bal_cr','fin_year')->where('fin_year', $currentYear);
            },
            'voucher_acc_details'=> function ($qr) use ($currentYear,$branch_id) {
                $qr->select('id','voucher_id','account_id','nature','dr_amount','cr_amount','active_status')->where('fin_year', $currentYear);

                if($branch_id){
                    $qr->whereHas('voucher', function ($subQuery) use ($branch_id,$currentYear) {
                        $subQuery->where('branch_id', $branch_id)->where('fin_year', $currentYear);
                    });
                }
            },
            'voucher_acc_details.voucher:id,branch_id,voucher_number,voucher_date,voucher_name,t_against_id,t_against,t_against_no,narration,active_status','voucher_acc_details.account:id,account_name'])
            ->find($pk);
 
        
        $account_nature = $account->group->nature;
        
        if($account_nature=='Dr')
        $opening_balance = $account->balances[0]->opening_bal_dr - $account->balances[0]->opening_bal_cr;
        elseif($account_nature=='Cr')
        $opening_balance = $account->balances[0]->opening_bal_cr - $account->balances[0]->opening_bal_dr;

        // Calculate opening balance
        if ($min_date !== $from_date) {
            $opening_bal = $account->voucher_acc_details
                                ->filter(function ($voucher) use ($min_date, $from_date) {
                                    return $voucher->voucher->voucher_date >= $min_date && $voucher->voucher->voucher_date < $from_date && $voucher->active_status;
                                })
                                ->sum(function ($voucher) {
                                    return $voucher->dr_amount - $voucher->cr_amount;
                                });
                           
                         
            $opening_balance = $opening_balance+$opening_bal;
        } 
        
        
    
        // Filter voucher details by date range
        $filtered_voucher_details = $account->voucher_acc_details->filter(function ($voucher) use ($from_date, $to_date) {
            return $voucher->voucher->voucher_date >= $from_date && $voucher->voucher->voucher_date <= $to_date && $voucher->active_status;
        });


        $final_result = [];

        foreach ($filtered_voucher_details as $entry) {
            if ($entry->nature === 'dr') {
                // Find the corresponding Cr entry
                $opp = VoucherAccDetails::select('id','voucher_id','account_id')->where('fin_year',$currentYear)->where('voucher_id',$entry->voucher_id)->where('nature','Cr')->with(['account:id,account_name'])->where('active_status',1)->get();

                $entry->opposite_accounts = $opp;

            } elseif ($entry->nature === 'cr') {
                // Find the corresponding Dr entry
                $opp = VoucherAccDetails::select('id','voucher_id','account_id')->where('fin_year',$currentYear)->where('voucher_id',$entry->voucher_id)->where('nature','Dr')->with(['account:id,account_name'])->where('active_status',1)->get();

                $entry->opposite_accounts = $opp;
            }

            $formatted_entry = [
                'voucher_acc_details_id' => $entry->id,
                'voucher_date' => Carbon::parse($entry->voucher->voucher_date)->format('d-M-y'),
                'opposite_accounts' => $entry->opposite_accounts,
                'narration' => $entry->voucher->narration,
                'voucher_type' => $entry->voucher->voucher_name,
                'voucher_id' => $entry->voucher_id,
                'voucher_number' => $entry->voucher->voucher_number,
                'account_id' => $entry->account->id,
                'nature_of_transaction' => $entry->nature,
                'dr_amount' => $entry->dr_amount,
                'cr_amount' => $entry->cr_amount,
                't_against' => $entry->voucher->t_against,
                't_against_id' => $entry->voucher->t_against_id,
                't_against_no' => $entry->voucher->t_against_no,
            ];
        
            $final_result[] = $formatted_entry;
        }
        


        // Additional logic for handling 'Monthly' and 'Daily' print_type
            if ($print_type === 'Monthly') {
                // Group transactions by month and calculate sums
                $monthly_transactions = [];
                foreach ($final_result as $entry) {
                    $voucher_date = Carbon::parse($entry['voucher_date']);
                    $month = $voucher_date->format('M');
                    
                    if (!isset($monthly_transactions[$month])) {
                        $monthly_transactions[$month] = [
                            'dr_amount' => 0,
                            'cr_amount' => 0,
                        ];
                    }

                    $monthly_transactions[$month]['dr_amount'] += $entry['dr_amount'];
                    $monthly_transactions[$month]['cr_amount'] += $entry['cr_amount'];
                    $monthly_transactions[$month]['date_or_month'] = $month;
                    $monthly_transactions[$month]['account_id'] = $entry['account_id'];
                }

                $data = $monthly_transactions;
            } elseif ($print_type === 'Daily') {
                // Group transactions by date and calculate sums
                $daily_transactions = [];
                foreach ($final_result as $entry) {
                    $voucher_date = Carbon::parse($entry['voucher_date']);
                    $date = $voucher_date->format('Y-m-d');
                    $formatted_date = $voucher_date->format('d-M-Y');
                    
                    if (!isset($daily_transactions[$date])) {
                        $daily_transactions[$date] = [
                            'dr_amount' => 0,
                            'cr_amount' => 0,
                        ];
                    }

                    $daily_transactions[$date]['dr_amount'] += $entry['dr_amount'];
                    $daily_transactions[$date]['cr_amount'] += $entry['cr_amount'];
                    $daily_transactions[$date]['date_or_month'] = $formatted_date;
                    $daily_transactions[$date]['account_id'] = $entry['account_id'];
                }

                $data = $daily_transactions;
            }
            elseif ($print_type === 'All'){
                $data = $final_result;
            }
        
            //$data to array
            $data = array_values($data);

        $opening_balance_dr = 0;
        $opening_balance_cr = 0;

        if($opening_balance>0 && $account_nature == 'Dr'){
            $opening_balance_dr = $opening_balance;
        }
        else if($opening_balance>0 && $account_nature == 'Cr'){
            $opening_balance_cr = $opening_balance;
        }
        else if($opening_balance<0 && $account_nature == 'Dr'){
            $opening_balance_cr = abs($opening_balance);
        }
        else if($opening_balance<0 && $account_nature == 'Cr'){
            $opening_balance_dr = abs($opening_balance);
        }else{
            $opening_balance_dr = 0;
            $opening_balance_cr = 0;
        }
    
        $datax = [
            'opening_bal_dr' => $opening_balance_dr,
            'opening_bal_cr' => $opening_balance_cr,
            'listItems' =>  $data,
            'account_nature' => $account_nature,
        ];
    
        return response()->json($datax);
    }

    public function get_trial_balance(Request $request) {
        $currentYear = $request->current_year;
        $min_max_dates = $this->get_min_max_date($currentYear);

        $branch_id = $request->branch_id;
        $min_date = $request->min_date ? $request->min_date : $min_max_dates['min_date'] ;
        $from_date = $request->from_date ? $request->from_date : $min_max_dates['min_date'] ;
        $to_date = $request->to_date ? $request->to_date : $min_max_dates['max_date'];
    
        $groups = Group::select('id', 'group_name', 'nature')->where('is_primary', 1)->get();
    
        $data = [];
        
        $sum_of_opening_dr = 0;
        $sum_of_opening_cr = 0;
    
        foreach ($groups as $group) {
            $under_group = Group::select('id', 'group_name', 'nature')->where('under_grp_id', $group->id)->get();
            $ug_ids = $under_group->pluck('id');
    
            $accounts = Account::select('id')->where('group_id', $group->id)
                ->orWhereIn('group_id', $ug_ids)
                ->pluck('id');
    
            $dr_total = 0;
            $cr_total = 0;
    
            foreach ($accounts as $account_id) {
                $opening_balance_dr = 0;
                $opening_balance_cr = 0;
    
                $balance = $this->calculate_balance_for_account($account_id, $min_date,$from_date,$to_date,$currentYear, $branch_id);
                
                if($balance['nature_of_op_bal'] == 'Dr'){
                    $opening_balance_dr = $balance['opening_balance'];
                }
                else if($balance['nature_of_op_bal'] == 'Cr'){
                    $opening_balance_cr = $balance['opening_balance'];
                }


                if($balance['nature_of_balance']=='Dr'){
                    $dr_total += $balance['current_balance'];
                }
                else{
                    $cr_total += $balance['current_balance'];
                }

                $sum_of_opening_dr += $opening_balance_dr;
                $sum_of_opening_cr += $opening_balance_cr;
            }
    
            $diff = $dr_total - $cr_total;
            if($diff>=0 ){
                $dr_balance = $diff;
                $cr_balance = 0;
            }
            else if($diff<0){
                $dr_balance = 0;
                $cr_balance = abs($diff);
            } 
    
            if($dr_balance>0 || $cr_balance>0){
                $data[$group->group_name] = [
                    'group' => $group,
                    'dr_balance' => $dr_balance,
                    'cr_balance' => $cr_balance,
                ];

            }
            
        }
    
        $data = array_values($data);
        $opening_difference_dr = max($sum_of_opening_cr - $sum_of_opening_dr, 0);
        $opening_difference_cr = max($sum_of_opening_dr - $sum_of_opening_cr, 0);
    
        $datax = [
            'opening_difference_dr' => $opening_difference_dr,
            'opening_difference_cr' => $opening_difference_cr,
            'listItems' => $data,
        ];
    
        return response()->json($datax);
    }

    public function get_truck_pl_ledger(Request $request, $pk) {
        $currentYear = $request->current_year;
        $min_max_dates = $this->get_min_max_date($currentYear);

        $branch_id = $request->branch_id;

        $truck_id = $pk;

        $min_date = $request->min_date ? $request->min_date : $min_max_dates['min_date'] ;
        $from_date = $request->from_date ? $request->from_date : $min_max_dates['min_date'] ;
        $to_date = $request->to_date ? $request->to_date : $min_max_dates['max_date'];
        $print_type = $request->print_type ? $request->print_type : 'Monthly';

        $payment_voucher = VoucherDesc::select('id','voucher_id','voucher_number','receipt_amt','v_acc_det_id')->where('against', 'VEHICLE_COST_CENTER')
                            ->where('against_id', $truck_id)
                            ->whereIn('voucher_name', ['Payment Voucher', 'Journal'])
                            ->where('fin_year', $currentYear)
                            ->with(['voucher:id,voucher_number,voucher_date,narration','voucher_acc_details:id,account_id','voucher_acc_details.account:id,account_name'])
                            ->whereHas('voucher', function ($query) use ($from_date, $to_date,$branch_id) {
                                $query->where('voucher_date', '>=', $from_date)
                                    ->where('voucher_date', '<=', $to_date)
                                    ->when($branch_id, function ($query, $branch_id) {
                                        return $query->where('branch_id', $branch_id);
                                    });
                            })
                            ->get();

       
        $diesel_receipt = DieselReceipt::select('id','dr_number','booking_date','amount','remarks','dr_on_account')
                            ->where('vehicle_id', $truck_id)
                            ->where('fin_year', $currentYear)
                            ->where('booking_date', '>=', $from_date)
                            ->where('booking_date', '<=', $to_date)
                            ->with(['dr_on_account:id,account_name'])
                            ->when($branch_id, function ($query, $branch_id) {
                                return $query->where('branch_id', $branch_id);
                            })
                            ->get();

        $purchase_desc = PurchaseDesc::select('id','bill_id','bill_number','net_amount','dr_account_id','vehicle_id','product_id')
                            ->where('vehicle_id',$truck_id)
                            ->whereHas('purchase', function ($query) use ($from_date, $to_date,$branch_id) {
                                $query->where('bill_date', '>=', $from_date)
                                    ->where('bill_date', '<=', $to_date)
                                    ->when($branch_id, function ($query, $branch_id) {
                                        return $query->where('branch_id', $branch_id);
                                    });
                            })
                            ->with(['dr_account:id,account_name','purchase:id,bill_name,bill_number,bill_date,remarks','product:id,product_name'])
                           ->where('fin_year',$currentYear)
                           ->get();
                        
        $pods = Pod::select('id','pod_number','delivery_date','grand_total','remarks','income_account_id')->where('vehicle_id', $truck_id)
                            ->where('fin_year', $currentYear)
                            ->where('delivery_date', '>=', $from_date)
                            ->where('delivery_date', '<=', $to_date)
                            ->with(['income:id,account_name'])
                            ->when($branch_id, function ($query, $branch_id) {
                                return $query->where('branch_id', $branch_id);
                            })
                            ->get();
                        
        $gr_without_pod = BookGr::select('id','gr_number','gr_date','grand_total','remarks','income_account_id')->where('vehicle_id', $truck_id)
                            ->where('fin_year', $currentYear)
                            ->where('gr_date', '>=', $from_date)
                            ->where('gr_date', '<=', $to_date)
                            ->with(['income:id,account_name'])
                            ->when($branch_id, function ($query, $branch_id) {
                                return $query->where('branch_id', $branch_id);
                            })
                            ->whereDoesntHave('pod_desc')
                            ->get();
                        
        $loading_slip = LoadingSlip::select('id','ls_number','ls_date','branch_id','total_without_gst','grand_total','remarks','income')->where('vehicle_id', $truck_id)
                            ->where('fin_year', $currentYear)
                            ->where('ls_date', '>=', $from_date)
                            ->where('ls_date', '<=', $to_date)
                            ->with(['income:id,account_name'])
                            ->when($branch_id, function ($query, $branch_id) {
                                return $query->where('branch_id', $branch_id);
                            })
                            ->get();

        $diesel_allotment = DieselAllotment::whereHas('loading_slip', function ($query) use ($truck_id, $currentYear, $from_date, $to_date, $branch_id) {
                                $query->where('vehicle_id', $truck_id)
                                      ->where('fin_year', $currentYear)
                                      ->where('ls_date', '>=', $from_date)
                                      ->where('ls_date', '<=', $to_date)
                                      ->when($branch_id, function ($query, $branch_id) {
                                          return $query->where('branch_id', $branch_id);
                                      });
                            })->with(['loading_slip','loading_slip.income:id,account_name'])->get();
                            
        
        
        
        // LOOPING ALL THE ABOVE
        $final_result = [];
        

        // Combine results into a single array
        foreach ($payment_voucher as $item) {
            $final_result[] = [
                'date' => $item->voucher->voucher_date,
                'remarks' => $item->voucher->narration,
                'on_account' => $item->voucher_acc_details->account, // Set the value based on your logic
                'against' => 'Voucher', // Set the value based on your logic
                'against_id' => $item->voucher_id,
                'against_no' => $item->voucher_number,
                'transaction_type' => 'Voucher', // Set the value based on your logic
                'dr_amount' => $item->receipt_amt,
                'cr_amount' => 0, // Set the value based on your logic
            ];
        }

        foreach ($diesel_receipt as $item) {
            $hehe = $item->toArray();
            $drOnAccount = $hehe['dr_on_account'];

            // Similar logic to extract data from $item and add it to $final_result
            $final_result[] = [
                'date' => $item->booking_date,
                'remarks' => $item->remarks,
                'on_account' => $drOnAccount, // Set the value based on your logic
                'against' => 'Diesel Receipt', // Set the value based on your logic
                'against_id' => $item->id,
                'against_no' => $item->dr_number,
                'transaction_type' => 'Diesel Receipt', // Set the value based on your logic
                'dr_amount' => $item->amount,
                'cr_amount' => 0, // Set the value based on your logic
            ];
        }

        foreach ($purchase_desc as $item) {

            // Similar logic to extract data from $item and add it to $final_result
            $final_result[] = [
                'date' => $item->purchase->bill_date,
                'remarks' => $item->purchase->remarks,
                'on_account' => $item->dr_account, // Se the value based on your logic
                'against' => $item->purchase->bill_name, // Set the value based on your logic
                'against_id' => $item->bill_id,
                'against_no' => $item->bill_number,
                'transaction_type' => $item->purchase->bill_name, // Set the value based on your logic
                'dr_amount' => $item->net_amount,
                'cr_amount' => 0, // Set the value based on your logic
            ];
        }

        foreach ($pods as $item) {
            // Similar logic to extract data from $item and add it to $final_result
            $final_result[] = [
                'date' => $item->delivery_date,
                'remarks' => $item->remarks,
                'on_account' => $item->income, // Set the value based on your logic
                'against' => 'POD', // Set the value based on your logic
                'against_id' => $item->id,
                'against_no' => $item->pod_number,
                'transaction_type' => 'POD', // Set the value based on your logic
                'dr_amount' => '',
                'cr_amount' => $item->grand_total, // Set the value based on your logic
            ];
        }

        foreach ($gr_without_pod as $item) {
            // Similar logic to extract data from $item and add it to $final_result
            $final_result[] = [
                'date' => $item->gr_date,
                'remarks' => $item->remarks,
                'on_account' => $item->income, // Set the value based on your logic
                'against' => 'GR', // Set the value based on your logic
                'against_id' => $item->id,
                'against_no' => $item->gr_number,
                'transaction_type' => 'GR', // Set the value based on your logic
                'dr_amount' => '',
                'cr_amount' => $item->grand_total, // Set the value based on your logic
            ];
        }

        foreach ($loading_slip as $item) {
            $hehe = $item->toArray();
            $income = $hehe['income'];

            // Similar logic to extract data from $item and add it to $final_result
            $final_result[] = [
                'date' => $item->ls_date,
                'remarks' => $item->remarks,
                'on_account' => $income, // Set the value based on your logic
                'against' => 'Loading Slip', // Set the value based on your logic
                'against_id' => $item->id,
                'against_no' => $item->ls_number,
                'transaction_type' => 'Loading Slip', // Set the value based on your logic
                'dr_amount' => '',
                'cr_amount' => $item->grand_total, // Set the value based on your logic
            ];
        }

        foreach ($diesel_allotment as $item) {

            // Similar logic to extract data from $item and add it to $final_result
            $final_result[] = [
                'date' => $item->loading_slip->ls_date,
                'remarks' => $item->loading_slip->remarks,
                'on_account' => ['id'=>18, 'account_name'=>'Diesel Expense'], // Se the value based on your logic
                'against' => 'Loading Slip', // Set the value based on your logic
                'against_id' => $item->loading_slip->id,
                'against_no' => $item->loading_slip->ls_number,
                'transaction_type' =>'Loading Slip', // Set the value based on your logic
                'dr_amount' => $item->amt,
                'cr_amount' => 0, // Set the value based on your logic
            ];
        }

                

        // Sort final_result based on 'date'.
        usort($final_result, function ($a, $b) {
            return strtotime($a['date']) - strtotime($b['date']);
        });




    
        return response()->json($final_result);
    }

    public function get_truck_ledger_balance(Request $request) {
        $currentYear = $request->current_year;
        $min_max_dates = $this->get_min_max_date($currentYear);

        $branch_id = $request->branch_id;
        $min_date = $request->min_date ? $request->min_date : $min_max_dates['min_date'] ;
        $from_date = $request->from_date ? $request->from_date : $min_max_dates['min_date'] ;
        $to_date = $request->to_date ? $request->to_date : $min_max_dates['max_date'];

        $trucks = Vehicle::select('id', 'vehicle_number', 'account_id')->get();

        $data = [];

        foreach ($trucks as $truck) {
            $truck_id = $truck->id;
            $truck_no = $truck->vehicle_number;

            $truck_data = $this->get_truck_data($truck_id, $truck_no, $min_date, $from_date, $to_date, $currentYear, $branch_id);
            
            $truck_data['account_id'] = $truck['account_id'];

            if($truck_data['dr_amount']>0 || $truck_data['cr_amount']>0){
                $data[] = $truck_data;
            }
        }


        return response()->json($data);
    }

    public function get_expanded_group(Request $request, $pk) {

        $currentYear = $request->current_year;

        $min_max_dates = $this->get_min_max_date($currentYear);

        $branch_id = $request->branch_id;
        $min_date = $request->min_date ? $request->min_date : $min_max_dates->min_date ;
        $from_date = $request->from_date ? $request->from_date : $min_max_dates->min_date ;
        $to_date = $request->to_date ? $request->to_date : $min_max_dates->max_date;


    
        $group = Group::select('id', 'group_name', 'nature', 'group_type')->find($pk);
    
            $under_group = Group::select('id', 'group_name', 'nature')->where('under_grp_id', $group->id)->get();
            $ug_ids = $under_group->pluck('id');
    
            $accounts = Account::select('id','account_name','group_id')->where('group_id', $group->id)
                        ->orWhereIn('group_id', $ug_ids)
                        ->get();
    
            $group_total_dr = 0;
            $group_total_cr = 0;

    
            foreach ($accounts as $account) {
                $dr_balance = 0;
                $cr_balance = 0;

                $balance = $this->calculate_balance_for_account($account->id, $min_date,$from_date,$to_date,$currentYear,$branch_id);
                
                if($balance['nature_of_balance']=='Dr'){
                    $dr_balance = $balance['current_balance'];
                    $group_total_dr += $dr_balance;
                }
                elseif($balance['nature_of_balance']=='Cr'){
                    $cr_balance = $balance['current_balance'];
                    $group_total_cr += $cr_balance;  
                }
                  
                // Check if $account->group_id inside $ug_ids
                $is_under_group = $ug_ids->contains($account->group_id);
                $sub_group = $is_under_group ? $account->group->group_name : $group->group_name;
    
                if($dr_balance>0 || $cr_balance>0){
                    $account_data[$sub_group][] = [
                        'account' => $account,
                        'dr_balance' => $dr_balance,
                        'cr_balance' => $cr_balance,
                    ];
                    $sg_tot_dr = $account_data[$sub_group]['sub_group_total_dr'] ?? 0;
                    $sg_tot_cr = $account_data[$sub_group]['sub_group_total_cr'] ?? 0;
                    $account_data[$sub_group]['sub_group_total_dr'] = $sg_tot_dr + $dr_balance;
                    $account_data[$sub_group]['sub_group_total_cr'] = $sg_tot_cr + $cr_balance;
                }
                                
            }
    

            $group_data = [
                'group_total_dr' => $group_total_dr,
                'group_total_cr' => $group_total_cr,
                'group_name' => $group->group_name,
                'listItems' => $account_data??null,
            ];
    
        return response()->json($group_data);
    }

    public function get_profit_and_loss(Request $request){
        
        $currentYear = $request->current_year;


        $min_max_dates = $this->get_min_max_date($currentYear);


        $primary_groups = Group::select('id', 'group_name', 'nature', 'group_type')->where('is_primary', 1)->whereIn('group_type',['Income','Expense'])->get();

        $expense_side = [];
        $income_side = [];

        foreach ($primary_groups as $primary_grp) {
            
            $data = $this->get_expanded_group($request, $primary_grp->id)->original;

            
            if($primary_grp->group_type == 'Expense'){
                if(isset($data['listItems'])){
                    $expense_side[] = $data;
                }
            }
            else if($primary_grp->group_type == 'Income'){
                if(isset($data['listItems'])){
                    $income_side[] = $data;
                }
            }
        

        }

        return response()->json(['expense_side' => $expense_side, 'income_side' => $income_side]);
        

    }

    public function get_balance_sheet(Request $request){
        $currentYear = $request->current_year;
        $min_max_dates = $this->get_min_max_date($currentYear);

        $primary_groups = Group::select('id', 'group_name', 'nature', 'group_type')->where('is_primary', 1)->whereIn('group_type',['Asset','Liability'])->get();

        $asset_side = [];
        $liability_side = [];

        foreach ($primary_groups as $primary_grp) {
            
            $data = $this->get_expanded_group($request, $primary_grp->id)->original;
            
            if($primary_grp->group_type == 'Asset'){
                if(isset($data['listItems'])){
                    $asset_side[] = $data;
                }
            }
            else if($primary_grp->group_type == 'Liability'){
                if(isset($data['listItems'])){
                    $liability_side[] = $data;
                }
            }
        }

        // Calculate the total of asset_side and liability_side
        $profit_and_loss_amount = $this->get_profit_and_loss_amount($request);
        
        if($profit_and_loss_amount['profit_and_loss']>=0){
            $liability_side[] = [
                'group_total_dr' => 0,
                'group_total_cr' => $profit_and_loss_amount['profit_and_loss'],
                'group_name' => 'Net Profit',
                'listItems' => null,
            ];
        }
        else{
            $asset_side[] = [
                'group_total_dr' => abs($profit_and_loss_amount['profit_and_loss']),
                'group_total_cr' => 0,
                'group_name' => 'Net Loss',
                'listItems' => null,
            ];
        }

        return response()->json(['asset_side' => $asset_side, 'liability_side' => $liability_side]);
        

    }


    public function get_balance_for_account(Request $request, $pk){
        $balance = $this->calculate_balance_for_account($pk,$request->min_date,$request->from_date,$request->to_date,$request->current_year,null);
        return response()->json($balance);
    }

    public function update_balance_of_account(Request $request){
        $res = $this->update_balance_for_account($request->current_year, $request->account_ids);
        return response()->json("Success");
    }

    public function update_openings(Request $request){

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();

        $account_ids = Account::whereNotNull('group_id')
                        ->whereHas('group', function($qr) {
                            $qr->whereIn('group_type', ['Asset', 'Liability']);
                        })
                        ->pluck('id')
                        ->toArray();
        $this->update_balance_for_account($request->current_year, $account_ids);

        // Update the opening balance of profit and loss account
        $allowed_fin_years = explode(',', config('app.ALLOWED_FIN_YEAR'));

        $sum_of_pl = 0;

        $index = array_search($request->current_year, $allowed_fin_years);
        if ($index !== false) {
            // Get an array from the index and so on
            $update_fin_years = array_slice($allowed_fin_years, $index);

             // Loop through the array and update the balances OF P&L
             for ($i = 0; $i < count($update_fin_years); $i++) {
                $fin_year = $update_fin_years[$i];
    
                $min_max_date = $this->get_min_max_date($fin_year);
                $min_date = $min_max_date->min_date;
                $max_date = $min_max_date->max_date;


                $request->merge(['current_year' => $fin_year]);

                $profit_and_loss_amt = $this->get_profit_and_loss_amount($request); 

                // If not in the last iteration
                if ($i < count($update_fin_years) - 1) { 
                        $opening_for_next_year = Balances::where('account_id', 1)
                            ->where('fin_year', $update_fin_years[$i + 1])
                            ->first();

                        $closing_for_current_year = Balances::where('account_id', 1)
                            ->where('fin_year', $update_fin_years[$i])
                            ->first();
                        
                        if(!$opening_for_next_year){
                            $opening_for_next_year = new Balances();
                            $opening_for_next_year->fin_year = $update_fin_years[$i + 1];
                            $opening_for_next_year->account_id = 1;
                        }
                        
                        if ($opening_for_next_year) {
                            if ($profit_and_loss_amt['profit_and_loss'] < 0) {
                                $opening_for_next_year->opening_bal_dr = 0.00;
                                $opening_for_next_year->opening_bal_cr = 0.00;

                                if($closing_for_current_year){
                                    $closing_for_current_year->closing_bal_dr = abs($profit_and_loss_amt['profit_and_loss']);
                                    $closing_for_current_year->closing_bal_cr = 0.00;
                                    $closing_for_current_year->closing_bal_date = $max_date;
                                    $closing_for_current_year->save();
                                }
                            } else {
                                $opening_for_next_year->opening_bal_dr = 0.00;
                                $opening_for_next_year->opening_bal_cr = 0.00;

                                if($closing_for_current_year){
                                    $closing_for_current_year->closing_bal_dr = 0.00;
                                    $closing_for_current_year->closing_bal_cr = abs($profit_and_loss_amt['profit_and_loss']);
                                    $closing_for_current_year->closing_bal_date = $max_date;
                                    $closing_for_current_year->save();
                                }
                            }
        
                            $min_date_for_next = $this->get_min_max_date($update_fin_years[$i + 1])->min_date;
                            $opening_for_next_year->opening_bal_date = $min_date_for_next;
        
                            $opening_for_next_year->save();
                        }
                    }
                    else{
                        $closing_for_last_year = Balances::where('account_id', 1)
                            ->where('fin_year', $update_fin_years[$i])
                            ->first();
                        
                        
                        if ($closing_for_last_year) {
                            if ($profit_and_loss_amt['profit_and_loss'] < 0) {
                                $closing_for_last_year->closing_bal_dr = abs($profit_and_loss_amt['profit_and_loss']);
                                $closing_for_last_year->closing_bal_cr = 0.00;
                            } else {
                                $closing_for_last_year->closing_bal_dr = 0.00;
                                $closing_for_last_year->closing_bal_cr = abs($profit_and_loss_amt['profit_and_loss']);
                            }
        
                            $max_date_for_current = $this->get_min_max_date($update_fin_years[$i])->max_date;
                            $closing_for_last_year->closing_bal_date = $max_date_for_current;
        
                            $closing_for_last_year->save();
                        }
                    }




             }

             // Loop through the array and update the balances OF CAPITAL
             for ($i = 0; $i < count($update_fin_years); $i++) {
                $fin_year = $update_fin_years[$i];
    
                $min_max_date = $this->get_min_max_date($fin_year);
                $min_date = $min_max_date->min_date;
                $max_date = $min_max_date->max_date;


                $request->merge(['current_year' => $fin_year]);

                $profit_and_loss_amt = $this->get_profit_and_loss_amount($request); 

                $capital_account_id = $this->get_acc_id('Capital Account');

                if(!$capital_account_id){
                    return response('Capital Account Not Found',500);
                }


                $sum_of_pl += $profit_and_loss_amt['profit_and_loss'];

                // If not in the last iteration
                if ($i < count($update_fin_years) - 1) { 
                        $capital_opening_for_next_year = Balances::where('account_id', $capital_account_id)
                            ->where('fin_year', $update_fin_years[$i + 1])
                            ->first();

                        $capital_closing_for_current_year = Balances::where('account_id', $capital_account_id)
                            ->where('fin_year', $update_fin_years[$i])
                            ->first();
                        
                        if(!$capital_opening_for_next_year){
                            $capital_opening_for_next_year = new Balances();
                            $capital_opening_for_next_year->fin_year = $update_fin_years[$i + 1];
                            $capital_opening_for_next_year->account_id = $capital_account_id;
                        }
                        
                        if ($capital_opening_for_next_year) {

                                if($capital_closing_for_current_year){
                                   if($capital_closing_for_current_year->closing_bal_dr>0){
                                        $closing_factor = -($capital_closing_for_current_year->closing_bal_dr) + $sum_of_pl;
                                   }
                                   else{
                                       $closing_factor = $capital_closing_for_current_year->closing_bal_cr + $sum_of_pl;
                                   }

                                   if($closing_factor>=0){
                                    $capital_closing_for_current_year->closing_bal_cr = abs($closing_factor);
                                    $capital_closing_for_current_year->closing_bal_dr = 0.00;
                                   }
                                   else{
                                    $capital_closing_for_current_year->closing_bal_dr = abs($closing_factor);
                                    $capital_closing_for_current_year->closing_bal_cr = 0.00;
                                   }
                                }

                                $capital_opening_for_next_year->opening_bal_dr = $capital_closing_for_current_year->closing_bal_dr;
                                $capital_opening_for_next_year->opening_bal_cr = $capital_closing_for_current_year->closing_bal_cr;
        
                            $min_date_for_next = $this->get_min_max_date($update_fin_years[$i + 1])->min_date;
                            $capital_opening_for_next_year->opening_bal_date = $min_date_for_next;
        
                            $capital_closing_for_current_year->save();
                            $capital_opening_for_next_year->save();
                        }
                    }
                    else{
                        $closing_for_last_year = Balances::where('account_id', $capital_account_id)
                            ->where('fin_year', $update_fin_years[$i])
                            ->first();

                        if($closing_for_last_year){
                            if($closing_for_last_year->closing_bal_dr>0){
                                 $closing_factor = -($closing_for_last_year->closing_bal_dr) + $sum_of_pl;
                            }
                            else{
                                $closing_factor = $closing_for_last_year->closing_bal_cr + $sum_of_pl;
                            }

                            if($closing_factor>=0){
                             $closing_for_last_year->closing_bal_cr = abs($closing_factor);
                             $closing_for_last_year->closing_bal_dr = 0.00;
                            }
                            else{
                             $closing_for_last_year->closing_bal_dr = abs($closing_factor);
                             $closing_for_last_year->closing_bal_cr = 0.00;
                            }

                            $max_date_for_current = $this->get_min_max_date($update_fin_years[$i])->max_date;
                            $closing_for_last_year->closing_bal_date = $max_date_for_current;
        
                            $closing_for_last_year->save();
                        }
                        
                        
                    }


            }
        }


            // TRANSACTIONS COMMITTED
            DB::commit();
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }


    protected function get_profit_and_loss_amount($request){
        $data = $this->get_profit_and_loss($request)->original;


        $expense_side = $data['expense_side'];
        $income_side = $data['income_side'];

        $expense_total = 0;
        $income_total = 0;

        foreach ($expense_side as $expense) {
            $expense_total += $expense['group_total_dr'] - $expense['group_total_cr'];
        }

        foreach ($income_side as $income) {
            $income_total += $income['group_total_cr'] - $income['group_total_dr'];
        }

        $profit_and_loss = $income_total - $expense_total;



        return ['expense_total' => $expense_total, 'income_total' => $income_total, 'profit_and_loss' => $profit_and_loss];
    }

    protected function get_truck_data($truck_id, $truck_no, $min_date, $from_date, $to_date, $currentYear, $branch_id) {
        $truck_data = [
            'vehicle_id' => $truck_id,
            'vehicle_number' => $truck_no,
            'dr_amount' => 0,
            'cr_amount' => 0,
        ];

        $payment_voucher_sum = VoucherDesc::where('against', 'VEHICLE_COST_CENTER')
                            ->where('against_id', $truck_id)
                            ->whereIn('voucher_name', ['Payment Voucher', 'Journal'])
                            ->where('fin_year', $currentYear)
                            ->whereHas('voucher', function ($query) use ($from_date, $to_date,$branch_id) {
                                $query->where('voucher_date', '>=', $from_date)
                                    ->where('voucher_date', '<=', $to_date)
                                    ->when($branch_id, function ($query, $branch_id) {
                                        return $query->where('branch_id', $branch_id);
                                    });
                            })
                            ->sum('receipt_amt');

       
        $diesel_receipt_sum = DieselReceipt::where('vehicle_id', $truck_id)
                            ->where('fin_year', $currentYear)
                            ->where('booking_date', '>=', $from_date)
                            ->where('booking_date', '<=', $to_date)
                            ->when($branch_id, function ($query, $branch_id) {
                                return $query->where('branch_id', $branch_id);
                            })
                            ->sum('amount');

        $sum_of_qty_in_purchase = PurchaseDesc::where('vehicle_id',$truck_id)
                                    ->whereHas('purchase', function ($query) use ($from_date, $to_date,$branch_id) {
                                        $query->where('bill_date', '>=', $from_date)
                                            ->where('bill_date', '<=', $to_date)
                                            ->when($branch_id, function ($query, $branch_id) {
                                                return $query->where('branch_id', $branch_id);
                                            });
                                    })
                                   ->where('fin_year',$currentYear)
                                   ->sum('net_amount');

                        
        $pods_sum = Pod::where('vehicle_id', $truck_id)
                            ->where('fin_year', $currentYear)
                            ->where('delivery_date', '>=', $from_date)
                            ->where('delivery_date', '<=', $to_date)
                            ->when($branch_id, function ($query, $branch_id) {
                                return $query->where('branch_id', $branch_id);
                            })
                            ->sum('grand_total');
                        
        $gr_without_pod_sum = BookGr::where('vehicle_id', $truck_id)
                            ->where('fin_year', $currentYear)
                            ->where('gr_date', '>=', $from_date)
                            ->where('gr_date', '<=', $to_date)
                            ->when($branch_id, function ($query, $branch_id) {
                                return $query->where('branch_id', $branch_id);
                            })
                            ->whereDoesntHave('pod_desc')
                            ->sum('grand_total');
                        
        $loading_slip_sum = LoadingSlip::where('vehicle_id', $truck_id)
                            ->where('fin_year', $currentYear)
                            ->where('ls_date', '>=', $from_date)
                            ->where('ls_date', '<=', $to_date)
                            ->when($branch_id, function ($query, $branch_id) {
                                return $query->where('branch_id', $branch_id);
                            })
                            ->sum('grand_total');

        $diesel_allotment_sum = DieselAllotment::whereHas('loading_slip', function ($query) use ($truck_id, $currentYear, $from_date, $to_date, $branch_id) {
                                $query->where('vehicle_id', $truck_id)
                                      ->where('fin_year', $currentYear)
                                      ->where('ls_date', '>=', $from_date)
                                      ->where('ls_date', '<=', $to_date)
                                      ->when($branch_id, function ($query, $branch_id) {
                                          return $query->where('branch_id', $branch_id);
                                      });
                            })->sum('amt');
        
        $debit_minus_credit = ($payment_voucher_sum + $diesel_receipt_sum + $sum_of_qty_in_purchase + $diesel_allotment_sum) - ($pods_sum + $gr_without_pod_sum + $loading_slip_sum);

        $truck_data['dr_amount'] = $debit_minus_credit > 0 ? $debit_minus_credit : 0;
        $truck_data['cr_amount'] = $debit_minus_credit < 0 ? abs($debit_minus_credit) : 0;

        return $truck_data;
    }

    
    protected function get_data_for_obj($pk,$currentYear)
    {
        // Fetch and send the account
        $account = Account::select('id','account_name','account_type','group_id')->with(['group:id,group_name','balances'=> function ($query) use ($currentYear) {
            $query->select('id','account_id','opening_bal_dr','opening_bal_cr','fin_year')->where('fin_year', $currentYear)->first();
        }])->find($pk);

        return ['account' => $account];
    }


    
}
