<?php

namespace App\Http\Controllers\Accounts;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Http\Traits\EditLogTrait;
use App\Http\Traits\PermissionsTrait;
use App\Models\Account\Group;
use Illuminate\Support\Facades\DB;
use Carbon\Carbon;

class GroupController extends Controller
{
    use PermissionsTrait;
    use EditLogTrait;

    public function index(Request $request)
    {
        $test = $this->is_read_permitted($request->user->id, 'Groups');
        if (!$test) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        // Fetch and send all Groups
        $groups = Group::select('id','group_name','group_type','under_grp_id','is_primary','created_by')->with(['under_grp:id,group_name'])->get();
        return response()->json($groups);
    }

    public function show(Request $request, $pk)
    {
        // Fetch and send the Group
        $group = Group::select('id','group_name','group_type','under_grp_id','is_primary','created_by')->with(['under_grp:id,group_name'])->find($pk);
        return response()->json(['group' => $group]); 
    }

    public function store(Request $request)
    {
        $test = $this->is_new_permitted($request->user->id, 'Groups');
        if (!$test) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        $validatedData = $request->validate([
            'group_name' => 'required|unique:groups,group_name',
        ], [
            'group_name.required' => 'Required',
            'group_name.unique' => 'Already Exists!',
        ]);

        try {
            // Transaction begins
            DB::beginTransaction();

            // Save a new Group
            $group = new Group();
            $group->under_grp_id = $request->under_group ? $request->under_group['id'] : null;
            $group->group_name = $request->group_name;
            $group->group_type = $request->group_type;

            if($request->group_type=="Asset" || $request->group_type=="Expense")
            {
                $group->nature = "Dr";
            }
            else if($request->group_type=="Liability" || $request->group_type=="Income")
            {
                $group->nature = "Cr";
            }

            $group->is_primary = $request->under_group ? 0 : 1;
            
            $group->created_by = $request->user->id;
            $group->save();

            // Save the edit log
            $newVal = $this->get_data_for_obj($group->id);
            $this->createEditLog('created', 'groups', $group->id, $group->group_name, null, $newVal, $request->user->id,$request->current_year); // Create edit log


            // Transaction committed
            DB::commit();

            
            return response()->json("Success", 200); // Returned response

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function destroy(Request $request, $pk)
    {
        $test = $this->is_delete_permitted($request->user->id, 'Groups');
        $test_delete = $test->delete;
        $delete_valid_till_hours = $test->delete_valid_till;

        if (!$test_delete) {
            return response()->json(['Permission Denied For The Action'], 401);
        }
        
        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();
            
            $group = Group::find($pk);

            if(!$group->created_by){
                return response()->json(['Pre Set Groups Cannot Be Removed'], 401);
            }

            if ($delete_valid_till_hours != null) {
                $validity = Carbon::parse($group->created_at)->addHours($delete_valid_till_hours);
                if (!$validity->isFuture()) {
                    return response()->json(['Permission Timed Out'], 500);
                }
            }

            // Fetch the old value for log
            $oldValue = $this->get_data_for_obj($pk);

            // Find and delete the Group
            $group->delete();

            $this->createEditLog('deleted', 'groups', $group->id, $group->group_name, $oldValue, null, $request->user->id,$request->current_year); // Create edit log

            // TRANSACTIONS COMMITTED
            DB::commit();
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function autocomplete(Request $request)
    {
        // Perform search and return matching Groups
        // WHILE SEARCHING THIS WILL RUN
        $query = Group::select('id', 'group_name');

        // Apply any additional filters based on other columns as needed
        if ($request->has('filterby')) {
            $orWhereFilters = isset($request->filterby['orWhere']) ? $request->filterby['orWhere'] : [];
            $whereFilters = isset($request->filterby['where']) ? $request->filterby['where'] : [];

           // Loop through the whereFilters array
           foreach ($whereFilters as $whereFilter) {
               // Apply the filter to the query
               $query->where($whereFilter['key'], $whereFilter['value']);
           }

           // Loop through the orWhereFilters array
           foreach ($orWhereFilters as $orWhereFilter) {
               // Apply the filter to the query
               $query->orWhere($orWhereFilter['key'], $orWhereFilter['value']);
           }
        
        }
        
        // Check if searchVariable is provided and apply the filter
        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('group_name', 'like', '%'. $searchVariable . '%');
        }

        // Limit the number of results
        $query->take($request->limit);

        // Retrieve the filtered 
        $groups = $query->get();
        return response(['groups' => $groups, 'draw' => $request->draw]);
    }

    public function datatable(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'Groups');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $query = Group::select('id','group_name','group_type','under_grp_id','is_primary','created_by')->with(['under_grp:id,group_name'])->orderBy($column, $dir);

        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('group_name', 'like', $searchValue . '%');
            });
        }

        $data = $query->paginate($length);

        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    public function check_unique(Request $request)
    {
        $key = $request->key;
        $value = $request->value;

        if (!$value) {
            return response()->json(['exists' => false]);
        }

        $exists = Group::where($key, $value)->exists();
        if ($exists) {
            return response()->json(['errors' => [$key  => ['Already Exists !']]], 422);
        }

        return response()->json(['exists' => false]);
    }



    public function get_primary_groups(Request $request)
    {
        $groups = Group::select('id','group_name','group_type')->where('is_primary',1)->get();
        return response()->json(['groups' => $groups]);
    }



    protected function get_data_for_obj($pk)
    {
        // Fetch and send the Group
        $group = Group::select('id','group_name','group_type','under_grp_id','is_primary','created_by')->with(['under_grp:id,group_name'])->find($pk);
        return ['group' => $group];
    }
}
