<?php

namespace App\Http\Controllers\Auth;

use App\Http\Controllers\Controller;
use App\Models\Auth\LoginHistory;
use App\Models\Business\Company;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Tymon\JWTAuth\Facades\JWTAuth;

class AuthController extends Controller
{
    public function username()
    {
        return 'username'; // Change this to 'email' if you want to switch back to email authentication
    }

    public function user(Request $request){

        $user = User::find($request->user->id);

        // Get all branches associated with the user
        $branches = $user->branches()->select('branches.id','branch_name','branch_code')->get();

        $permissions = $user->permissions->groupBy('module');

        return response()->json( ['permissions'=>$permissions, 'user_branches'=>$branches] );
    }

    public function login(Request $request){

        $validatedData = $request->validate([
            'username' => 'required|string',
            'password' => 'required|min:5',
            'company' => 'required|min:3',
            'current_year' => 'required'
        ], [
            'username.required' => 'Required',
            'password.required' => 'No password',
            'password.min' => 'Incorrect Credentials', // We return incorrect credentials, because we can't tell anyone the length while login.
            'company.required' => 'No Company Code Provided!',
            'current_year' => 'Please select a financial year.'
        ]);


        $this->connectToTheDB($request->company);

        $credentials = $request->only('username', 'password');

        if (!$token = auth()->attempt($credentials) ) {
            return response()->json(['error' => 'Unauthorized User'], 500);
        }
        

        $user = auth()->user()->select('id', 'username', 'email','power','party_id')->where('id', auth()->id())->first();
        $branch_ids = $user->branches()->pluck('branches.id')->toArray();

        $company_details = Company::select('id','company_code','company_name','email','ftl','part','parchoon','packer','year_end')->where('company_code', strtolower($request->company) )->first();
        
        if(!$company_details){
            return response()->json(['message'=>'Company Details Not Found'], 500);
        }

        $customClaims = [
            'user' => $user,
            'sub' => $user->username,
            'company' => $company_details,
            'current_year' => $request->current_year,
            'user_branch_ids' => $branch_ids
        ];
    
        $access = JWTAuth::claims($customClaims)->fromUser($user);

        $branches = $user->branches()->select('branches.id', 'branch_name', 'branch_code')->get();
        $permissions = $user->permissions->groupBy('module');

        LoginHistory::create([
            'user_id' => $user->id,
            'username' => $user->username,
            'ip_address' => $request->ip(),
        ]);

        return response()->json(['token'=>$this->respondWithToken($access), 'permissions'=>$permissions, 'user_branches'=>$branches]);
    }

    public function logout(){
       auth()->logout();
       return response('Logged Out Successfully');
    }

    public function refresh(){
        return $this->respondWithToken( auth()->refresh() );
    }

    protected function respondWithToken($token){
        return response()->json(
            [
                'access_token'=>$token,
                'type' => 'Bearer'
            ]
            );
    }

    protected function connectToTheDB($company){
        try {
            // Update the 'database' config in the 'dynamic' connection to point to the current company's database
            config(['database.connections.dynamic.database' => config('app.SERVER_PREFIX') .strtolower($company)]);
        
            // Reconnect to the updated database connection
            DB::connection('dynamic')->reconnect();
        
            DB::setDefaultConnection('dynamic');

        } catch (\Exception $e) {
            // Handle the exception here, for example:
            return response()->json(['message' => 'Failed to update dynamic database connection'], 500);
        }
        
    }
}
