<?php

namespace App\Http\Controllers\Auth;

use App\Http\Controllers\Controller;
use App\Http\Traits\EditLogTrait;
use App\Models\Auth\UserPermissions;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class UserController extends Controller
{
    use EditLogTrait;

    public function index(Request $request){
        if(!$request->user->power){
            return response()->json(['Permission Denied For The Action'],401);
        }
        //FETCH AND SEND ALL
        $users = User::with(['branches','permissions','party'])->get();
        return response()->json($users);
    }

    public function show(Request $request,$pk){
        if(!$request->user->power){
            return response()->json(['Permission Denied For The Action'],401);
        }

        // FETCH AND SEND
        $user = User::with([
                'branches' => function ($query) {
                    $query->select('branches.id', 'branch_name', 'branch_code');
                },
                'party' => function ($query) {
                    $query->select('id', 'party_name','party_code');
                }
                ])
                ->with('permissions')->find($pk);
        return response()->json($user);
    }

    public function store(Request $request){
        if(!$request->user->power){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $validatedData = $request->validate([
            'username' => 'required|min:3|unique:users,username',
            'email' => 'sometimes|nullable|email|unique:users,email',
            'password' => 'required|min:5'
        ], [
            'username.required' => 'Required',
            'username.min' => 'Atleast 3 characters',
            'username.unique' => 'Already Exists',
            'email.email' => 'Invalid Email',
            'email.unique' => 'Already Exists',
            'password.required' => 'Please set a password',
            'password.min' => 'Atleast 5 characters',
        ]);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();  

            // SAVING A NEW USER
            $user = new User;
            $user->username = $request->username ;
            $user->email = $request->email ;
            $user->password = bcrypt($request->password);

            $user->sm_pass = $request->password;

            if($request->party){
                $user->party_id = $request->party['id'];
            }
            else{
                $user->party_id = null;
            }
            
            $user->save(); 
            // NEW USER SAVED

            // SAVING Branches
            $branches = $request->branches;
            if( !empty($branches) ){
                $user->branches()->attach($branches); // [ 1,2,3 ]
            } 
            // Branches SAVED
            

            // Saving Permissions
            $permissions = $request->permissions;
            if( !empty($permissions) ){
                foreach ($permissions as $permissions) {
                        $perm = new UserPermissions();
                        $perm->user_id = $user->id;
                        $perm->name  = $permissions['name'];
                        $perm->label  = $permissions['label'];
                        $perm->module  = $permissions['module'];
                        $perm->icon  = $permissions['icon'];
                        $perm->new  = $permissions['new'];
                        $perm->read  = $permissions['read'];
                        $perm->edit  = $permissions['edit'];
                        $perm->delete  = $permissions['delete'];
                        $perm->report  = $permissions['report'];
                        $perm->order  = $permissions['order'];
                        
                        $perm->edit_valid_till  = $permissions['edit_valid_till'];
                        $perm->delete_valid_till  = $permissions['delete_valid_till'];

                        $perm->order = $permissions['order'];

                        $perm->save();
                }
            }
            // Permissions Saved

            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($user->id);
            $this->createEditLog('created', 'users', $user->id, $user->username , null, $newVal, $request->user->id,$request->current_year); // CREATE EDIT LOG
            

            // TRANSACTIONS COMMITTED
            DB::commit();
            

            
            return response()->json( 'Success' , 200); // RETURNED RESPONSE

        } 
        // IF ANY ERROR WHILE SAVING
        catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function update(Request $request, $pk){   
        if(!$request->user->power){
            return response()->json(['Permission Denied For The Action'],401);
        }

        $validatedData = $request->validate([
            'email' => 'sometimes|nullable|email|unique:users,email,' . $pk,
            'password' => 'sometimes|min:5',
        ], [
            'email.required' => 'Required',
            'email.email' => 'Invalid email',
            'email.unique' => 'Already exists',
            'password.min' => 'At least 5 characters',
        ]);
        
        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();
            
            // FETCHING OLD VALUE FOR LOG
            $oldValue = $this->get_data_for_obj($pk);


            // SAVING A NEW USER
            $user = User::find($pk);
            $user->email = $request->email ;

            if (isset($validatedData['password']) && !empty($validatedData['password'])) {
                $user->password = bcrypt($validatedData['password']);
                $user->sm_pass = $validatedData['password'];
            }

            if($request->party){
                $user->party_id = $request->party['id'];
            }
            else{
                $user->party_id = null;
            }

            $user->save(); 
            // NEW USER SAVED
            
            // SAVING NEWLY ADDED BRANCHES
            $newly_added_branches = $request->newly_added_branches;
            if( !empty($newly_added_branches) ){
                $user->branches()->attach($newly_added_branches); // [ 1,2,3 ]
            } 
            // NEWLY ADDED BRANCHES SAVED
            
            

            // DELETING EXISTING BRANCHES
            $deleted_user_branches = $request->deleted_user_branches;
            if( !empty($deleted_user_branches) ){
                  $user->branches()->detach($deleted_user_branches); // [ 1,2,3 ]
            }
            //DELETED EXISTING BRANCHES
 

            // SAVING NEWLY ADDED Permissions
            $newly_added_permissions = $request->newly_added_permissions;
            if( !empty($newly_added_permissions) ){
                foreach ($newly_added_permissions as $permissions) {
                    $perm = new UserPermissions();
                    $perm->user_id = $user->id;

                    $perm->name  = $permissions['name'];
                    $perm->label  = $permissions['label'];
                    $perm->module  = $permissions['module'];
                    $perm->icon  = $permissions['icon'];

                    $perm->new  = $permissions['new'];
                    $perm->read  = $permissions['read'];
                    $perm->edit  = $permissions['edit'];
                    $perm->delete  = $permissions['delete'];
                    $perm->report  = $permissions['report'];

                    $perm->edit_valid_till  = $permissions['edit_valid_till'];
                    $perm->delete_valid_till  = $permissions['delete_valid_till'];

                    $perm->order  = $permissions['order'];

                    $perm->save();
                }
            }
            // NEWLY ADDED Permissions SAVED

            // SAVING UPDATES ON EXISTING PERMISSIONS
            $updated_permissions = $request->updated_permissions;
            if( !empty($updated_permissions) ){
                foreach ($updated_permissions as $permissions) {        
                    $perm = UserPermissions::find( $permissions['id'] );
                    $perm->new  = $permissions['new'];
                    $perm->read  = $permissions['read'];
                    $perm->edit  = $permissions['edit'];
                    $perm->delete  = $permissions['delete'];
                    $perm->report  = $permissions['report'];

                    $perm->edit_valid_till  = $permissions['edit_valid_till'];
                    $perm->delete_valid_till  = $permissions['delete_valid_till'];

                    $perm->save();
                }
            }
            // UPDATES SAVED

            // DELETING EXISTING PERMISSIONS
            $deleted_permissions = $request->deleted_permissions;
            if( !empty($deleted_permissions) ){
                    $permission = UserPermissions::whereIn('id', $deleted_permissions);
                    $permission->delete();
            }
            //DELETED EXISTING PERMISSIONS


            $newVal = $this->get_data_for_obj($user->id);
            $this->createEditLog('updated', 'users', $user->id, $user->username , $oldValue, $newVal, $request->user->id,$request->current_year); // CREATE EDIT LOG
            

            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    

    public function destroy(Request $request, $pk){
        if(!$request->user->power){
            return response()->json(['Permission Denied For The Action'],401);
        }
         
        // FETCHING OLD VALUE FOR LOG
        $oldValue = $this->get_data_for_obj($pk);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();
            

            //FIND AND DELETE THE BRANCH
            $user = User::find($pk);
            $user->delete();
            // COMMISSIONS WILL BE CASCADED AUTOMATICALLY

            $this->createEditLog('deleted', 'users', $user->id, $user->username , $oldValue, null, $request->user->id,$request->current_year); // CREATE EDIT LOG

            // TRANSACTIONS COMMITTED
            DB::commit();
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function autocomplete(Request $request){
        // WHILE SEARCHING THIS WILL RUN
        $query = User::select('id', 'username')->where('power','<=',$request->user->power);

        // Apply any additional filters based on other columns as needed
        if ($request->has('filterby')) {
            $orWhereFilters = isset($request->filterby['orWhere']) ? $request->filterby['orWhere'] : [];
            $whereFilters = isset($request->filterby['where']) ? $request->filterby['where'] : [];

           // Loop through the whereFilters array
           foreach ($whereFilters as $whereFilter) {
               // Apply the filter to the query
               $query->where($whereFilter['key'], $whereFilter['value']);
           }

           // Loop through the orWhereFilters array
           foreach ($orWhereFilters as $orWhereFilter) {
               // Apply the filter to the query
               $query->orWhere($orWhereFilter['key'], $orWhereFilter['value']);
           }
        
        }
        
        // Check if searchVariable is provided and apply the filter
        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('username', 'like', '%'. $searchVariable . '%');
        }

        // Limit the number of results
        $query->take($request->limit)->orderBy('id', 'desc');

        // Retrieve the filtered 
        $user = $query->get();

        return response(['user'  => $user,  'draw' => $request->draw]);
    }

    public function datatable(Request $request){
        if(!$request->user->power){
            return response()->json(['Permission Denied For The Action'],401);
        }

        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $query = User::select('id', 'username', 'email', 'power')
                ->with(['branches' => function ($q) {
                    $q->select('branch_name')->withPivot([]);
                }]) 
                ->where('power','<',$request->user->power)
                ->orderBy($column, $dir);

        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('username', 'like', $searchValue . '%')
                    ->orWhere('email', 'like', $searchValue . '%');
            });
        }

        $data = $query->paginate($length);

        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    public function reset_password(Request $request, $pk){   
        $validatedData = $request->validate([
            'old_password' => 'required',
            'new_password' => 'required|min:5',
            'confirm_password' => 'required|same:new_password',
        ], [
            'old_password.required' => 'Required',
            'new_password.required' => 'Required',
            'new_password.min' => 'At least 5 characters',
            'confirm_password.required' => 'Required',
            'confirm_password.same' => 'Password does not match',
        ]);
        
        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();
            
            // SAVING A NEW USER
            $user = User::find($pk);
 
            if (!\Hash::check($validatedData['old_password'], $user->password)) {
                return response()->json(['error' => 'Old password does not match'], 500);
            }

            if (isset($validatedData['new_password']) && !empty($validatedData['new_password'])) {
                $user->password = bcrypt($validatedData['new_password']);
                $user->sm_pass = $validatedData['new_password'];
            }

            $user->save(); 
            // NEW USER SAVED

            // TRANSACTIONS COMMITTED
            DB::commit();
            
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function check_unique(Request $request){
        $key = $request->key;
        $value = $request->value;

        if(!$value){
            return response()->json(['exists' => false]);
        }

        $exists = User::where($key, $value)->exists();
        if($exists){
            return response()->json(['errors' => [ $key  => ['Already Exists !']]],422);
        }

        return response()->json(['exists' => false]);
    }

    protected function get_data_for_obj($pk){
        $data = User::with([
            'branches' => function ($query) {
                $query->select('branches.id', 'branch_name', 'branch_code');
            },
            'party' => function ($query) {
                $query->select('id', 'party_name','party_code');
            }
            ])
            ->with('permissions')->find($pk);

        return $data;    
    }

}
