<?php

namespace App\Http\Controllers\Bill;

use App\Http\Controllers\Controller;
use App\Http\Traits\CommonTrait;
use App\Http\Traits\EditLogTrait;
use App\Http\Traits\PermissionsTrait;
use App\Models\Bill\Bill;
use App\Models\Bill\CoverLetter;
use App\Models\Bill\CoverLetterDesc;
use App\Models\Bill\CreditNote;
use App\Models\Bill\CreditNoteDesc;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class CoverLetterController extends Controller
{
    use PermissionsTrait;
    use EditLogTrait;
    use CommonTrait;

    public function index(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'CoverLetter');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        //FETCH AND SEND ALL
        $cover_letter = CoverLetter::with([
            'branch:id,branch_name,branch_code',
            'acc_party:id,account_name',
            'cl_desc',
            'cl_desc.bill:id,bill_number,bill_date,grand_total_amount,acc_party_id'
        ])->where('fin_year',$request->current_year)->get();

        $cover_letter->cl_desc->transform(function ($desc) use ($cover_letter) {  
            return [
                'cl_desc_id' => $desc->id,
                'cl_id' => $desc->cl_id,
                'bill_id' => $desc->bill_id,
                'bill_number' => $desc->bill->bill_number,
                'bill_date' => $desc->bill->bill_date,
                'bill_amount' => $desc->bill->grand_total_amount,
                'party_name' => $cover_letter->acc_party->account_name,
            ];
        });

        return response()->json($cover_letter);
    }

    public function show(Request $request, $pk){
        // FETCH AND SEND
        $cover_letter = CoverLetter::with([
            'branch:id,branch_name,branch_code',
            'acc_party:id,account_name',
            'cl_desc',
            'cl_desc.bill:id,bill_number,bill_date,grand_total_amount,acc_party_id'
        ])->find($pk);

        $cover_letter->cl_desc->transform(function ($desc) use ($cover_letter)  {  
            return [
                'cl_desc_id' => $desc->id,
                'cl_id' => $desc->cl_id,
                'bill_id' => $desc->bill_id,
                'bill_number' => $desc->bill->bill_number,
                'bill_date' => $desc->bill->bill_date,
                'bill_amount' => $desc->bill->grand_total_amount,
                'party_name' => $cover_letter->acc_party->account_name,
            ];
        });


        return response()->json(['cover_letter' =>$cover_letter]);
    }

    public function store(Request $request){

        $test = $this->is_new_permitted($request->user->id, 'CoverLetter');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $validatedData = $request->validate([
            'branch' => 'required',
            'cl_number' => 'required|unique:cover_letters,cl_number',
            'acc_party' => 'required',
        ], [
            'branch.required' => 'Required',
            'cl_number.required' => 'Required',
            'acc_party.required' => 'Required',
            'cl_number.unique' => 'Already Exist !',
        ]);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();  
            

            // IN CASE THE COVER TYPE IS 'PRINTED', WE NEED TO CHECK IF THE BOOK IS ACTUALLY ISSUED TO THE BRANCH OR NOT
            $max_number = $this->get_max_number_from_cl_number($request->cl_number);
            
            
            // SAVING A NEW GR
            $cover_letter = new CoverLetter();
            $cover_letter->branch_id = $request->branch['id'];
            $cover_letter->max_number = $max_number;
            $cover_letter->cl_number = $request->cl_number;
            $cover_letter->acc_party_id = $request->acc_party['id'];
            $cover_letter->cl_date = $request->cl_date;
            $cover_letter->from_date = $request->from_date;
            $cover_letter->to_date = $request->to_date;

            $cover_letter->fin_year = $request->current_year;

            $cover_letter->created_by = $request->user->id;

            // Save the model to the database
            $cover_letter->save();

            

            // SAVING PACKAGE DETAILS
            $cover_letter_desc = $request->cl_desc;
            foreach ($cover_letter_desc as $cld) {
                
                $cl_dc = new CoverLetterDesc();

                $cl_dc->cl_id = $cover_letter->id;
                $cl_dc->cl_number = $cover_letter->cl_number;
                $cl_dc->bill_id = $cld['bill_id'];
                $cl_dc->bill_number = $cld['bill_number'];
                
                $cl_dc->fin_year = $request->current_year;
                $cl_dc->save();
            }
            // PACKAGE DETAILS SAVED



             
            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($cover_letter->id);
            $this->createEditLog('created', 'cover_letter', $cover_letter->id, $cover_letter->cl_number,null, $newVal, $request->user->id,$request->current_year); // CREATE EDIT LOG
            
            
            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json(['id'=>$cover_letter->id, 'ds_number'=>$cover_letter->COVER_number], 200); // RETURNED RESPONSE

        } 
        // IF ANY ERROR WHILE SAVING
        catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function update(Request $request, $pk){   
        $test = $this->is_edit_permitted($request->user->id, 'CoverLetter');
        $test_edit = $test->edit;
        $edit_valid_till_hours = $test->edit_valid_till;

        if( !$test_edit ){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $cover_letter = CoverLetter::find($pk);

        // $COVER_check = $this->check_in_COVER('COVER_number','COVER_number', 'cl_id', $pk);
        // if($COVER_check['status']){
        //    $status= $COVER_check['status'] ? $COVER_check : false;
        //    return response()->json(['error'=>'Document in use', 'data'=>$status],500);
        // };


        if( $edit_valid_till_hours!=null ){
            $validity = Carbon::parse($cover_letter->created_at)->addHours($edit_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }

        $validatedData = $request->validate([
            'branch' => 'required',
            'cl_number' => 'required',
            'acc_party' => 'required',
        ], [
            'branch.required' => 'Required',
            'cl_number.required' => 'Required',
            'acc_party.required' => 'Required',
        ]);
          

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();

            

            // FETCHING OLD VALUE FOR LOG
            $oldValue = $this->get_data_for_obj($pk);

            // SAVING A NEW COVER
            $cover_letter->cl_date = $request->cl_date;
            $cover_letter->from_date = $request->from_date;
            $cover_letter->to_date = $request->to_date;

            $cover_letter->updated_by = $request->user->id;

            $cover_letter->save();
            // NEW COVER SAVED
            
            // SAVING PACKAGE DETAILS
            $newly_added_cl_desc = $request->newly_added_cl_desc;
            foreach ($newly_added_cl_desc as $nald) {
                
                $cl_dc = new CoverLetterDesc();

                $cl_dc->cl_id = $cover_letter->id;
                $cl_dc->cl_number = $cover_letter->cl_number;
                $cl_dc->bill_id = $nald['bill_id'];
                $cl_dc->bill_number = $nald['bill_number'];
                
                $cl_dc->fin_year = $request->current_year;
                $cl_dc->save();
            }
            // PACKAGE DETAILS SAVED

            // DELETING EXISTING PACKAGE DETAILS
            $deleted_cl_desc = $request->deleted_cl_desc;
            if( !empty($deleted_cl_desc) ){
                foreach ($deleted_cl_desc as $dcd) {
                    $cld = CoverLetterDesc::find($dcd);
                    $cld->delete();
                }
                    
            }
            //DELETED EXISTING PACKAGE DETAILS


            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($cover_letter->id);
            $this->createEditLog('updated', 'cover_letter', $cover_letter->id, $cover_letter->cl_number,$oldValue, $newVal, $request->user->id,$request->current_year); // CREATE EDIT LOG
            

            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function destroy(Request $request, $pk){
        $test = $this->is_delete_permitted($request->user->id, 'CoverLetter');
        $test_delete = $test->delete;
        $delete_valid_till_hours = $test->delete_valid_till;

        if( !$test_delete ){
            return response()->json(['Permission Denied For The Action'],401);
        }

        // FETCHING OLD VALUE FOR LOG
        $oldValue = $this->get_data_for_obj($pk);
        
        $cover_letter = CoverLetter::find($pk);

        if( $delete_valid_till_hours!=null ){
            $validity = Carbon::parse($cover_letter->created_at)->addHours($delete_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();

            $cover_letter_desc_ids = CoverLetterDesc::select('id')->where('cl_id', $pk)->where('fin_year',$request->current_year)->pluck('id')->toArray();

            $cover_letter_desc = CoverLetterDesc::whereIn('id', $cover_letter_desc_ids)->where('fin_year',$request->current_year)->delete();
    

            //FIND AND DELETE THE BRANCH
            $cover_letter->delete();
            // COMMISSIONS WILL BE CASCADED AUTOMATICALLY



            $this->createEditLog('deleted', 'cover_letter', $cover_letter->id, $cover_letter->cl_number, $oldValue, null, $request->user->id,$request->current_year); // CREATE EDIT LOG

            // TRANSACTIONS COMMITTED
            DB::commit();
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function autocomplete(Request $request){
        // Initialize the query builder
        $query = CoverLetter::select('id', 'cl_number')->where('fin_year',$request->current_year);

        $user_branch_ids = $request->user_branch_ids;
        if ( $user_branch_ids ) {
            $query->whereIn('branch_id', $user_branch_ids );
        }

        if ($request->has('filterby')) {
            $orWhereFilters = isset($request->filterby['orWhere']) ? $request->filterby['orWhere'] : [];
            $whereFilters = isset($request->filterby['where']) ? $request->filterby['where'] : [];

        // Loop through the whereFilters array
        foreach ($whereFilters as $whereFilter) {
            // Apply the filter to the query
            $query->where($whereFilter['key'], $whereFilter['value']);
        }

        // Loop through the orWhereFilters array
        foreach ($orWhereFilters as $orWhereFilter) {
            // Apply the filter to the query
            $query->orWhere($orWhereFilter['key'], $orWhereFilter['value']);
        }
        
        }

        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('cl_number', 'like', '%'. $searchVariable . '%');
        }

        // Limit the number of results
        $query->take($request->limit)->orderBy('id', 'desc');

        // Retrieve the filtered 
        $cover_letter = $query->get();
        return response(['cover_letter' => $cover_letter, 'draw' => $request->draw]);
    }

    public function fetch_against_details(Request $request){
        $acc_party_id = $request->acc_party;
        $from_date = $request->from_date;
        $to_date = $request->to_date;

        $data = [];

        // GET ALL BILLS AGAINST THE PARTY
        // IF IT HAS A credit_note against it, then add a special field 'has_credit_note' => true in my response
        $bills = Bill::select('id','bill_number','bill_date','grand_total_amount','acc_party_id')
                ->where('acc_party_id', $acc_party_id)
                ->where('fin_year',$request->current_year)
                ->whereBetween('bill_date', [$from_date, $to_date])->get();

                
    
        foreach ($bills as $bill) {
            // Check if there is a credit note against the current bill
            $hasCreditNote = CreditNoteDesc::where('bill_id', $bill->id)->where('fin_year',$request->current_year)->exists();
        
            // Add the bill data to the response array
            $data[] = [
                'id' => $bill->id,
                'bill_id' => $bill->id,
                'bill_number' => $bill->bill_number,
                'bill_date' => $bill->bill_date,
                'bill_amount' => $bill->grand_total_amount,
                'party_name' => $request->acc_party['account_name'],
                'has_credit_note' => $hasCreditNote,
                'checked' => $hasCreditNote ? false : true,
            ];
        }

        return response()->json(['bills' => $data]);
    }

    public function datatable(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'CoverLetter');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $query = CoverLetter::with([
                    'branch:id,branch_name,branch_code',
                    'acc_party:id,account_name',
                ])
                ->where('fin_year',$request->current_year)
                ->orderBy($column, $dir);

        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('cl_number', 'like', $searchValue . '%');
            });
        }

        $data = $query->paginate($length);

        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    public function check_unique(Request $request){
        $key = $request->key;
        $value = $request->value;

        if(!$value){
            return response()->json(['exists' => false]);
        }

        $exists = CoverLetter::where($key, $value)->exists();
        if($exists){
            return response()->json(['errors' => [ $key  => ['Already Exists !']]],422);
        }

        return response()->json(['exists' => false]);
    }

    public function get_recent(Request $request) {
        $cover_letterQuery = CoverLetter::select('id', 'cl_number')->where('fin_year',$request->current_year);
        
        if (!empty($request->user_branch_ids)) {
            $cover_letterQuery->whereIn('branch_id', $request->user_branch_ids);
        }
        
        $cover_letter = $cover_letterQuery->orderBy('id', 'desc')->take(10)->get();
        
        return response()->json(['cover_letter' => $cover_letter]);
    }
    


    public function get_next_cl_number(Request $request){
        $branch_id = $request->branch_id;
        $branch_code = $request->branch_code;

        $max = CoverLetter::where('branch_id',$branch_id)->max('max_number');
        $cover_letter_number = $branch_code.'-'.sprintf('%06d', $max+1);
        return response()->json(['cl_number' => strtoupper($cover_letter_number), 'max_number' => $max+1]);
        
    }

    protected function get_data_for_obj($pk){
        // FETCH AND SEND
        $cover_letter = CoverLetter::with([
            'branch:id,branch_name,branch_code',
            'acc_party:id,account_name',
            'cl_desc',
            'cl_desc.bill:id,bill_number,bill_date,grand_total_amount,acc_party_id',
        ])->find($pk);

        
        $cover_letter->cl_desc->transform(function ($desc) use ($cover_letter)  {  
            return [
                'cl_desc_id' => $desc->id,
                'cl_id' => $desc->cl_id,
                'bill_id' => $desc->bill_id,
                'bill_number' => $desc->bill->bill_number,
                'bill_date' => $desc->bill->bill_date,
                'bill_amount' => $desc->bill->grand_total_amount,
                'party_name' => $cover_letter->acc_party->account_name,
            ];
        });

        return ['cover_letter' =>$cover_letter];
    }


    public function get_custom_fields(Request $request){
        $cover_letter = CoverLetter::select($request->custom_fields)->with($request->with_relations)->find($request->lh_id);
        return response()->json(['cover_letter' => $cover_letter]);
    }

    protected function get_max_number_from_cl_number($input){
        // Extract the integer value using regular expression
        $pattern = '/[-\/](\d+)$/';
        preg_match($pattern, $input, $matches);
        
        if(empty($matches)){
            return intval($input);
        }

        $integerValue = intval(end($matches));
        return $integerValue;
    }


}
