<?php

namespace App\Http\Controllers\Booking;

use App\Http\Controllers\Controller;
use App\Http\Traits\CommonTrait;
use App\Http\Traits\EditLogTrait;
use App\Http\Traits\PermissionsTrait;
use App\Models\Account\VoucherDesc;
use App\Models\Booking\BookGr;
use App\Models\Booking\EwayGr;
use App\Models\Booking\GrDesc;
use App\Models\Booking\PackagePosition;
use App\Models\Master\IssueBooks;
use App\Models\Master\Party;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class GrController extends Controller
{
    use PermissionsTrait;
    use EditLogTrait;
    use CommonTrait;

    public function index(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'GR');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        //FETCH AND SEND ALL
        $grs = BookGr::with([
                    'branch:id,branch_name,branch_code','source:id,station_name', 'destination:id,station_name', 'consignor:id,party_name', 'consignee:id,party_name', 'vehicle:id,vehicle_number',
                    'gr_desc',
                    'gr_desc.material' => function ($query) {
                        $query->select('id', 'material_name');
                    },
                    'eway_details',
                    'acc_party' => function ($query) {
                        $query->select('id', 'account_name','account_type');
                    },
                    'acc_party.party:id,account_id',
                    'acc_party_branch:id,branch_name',
                    'consignor_branch:id,branch_name',
                    'consignee_branch:id,branch_name',
                    ])->where('fin_year',$request->current_year)->get();
        return response()->json($grs);
    }

    public function show(Request $request, $pk){
        // FETCH AND SEND
        $gr = BookGr::with([
                    'branch:id,branch_name,branch_code','source:id,station_name', 'destination:id,station_name', 'consignor:id,party_name', 'consignee:id,party_name', 'vehicle:id,vehicle_number','driver:id,emp_name',
                    'gr_desc',
                    'gr_desc.material' => function ($query) {
                        $query->select('id', 'material_name');
                    },
                    'eway_details',
                    'acc_party' => function ($query) {
                        $query->select('id', 'account_name','account_type');
                    },
                    'acc_party.party:id,account_id',
                    'acc_party_branch:id,branch_name',
                    
                    'consignor_branch:id,branch_name',
                    'consignee_branch:id,branch_name',
                    ])->find($pk);

        return response()->json(['gr'=>$gr]);
    }

    public function store(Request $request){

        $test = $this->is_new_permitted($request->user->id, 'GR');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $validatedData = $request->validate([
            'branch' => 'required',
            'gr_type' => 'required',
            'gr_number' => 'required|unique:book_grs,gr_number',
            'source' => 'required',
            'destination' => 'required',
            'acc_party' => 'required'
        ], [
            'branch.required' => 'Required',
            'gr_type.required' => 'Required',
            'gr_number.required' => 'Required',
            'source.required' => 'Required',
            'destination.required' => 'Required',
            'gr_number.unique' => 'Already Exist !',
            'acc_party.required' => 'required'
        ]);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();  
            

            // IN CASE THE GR TYPE IS 'PRINTED', WE NEED TO CHECK IF THE BOOK IS ACTUALLY ISSUED TO THE BRANCH OR NOT
            $max_number = $this->get_max_number_from_gr_number($request->gr_number);
            if($request->gr_type == 'PRINTED'){
                $check = IssueBooks::select('prefix','party_id')->where('prefix',$request->prefix)->where('branch_id', $request->branch['id'])->where('book_type','GR')->where('issue_from', '<=', $max_number)->where('issue_to', '>=', $max_number)->first();
                if(!$check){
                    return response()->json(['message'=>'Book Not Issued To The Branch'], 422);
                }
                if($check->party_id){
                    if($check->party_id != $request->consignor['id']){
                        return response()->json(['message'=>'Book Not Issued To The Consignor'], 422);
                    }
                }
                
            }
            
            
            // SAVING A NEW GR
            $gr = new BookGr();
            $gr->branch_id = $request->branch['id'];
            $gr->gr_type = $request->gr_type;
            $gr->prefix = $request->prefix;
            $gr->max_number = $max_number;
            $gr->gr_number = $request->gr_number;
            $gr->gr_date = $request->gr_date;
            $gr->gr_time = $request->gr_time;
            $source_id = $this->add_if_not_in_master('Station', $request->source, $request->current_year);
            $gr->source_id = $source_id;
            $gr->source_name = $request->source['station_name'];

            $destination_id = $this->add_if_not_in_master('Station', $request->destination, $request->current_year);
            $gr->destination_id = $destination_id;
            $gr->destination_name = $request->destination['station_name'];
            $gr->tpt_mode = $request->tpt_mode;
            $gr->vehicle_id = $request->vehicle ? $request->vehicle['id'] : null;
            $gr->vehicle_number = $request->vehicle ? $request->vehicle['vehicle_number'] : null;
            $gr->vehicle_type = $request->vehicle_type;
            $gr->driver_id = $request->driver ? $request->driver['id'] : null;
            $gr->edd = $request->edd;
            $gr->pay_mode = $request->pay_mode;
               
            if($request->pay_mode == 'TO PAY'){
                $gr->income_account_id = 6;  // TO PAY
            }elseif($request->pay_mode == 'PAID'){
                $gr->income_account_id = 7;  // PAID
            }


            $gr->delivery_pt = $request->delivery_pt;

            $gr->consignor_id = $request->consignor ? $request->consignor['id'] : null;
            $gr->consignor_name = $request->consignor ? $request->consignor['party_name'] : null;
            $gr->consignor_branch_id = $request->consignor_branch ? $request->consignor_branch['id'] : null;
            $gr->consignor_gst_number = $request->consignor_gst_number;
            $gr->consignor_phone_number = $request->consignor_phone_number;
            $gr->consignor_address = $request->consignor_address;

            $gr->consignee_id = $request->consignee ? $request->consignee['id'] : null;
            $gr->consignee_name = $request->consignee ? $request->consignee['party_name'] : null;
            $gr->consignee_branch_id = $request->consignee_branch ? $request->consignee_branch['id'] : null;
            $gr->consignee_gst_number = $request->consignee_gst_number;
            $gr->consignee_phone_number = $request->consignee_phone_number;
            $gr->consignee_address = $request->consignee_address;

            $gr->acc_party_id = $request->acc_party ? $request->acc_party['id'] : null; 
            $gr->acc_party_branch_id = $request->acc_party_branch ? $request->acc_party_branch['id'] : null;

            $gr->total_pkg = $request->total_pkg;
            $gr->total_material_weight = $request->total_material_weight;
            $gr->total_km = $request->total_km;
            $gr->total_chg_qty = $request->total_chg_qty;
            $gr->total_desc_value = $request->total_desc_value;
            $gr->total_invoice_value = $request->total_invoice_value;

            $gr->p_marka = $request->p_marka;
            $gr->remarks = $request->remarks;

            $gr->booking_comm_percent = $request->booking_comm_percent;
            $gr->booking_comm_rate = $request->booking_comm_rate;
            $gr->booking_comm_charged_on = $request->booking_comm_charged_on;
            $gr->booking_comm_amount = $request->booking_comm_amount;

            $gr->eway_pic = $request->eway_pic;
            $gr->other_pic = $request->other_pic;

            $gr->print_at_save = $request->print_at_save;

            $gr->freight = $request->freight;
            $gr->gr_chg = $request->gr_chg;
            $gr->payment_dynamic_fields = $request->payment_dynamic_fields;
            $gr->gst_paid_by = $request->gst_paid_by;
            $gr->total_without_gst = $request->total_without_gst;
            $gr->igst_percent = $request->igst_percent;
            $gr->cgst_percent = $request->cgst_percent;
            $gr->sgst_percent = $request->sgst_percent;
            $gr->igst_amount = $request->igst_amount;
            $gr->cgst_amount = $request->cgst_amount;
            $gr->sgst_amount = $request->sgst_amount;
            $gr->total_without_rounding = $request->total_without_rounding;
            $gr->rounded_off = $request->rounded_off;
            $gr->grand_total = $request->grand_total;

            $gr->freight_bill = false;
            $gr->way_exp_bill = false;

            $gr->fin_year = $request->current_year; 

            $gr->created_by = $request->user->id;

            // Save the model to the database
            $gr->save();

            // SAVING PACKAGE DETAILS
            $pkg_details = $request->pkg_details;
            foreach ($pkg_details as $pd) {
                if(!$pd['pkg'] || !$pd['material'] || !$pd['chg_unit'] || !$pd['chg_qty'])
                continue;
                
                $pkg_det = new GrDesc();
                $pkg_det->gr_id = $gr->id;
                $pkg_det->gr_number = $gr->gr_number;
                $material_id = $this->add_if_not_in_master('Material', $pd['material'], $request->current_year);

                $pkg_det->material_id = $material_id;
                $pkg_det->destination_id = $request->destination['id'];
                $pkg_det->packing_method = $pd['packing_method'];
                $pkg_det->pkg = $pd['pkg'];
                $pkg_det->nos = $pd['nos'];
                $pkg_det->material_weight = $pd['material_weight'];
                $pkg_det->km = $pd['km'];
                $pkg_det->chg_unit = $pd['chg_unit'];
                $pkg_det->chg_qty = $pd['chg_qty'];
                $pkg_det->rate = $pd['rate'];
                $pkg_det->total = $pd['total'];
                $pkg_det->invoice_number = $pd['invoice_number'];
                $pkg_det->invoice_date = $pd['invoice_date'];
                $pkg_det->invoice_value = $pd['invoice_value'];
                $pkg_det->payment_dynamic_fields = $pd['payment_dynamic_fields'];
                $pkg_det->fin_year = $request->current_year;

                $pkg_det->free_pkgs = $pd['pkg'];
                $pkg_det->save();


                // SAVING PACKAGE POSITIONS
                $pkg_position = new PackagePosition();
                $pkg_position->gr_id = $gr->id;
                $pkg_position->gr_number = $gr->gr_number;
                $pkg_position->gr_desc_id = $pkg_det->id;
                $pkg_position->position = $request->branch['id'];
                $pkg_position->pkg = $pd['pkg'];
                $pkg_position->origin = $request->branch['id'];
                $pkg_position->save();
                // PACKAGE POSITIONS SAVED
            }
            // PACKAGE DETAILS SAVED

            // SAVING EWAY DETAILS
            $eway_details = $request->eway_details;
            foreach ($eway_details as $ed) {
                if(!$ed['eway_number'])
                continue;

                $check_unique = EwayGr::where('eway_number',$ed['eway_number'])->exists();
                if($check_unique){
                    return response()->json(['error'=>'E-Way in use - '.$ed['eway_number'], 'data'=>$ed['eway_number']],500);
                }
                
                $pkg_det = new EwayGr();
                $pkg_det->gr_id = $gr->id;
                $pkg_det->gr_number = $gr->gr_number;
                $pkg_det->eway_number = $ed['eway_number'];
                $pkg_det->eway_exp_date = $ed['eway_exp_date'];
                $pkg_det->fin_year = $request->current_year;
                $pkg_det->save();
            }
            // EWAY DETAILS SAVED


            // RECORD THE TRANSACTION IN ACCOUNTS
            if($request->pay_mode == 'TO PAY' || $request->pay_mode == 'PAID')
            $this->record_transaction_in_accounts($request, $gr);
            // TRANSACTION RECORDED IN ACCOUNTS

            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($gr->id);
            $this->createEditLog('created', 'book_grs', $gr->id, $gr->gr_number,null, $newVal, $request->user->id,$request->current_year); // CREATE EDIT LOG
            
        
            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json(['id'=>$gr->id, 'gr_number'=>$gr->gr_number], 200); // RETURNED RESPONSE

        } 
        // IF ANY ERROR WHILE SAVING
        catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function update(Request $request, $pk){   
        $test = $this->is_edit_permitted($request->user->id, 'GR');
        $test_edit = $test->edit;
        $edit_valid_till_hours = $test->edit_valid_till;

        if( !$test_edit ){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $gr = BookGr::find($pk);

        if( $edit_valid_till_hours!=null ){
            $validity = Carbon::parse($gr->created_at)->addHours($edit_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }

        $validatedData = $request->validate([
            'branch' => 'required',
            'gr_type' => 'required',
            'gr_number' => 'required',
            'source' => 'required',
            'destination' => 'required',
            'acc_party' => 'required'
        ], [
            'branch.required' => 'Required',
            'gr_type.required' => 'Required',
            'gr_number.required' => 'Required',
            'source.required' => 'Required',
            'destination.required' => 'Required',
            'acc_party.required' => 'required'
        ]);
          

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();

            // FETCHING OLD VALUE FOR LOG
            $oldValue = $this->get_data_for_obj($pk);

            $pod_check = $this->check_in_pod_desc('pod_id','pod_number', 'gr_id', $pk);
            $bill_check = $this->check_in_bill_desc('bill_id','bill_number', 'gr_id', $pk);
            $challan_check  = $this->check_in_challan_desc('challan_number','challan_number','gr_id',$pk);
            $voucher_check  = $this->check_in_voucher_desc('voucher_number','voucher_number','GR',$gr->gr_number);
            $lorry_hire_check = $this->check_in_lorry_hire_desc('lh_number','lh_number','GR',$gr->gr_number);
            $diesel_receipt_check = $this->check_in_diesel_receipt('dr_number','dr_number','GR',$gr->gr_number);
            $driver_settlement_desc_check = $this->check_in_driver_stl_desc('ds_number','ds_number','GR',$gr->gr_number);
                
            if($pod_check['status'] || $bill_check['status'] || $challan_check['status'] || $voucher_check['status'] || $lorry_hire_check['status'] || $diesel_receipt_check['status'] || $driver_settlement_desc_check['status'] ){
               $status = $bill_check['status'] ? $bill_check : ($pod_check['status'] ? $pod_check : ($challan_check['status'] ? $challan_check : ($voucher_check['status'] ? $voucher_check : ($lorry_hire_check['status'] ? $lorry_hire_check : ($diesel_receipt_check['status'] ? $diesel_receipt_check : $driver_settlement_desc_check) ))));
               return response()->json(['error'=>'Document in use', 'data'=>$status],500);
            };


            // SAVING A NEW GR
            $gr->gr_date = $request->gr_date;
            $gr->gr_time = $request->gr_time;
            $source_id = $this->add_if_not_in_master('Station', $request->source, $request->current_year);
            $gr->source_id = $source_id;
            $gr->source_name = $request->source['station_name'];

            $destination_id = $this->add_if_not_in_master('Station', $request->destination, $request->current_year);
            $gr->destination_id = $destination_id;
            $gr->destination_name = $request->destination['station_name'];
            $gr->tpt_mode = $request->tpt_mode;
            $gr->vehicle_id = $request->vehicle ? $request->vehicle['id'] : null;
            $gr->vehicle_number = $request->vehicle ? $request->vehicle['vehicle_number'] : null;
            $gr->vehicle_type = $request->vehicle_type;
            $gr->driver_id = $request->driver ? $request->driver['id'] : null;
            $gr->edd = $request->edd;
            $gr->pay_mode = $request->pay_mode;

            if($request->pay_mode == 'TO PAY'){
                $gr->income_account_id = 6;  // TO PAY
            }elseif($request->pay_mode == 'PAID'){
                $gr->income_account_id = 7;  // PAID
            }

            $gr->delivery_pt = $request->delivery_pt;

            $gr->consignor_id = $request->consignor ? $request->consignor['id'] : null;
            $gr->consignor_name = $request->consignor ? $request->consignor['party_name'] : null;
            $gr->consignor_branch_id = $request->consignor_branch ? $request->consignor_branch['id'] : null;
            $gr->consignor_gst_number = $request->consignor_gst_number;
            $gr->consignor_phone_number = $request->consignor_phone_number;
            $gr->consignor_address = $request->consignor_address;

            $gr->consignee_id = $request->consignee ? $request->consignee['id'] : null;
            $gr->consignee_name = $request->consignee ? $request->consignee['party_name'] : null;
            $gr->consignee_branch_id = $request->consignee_branch ? $request->consignee_branch['id'] : null;
            $gr->consignee_gst_number = $request->consignee_gst_number;
            $gr->consignee_phone_number = $request->consignee_phone_number;
            $gr->consignee_address = $request->consignee_address;

            $gr->acc_party_id = $request->acc_party ? $request->acc_party['id'] : null; 
            $gr->acc_party_branch_id = $request->acc_party_branch ? $request->acc_party_branch['id'] : null;

            $gr->total_pkg = $request->total_pkg;
            $gr->total_material_weight = $request->total_material_weight;
            $gr->total_km = $request->total_km;
            $gr->total_chg_qty = $request->total_chg_qty;
            $gr->total_desc_value = $request->total_desc_value;
            $gr->total_invoice_value = $request->total_invoice_value;

            $gr->p_marka = $request->p_marka;
            $gr->remarks = $request->remarks;

            $gr->booking_comm_percent = $request->booking_comm_percent;
            $gr->booking_comm_rate = $request->booking_comm_rate;
            $gr->booking_comm_charged_on = $request->booking_comm_charged_on;
            $gr->booking_comm_amount = $request->booking_comm_amount;

            $gr->eway_pic = $request->eway_pic;
            $gr->other_pic = $request->other_pic;

            $gr->print_at_save = $request->print_at_save;

            $gr->freight = $request->freight;
            $gr->gr_chg = $request->gr_chg;
            $gr->payment_dynamic_fields = $request->payment_dynamic_fields;
            $gr->gst_paid_by = $request->gst_paid_by;
            $gr->total_without_gst = $request->total_without_gst;
            $gr->igst_percent = $request->igst_percent;
            $gr->cgst_percent = $request->cgst_percent;
            $gr->sgst_percent = $request->sgst_percent;
            $gr->igst_amount = $request->igst_amount;
            $gr->cgst_amount = $request->cgst_amount;
            $gr->sgst_amount = $request->sgst_amount;
            $gr->total_without_rounding = $request->total_without_rounding;
            $gr->rounded_off = $request->rounded_off;
            $gr->grand_total = $request->grand_total;

            $gr->updated_by = $request->user->id;

            
            $gr->save();
            // NEW PARTY SAVED

            // RECORD THE TRANSACTION IN ACCOUNTS
            if($request->pay_mode == 'TO PAY' || $request->pay_mode == 'PAID')
            $this->record_transaction_in_accounts($request, $gr);
            // TRANSACTION RECORDED IN ACCOUNTS

            // IF EXISTS IN CHALLAN, SIMPLY 
            $challan_check  = $this->check_in_challan_desc('challan_number','challan_number','gr_id',$pk);

            if($challan_check['status']){
                DB::commit();

                $status= $challan_check; 
                return response()->json(['error'=>'Document in use', 'data'=>['message'=>'Challan '.$status['index'].' Restricted Desc.']],200);
            };
            
            // SAVING NEWLY ADDED PACKAGE DETAILS
            $newly_added_pkg_details = $request->newly_added_pkg_details;
            foreach ($newly_added_pkg_details as $napd) {
                if(!$napd['pkg'] || !$napd['material'] || !$napd['chg_unit'] || !$napd['chg_qty'])
                continue;
                
                $pkg_det = new GrDesc();
                $pkg_det->gr_id = $gr->id;
                $pkg_det->gr_number = $gr->gr_number;
                $material_id = $this->add_if_not_in_master('Material', $napd['material'], $request->current_year);

                $pkg_det->material_id = $material_id;
                $pkg_det->destination_id = $request->destination['id'];
                $pkg_det->packing_method = $napd['packing_method'];
                $pkg_det->pkg = $napd['pkg'];
                $pkg_det->nos = $napd['nos'];
                $pkg_det->material_weight = $napd['material_weight'];
                $pkg_det->km = $napd['km'];
                $pkg_det->chg_unit = $napd['chg_unit'];
                $pkg_det->chg_qty = $napd['chg_qty'];
                $pkg_det->rate = $napd['rate'];
                $pkg_det->total = $napd['total'];
                $pkg_det->invoice_number = $napd['invoice_number'];
                $pkg_det->invoice_date = $napd['invoice_date'];
                $pkg_det->invoice_value = $napd['invoice_value'];
                $pkg_det->payment_dynamic_fields = $napd['payment_dynamic_fields'];
                
                $pkg_det->fin_year = $request->current_year;

                $pkg_det->free_pkgs = $napd['pkg'];
                $pkg_det->save();

                // SAVING PACKAGE POSITIONS
                $pkg_position = new PackagePosition();
                $pkg_position->gr_id = $gr->id;
                $pkg_position->gr_number = $gr->gr_number;
                $pkg_position->gr_desc_id = $pkg_det->id;
                $pkg_position->position = $request->branch['id'];
                $pkg_position->pkg = $napd['pkg'];
                $pkg_position->origin = $request->branch['id'];
                $pkg_position->save();
                // PACKAGE POSITIONS SAVED
            }
            // NEWLY ADDED PACKAGE DETAILS SAVED

            // SAVING UPDATES ON EXISTING  PACKAGE DETAILS
            $updated_pkg_details = $request->updated_pkg_details;
            if( !empty($updated_pkg_details) ){
                foreach ($updated_pkg_details as $upd) {
                    if(!$upd['pkg'] || !$upd['material'] || !$upd['chg_unit'] || !$upd['chg_qty'])
                    continue;
                       
                        $pkg_det = GrDesc::find( $upd['id'] );
                        $material_id = $this->add_if_not_in_master('Material', $upd['material'], $request->current_year);

                $pkg_det->material_id = $material_id;
                        $pkg_det->destination_id = $request->destination['id'];
                        $pkg_det->packing_method = $upd['packing_method'];
                        $pkg_det->pkg = $upd['pkg'];
                        $pkg_det->nos = $upd['nos'];
                        $pkg_det->material_weight = $upd['material_weight'];
                        $pkg_det->km = $upd['km'];
                        $pkg_det->chg_unit = $upd['chg_unit'];
                        $pkg_det->chg_qty = $upd['chg_qty'];
                        $pkg_det->rate = $upd['rate'];
                        $pkg_det->total = $upd['total'];
                        $pkg_det->invoice_number = $upd['invoice_number'];
                        $pkg_det->invoice_date = $upd['invoice_date'];
                        $pkg_det->invoice_value = $upd['invoice_value'];
                        $pkg_det->free_pkgs = $upd['pkg'];
                        $pkg_det->payment_dynamic_fields = $upd['payment_dynamic_fields'];
                        $pkg_det->save();

                        // UPDATING PACKAGE POSITIONS
                        $pkg_position = PackagePosition::where('gr_desc_id', $pkg_det->id)->first();
                        $pkg_position->pkg = $upd['pkg'];
                        $pkg_position->save();
                        // UPDATING POSITIONS SAVED
                }
            }
            // UPDATES SAVED ON EXISTING PACKAGE DETAILS

            // DELETING EXISTING PACKAGE DETAILS
            $deleted_pkg_details = $request->deleted_pkg_details;
            if( !empty($deleted_pkg_details) ){
                    $pkg_det = GrDesc::whereIn('id', $deleted_pkg_details);

                    // DELETING PACKAGE POSITIONS
                    $pkg_position = PackagePosition::whereIn('gr_desc_id', $deleted_pkg_details);
                    $pkg_position->delete();

                    $pkg_det->delete();
                    
            }
            //DELETED EXISTING PACKAGE DETAILS



            // SAVING NEWLY ADDED  EWAY DETAILS
            $newly_added_eway_details = $request->newly_added_eway_details;
            foreach ($newly_added_eway_details as $naed) {
                if(!$naed['eway_number'])
                continue;

                $check_unique = EwayGr::where('eway_number',$naed['eway_number'])->where('fin_year',$request->current_year)->exists();
                if($check_unique){
                    return response()->json(['error'=>'E-Way in use - '.$naed['eway_number'], 'data'=>$naed['eway_number']],500);
                }
                
                $pkg_det = new EwayGr();
                $pkg_det->gr_id = $gr->id;
                $pkg_det->gr_number = $gr->gr_number;
                $pkg_det->eway_number = $naed['eway_number'];
                $pkg_det->eway_exp_date = $naed['eway_exp_date'];
                $pkg_det->fin_year = $request->current_year;
                $pkg_det->save();
            }
            // NEWLY ADDED  EWAY DETAILS SAVED

            // SAVING UPDATES ON EXISTING   EWAY DETAILS
            $updated_eway_details = $request->updated_eway_details;
            if( !empty($updated_eway_details) ){
                foreach ($updated_eway_details as $ued) {
                    if(!$ued['eway_number'])
                    continue;

                    $check_unique = EwayGr::where('eway_number',$ued['eway_number'])->where('gr_id','!=',$gr->id)->where('fin_year',$request->current_year)->exists();
                    if($check_unique){
                        return response()->json(['error'=>'E-Way in use - '.$ued['eway_number'], 'data'=>$ued['eway_number']],500);
                    }
                    
                    $pkg_det = EwayGr::find( $ued['id'] );
                    $pkg_det->eway_number = $ued['eway_number'];
                    $pkg_det->eway_exp_date = $ued['eway_exp_date'];
                    $pkg_det->save();
                }
            }
            // UPDATES SAVED ON EXISTING  EWAY DETAILS

            // DELETING EXISTING  EWAY DETAILS
            $deleted_eway_details = $request->deleted_eway_details;
            if( !empty($deleted_eway_details) ){
                    $pkg_det = EwayGr::whereIn('id', $deleted_eway_details);
                    $pkg_det->delete();
            }
            //DELETED EXISTING  EWAY DETAILS
    


            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($gr->id);
            $this->createEditLog('updated', 'book_grs', $gr->id, $gr->gr_number,$oldValue, $newVal, $request->user->id,$request->current_year); // CREATE EDIT LOG
            

            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function destroy(Request $request, $pk){
        $test = $this->is_delete_permitted($request->user->id, 'GR');
        $test_delete = $test->delete;
        $delete_valid_till_hours = $test->delete_valid_till;

        if( !$test_delete ){
            return response()->json(['Permission Denied For The Action'],401);
        }

        // FETCHING OLD VALUE FOR LOG
        $oldValue = $this->get_data_for_obj($pk);

        
    try {
        // TRANSACTION BEGINS
        DB::beginTransaction();
    

        $gr = BookGr::find($pk);

        $pod_check = $this->check_in_pod_desc('pod_id','pod_number', 'gr_id', $pk);
        $bill_check = $this->check_in_bill_desc('bill_id','bill_number', 'gr_id', $pk);
        $challan_check  = $this->check_in_challan_desc('challan_number','challan_number','gr_id',$pk);
        $voucher_check  = $this->check_in_voucher_desc('voucher_number','voucher_number','GR',$gr->gr_number);
        $lorry_hire_check = $this->check_in_lorry_hire_desc('lh_number','lh_number','GR',$gr->gr_number);
        $diesel_receipt_check = $this->check_in_diesel_receipt('dr_number','dr_number','GR',$gr->gr_number);
        $driver_settlement_desc_check = $this->check_in_driver_stl_desc('ds_number','ds_number','GR',$gr->gr_number);
            
        if($pod_check['status'] || $bill_check['status'] || $challan_check['status'] || $voucher_check['status'] || $lorry_hire_check['status'] || $diesel_receipt_check['status'] || $driver_settlement_desc_check['status'] ){
           $status = $bill_check['status'] ? $bill_check : ($pod_check['status'] ? $pod_check : ($challan_check['status'] ? $challan_check : ($voucher_check['status'] ? $voucher_check : ($lorry_hire_check['status'] ? $lorry_hire_check : ($diesel_receipt_check['status'] ? $diesel_receipt_check : $driver_settlement_desc_check) ))));
           return response()->json(['error'=>'Document in use', 'data'=>$status],500);
        };
        
        if( $delete_valid_till_hours!=null ){
            $validity = Carbon::parse($gr->created_at)->addHours($delete_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }

        //FIND AND DELETE THE PACKAGE POSITIONS
        $pkg_positions = PackagePosition::where('gr_id', $pk)->delete();
        $gr_desc_ids = GrDesc::where('gr_id', $pk)->delete();

        //FIND AND DELETE THE BRANCH
        $gr->delete();
        // COMMISSIONS WILL BE CASCADED AUTOMATICALLY
        
        // DELETE THE TRANSACTION VOUCHER
        $t_data = [];
        $t_data['t_against'] = 'GR';
        $t_data['t_against_no'] = $gr->gr_number;
        $t_data['t_against_id'] = $gr->id;
        $t_data['fin_year'] = $request->current_year;

        $this->delete_transaction_voucher($t_data);

        $this->createEditLog('deleted', 'book_grs', $gr->id, $gr->gr_number, $oldValue, null, $request->user->id,$request->current_year); // CREATE EDIT LOG

        // TRANSACTIONS COMMITTED
            DB::commit();
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function cancel_gr(Request $request, $pk){
        $test = $this->is_edit_permitted($request->user->id, 'GR');
        $test_edit = $test->edit;
        $edit_valid_till_hours = $test->edit_valid_till;

        if( !$test_edit ){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        // FETCHING OLD VALUE FOR LOG
        $oldValue = $this->get_data_for_obj($pk);

        
        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();
        

            $gr = BookGr::find($pk);

            if( $edit_valid_till_hours!=null ){
                $validity = Carbon::parse($gr->created_at)->addHours($edit_valid_till_hours);
                if( !$validity->isFuture() ){
                    return response()->json(['Permission Timed Out'],500);
                }
            }

            $challan_check  = $this->check_in_challan_desc('challan_number','challan_number','gr_id',$pk);
                
            if($challan_check['status']){
                DB::commit();

                $status= $challan_check; 
                return response()->json(['error'=>'Document in use', 'data'=>['message'=>'Challan '.$status['index'].' Restricted Desc.']],500);
            };
            

            $gr->is_cancelled = !$gr->is_cancelled;

            $gr->save();
            
            // DELETE THE TRANSACTION VOUCHER
            $t_data = [];
            $t_data['t_against'] = 'GR';
            $t_data['t_against_no'] = $gr->gr_number;
            $t_data['fin_year'] = $request->current_year;

            if($gr->is_cancelled){
                $this->set_transaction_status($t_data, false);;
            }else{
                // RECORD THE TRANSACTION IN ACCOUNTS
                $this->set_transaction_status($t_data, true);;
                // TRANSACTION RECORDED IN ACCOUNTS
            }

            $newVal = $this->get_data_for_obj($gr->id);
            $this->createEditLog('updated', 'book_grs', $gr->id, $gr->gr_number,$oldValue, $newVal, $request->user->id,$request->current_year); // CREATE EDIT LOG
                
                // TRANSACTIONS COMMITTED
                DB::commit();
                return response()->json(['status'=>$gr->is_cancelled], 200); // RETURNED RESPONSE

            } catch (\Exception $e) {
                DB::rollback();
                return response()->json(['error' => $e->getMessage()], 500);
            }
    }

    public function autocomplete(Request $request){
        // Initialize the query builder
        $query = BookGr::select('id', 'gr_number')->where('fin_year',$request->current_year);

        $user_branch_ids = $request->user_branch_ids;
        if ( $user_branch_ids ) {
            $query->whereIn('branch_id', $user_branch_ids );
        }
        if( $request->user->party_id ){
            $prt = Party::select('account_id')->find($request->user->party_id);
            
            $query->where(function($query) use ($prt){
                $query->where('acc_party_id', $prt->account_id)->where('acc_party_id','!=', null);
            });
        }

       // Apply any additional filters based on other columns as needed
        if ($request->has('filterby')) {
            $orWhereFilters = isset($request->filterby['orWhere']) ? $request->filterby['orWhere'] : [];
            $whereFilters = isset($request->filterby['where']) ? $request->filterby['where'] : [];
            $whereInFilters = isset($request->filterby['whereIn']) ? $request->filterby['whereIn'] : [];

        // Loop through the whereFilters array
        foreach ($whereFilters as $whereFilter) {
            // Apply the filter to the query
            $query->where($whereFilter['key'], $whereFilter['value']);
        }

        // Loop through the orWhereFilters array
        foreach ($orWhereFilters as $orWhereFilter) {
            // Apply the filter to the query
            $query->orWhere($orWhereFilter['key'], $orWhereFilter['value']);
        }

        foreach ($whereInFilters as $whereInFilter) {
            if(isset($whereInFilter['key']) && isset($whereInFilter['values'])){
                $query->whereIn($whereInFilter['key'], $whereInFilter['values']);
            }
        }  
        
        }
        
        // Check if searchVariable is provided and apply the filter
        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('gr_number', 'like', '%'. $searchVariable . '%');
        }

        // Limit the number of results
        $limit = $request->limit ? $request->limit : 10;

        $query->take($limit)->orderBy('id', 'desc');

        // Retrieve the filtered 
        $grs = $query->get();
        return response(['grs' => $grs, 'draw' => $request->draw]);
    }

    public function autocomplete_gr_for_pod(Request $request) {
        // Initialize the query builder
        $query = BookGr::select('id', 'gr_number')->where('is_cancelled',false)->whereHas('package_position', function($query) use ($request){
            $query->where('pkg','>',0)->where('is_delivered',0);
        });
    
        // Check if searchVariable is provided and apply the filter
        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('gr_number', 'like', '%'. $searchVariable . '%');
        }
    
        // Apply a custom where clause to filter records where 'pod_desc' does not exist
        $query->whereDoesntHave('pod_desc');
    
        // Limit the number of results
        $query->take($request->limit)->orderBy('id', 'desc');
    
        // Retrieve the filtered records
        $grs = $query->get();
    
        return response(['grs' => $grs]);
    }
    
    public function autocomplete_gr_for_challan(Request $request){
        // Initialize the query builder
        $currentYear = $request->current_year;

        $query = PackagePosition::select('gr_id as id', 'gr_number')->whereHas('bookGr', function($query) use ($currentYear){
            $query->where('is_cancelled',false);
        });

        $selected_branch_ids = $request->selected_branch_ids;
        
        $query->whereIn('position', $selected_branch_ids )->where('pkg','>',0)->where('is_delivered',0)->distinct();
        
        
        // Check if searchVariable is provided and apply the filter
        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('gr_number', 'like', '%'. $searchVariable . '%');
        }

        // Limit the number of results
        $query->take($request->limit)->orderBy('id', 'desc');

        // Retrieve the filtered 
        $grs = $query->get();
        return response(['grs' => $grs, 'draw' => $request->draw]);
    }

    public function autocomplete_gr_for_eway(Request $request){
        // Initialize the query builder
        $query = EwayGr::select('gr_id as id','gr_number')->where('fin_year',$request->current_year)->where('is_delivered',0)->distinct();

        // Check if searchVariable is provided and apply the filter
        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('gr_number', 'like', '%'. $searchVariable . '%');
        }

        // Limit the number of results
        $limit = $request->limit ? $request->limit : 10;

        $query->take($limit)->orderBy('id', 'desc');

        // Retrieve the filtered 
        $grs = $query->get();
        return response(['grs' => $grs, 'draw' => $request->draw]);
    }
    

    public function datatable(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'GR');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $query = BookGr::select('id', 'gr_number', 'grand_total', 'gr_date','gr_time' , 'branch_id','source_id', 'destination_id', 'consignor_id', 'consignee_id')
                ->with(['branch:id,branch_name,branch_code','source:id,station_name', 'destination:id,station_name', 'consignor:id,party_name', 'consignee:id,party_name']) 
                ->where('fin_year',$request->current_year)
                ->orderBy($column, $dir);

        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('gr_number', 'like', $searchValue . '%');
            });
        }

        $data = $query->paginate($length);

        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    public function check_issued_gr(Request $request){
        $gr_number = $request->gr_number;
        $branch_id = $request->branch_id;
        $gr_number = $request->gr_number;
        $prefix = $request->prefix;

        // Break GR NUMBER BASED ON ANY Separater
        $gr_number = $this->get_max_number_from_gr_number($gr_number);

        $check = IssueBooks::with(['branch' => function ($query) {
                        $query->select('id', 'branch_name','branch_code');
                    },
                    'party' => function ($query) {
                        $query->select('id', 'party_name','party_code','account_id');
                    }
                ])
                ->where('book_type', 'GR')
                ->where('prefix', $prefix)
                ->where('branch_id', $branch_id)
                ->where('issue_from', '<=', $gr_number)
                ->where('issue_to', '>=', $gr_number)
                ->first();

        if($check){
            return response()->json(['success'=>true, 'data'=>$check],200);
        }
        else{
            return response()->json(['success'=>false, 'msg'=>'Not Issued To Correct Branch'],200);
        }

    

    }

    public function check_unique(Request $request){
        $key = $request->key;
        $value = $request->value;

        if(!$value){
            return response()->json(['exists' => false]);
        }

        $exists = BookGr::where($key, $value)->exists();
        if($exists){
            return response()->json(['errors' => [ $key  => ['Already Exists !']]],422);
        }

        return response()->json(['exists' => false]);
    }

    public function get_recent(Request $request) {

        $grsQuery = BookGr::select('id', 'gr_number')->where('fin_year',$request->current_year);
         
        if ( !empty($request->user_branch_ids) ) {
            $grsQuery->whereIn('branch_id', $request->user_branch_ids );
        }

        if( $request->user->party_id ){
            $prt = Party::select('account_id')->find($request->user->party_id);
            
            $grsQuery->where(function($query) use ($prt){
                $query->where('acc_party_id', $prt->account_id)->where('acc_party_id','!=', null);
            });
        }
        
        $grs = $grsQuery->orderBy('id', 'desc')->take(10)->get();
        
        return response()->json(['grs' => $grs]);
    }

    public function track_gr(Request $request , $pk){

        $data = BookGr::select('id','gr_number','gr_date', 'consignor_id','consignee_id','source_id','destination_id','consignor_address','consignee_address','remarks','delivery_pt','pay_mode','other_pic')
        ->with([
             'branch:id,branch_name,branch_code','source:id,station_name', 'destination:id,station_name', 'consignor:id,party_name', 'consignee:id,party_name', 'vehicle:id,vehicle_number','driver:id,emp_name',
             'gr_desc',
             'gr_desc.material' => function ($query) {
                 $query->select('id', 'material_name');
             },
             'eway_details',
             'acc_party' => function ($query) {
                 $query->select('id', 'account_name','account_type');
             },
             'challan_desc:id,challan_number,challan_id,gr_id,loaded_pkgs',
             'challan_desc.challan:id,challan_number,challan_date,challan_time,source_id,destination_id,vehicle_id,vehicle_number,remarks',
             'challan_desc.challan.source:id,station_name',
             'challan_desc.challan.destination:id,station_name',
             'challan_desc.challan.vehicle:id,vehicle_number',
             'challan_desc.challan_rec_desc:id,challan_desc_id,gr_id,challan_receipt_id,challan_rec_number',
             'challan_desc.challan_rec_desc.challan_receipt:id,challan_rec_number,arrival_date,arrival_time,remarks',
             'pod_desc:id,gr_id,pod_id,pod_number',
             'pod_desc.pod:id,pod_number,delivery_date,delivery_time,remarks,pod_pic',
             'bill_desc:id,gr_id,bill_number,bill_id',
             'bill_desc.bill:id,bill_number,bill_date',
             'package_position:id,gr_id,gr_number,position,pkg,is_delivered',
             'package_position.branch:id,branch_name,branch_code'
        ])
        ->find($pk);


        $v_query = VoucherDesc::select('id', 'voucher_id', 'voucher_number', 'against_number', 'against_id', 'against', 'receipt_amt', 'voucher_name')
                    ->where('fin_year', $request->current_year);

                // Add 'GR' condition if $data['id'] is set
                if (isset($data['id'])) {
                    $v_query->where(function ($qr) use ($data) {
                        $qr->where('against', 'GR')->where('against_id', $data['id']);
                    });
                }

                // Add 'POD' condition if $data['pod_desc']['pod_id'] is set
                if (isset($data['pod_desc']['pod_id'])) {
                    $v_query->orWhere(function ($qr) use ($data) {
                        $qr->where('against', 'POD')->where('against_id', $data['pod_desc']['pod_id']);
                    });
                }

                // Add 'Bill' condition if $data['bill_desc']['bill_id'] is set
                if (isset($data['bill_desc']) && count($data['bill_desc'])) {
                    $v_query->orWhere(function ($qr) use ($data) {
                        // Extract 'bill_id' from each object in $data['bill_desc']
                        $bill_ids = collect($data['bill_desc'])->pluck('bill_id')->toArray();

                        // Add condition to the query
                        $qr->where('against', 'Bill')->whereIn('against_id', $bill_ids);
                        
                    });
                }

                $voucher_desc = $v_query->get();



        $data['voucher_desc'] = $voucher_desc;


        return response($data);

    }


    public function get_next_gr_number(Request $request){
        $type=$request->type;
        $prefix=$request->prefix;
        $branch_id = $request->branch_id;
        $branch_code = $request->branch_code;

        if($type=='SYSTEM'){
            $max = BookGr::where('branch_id',$branch_id)->where('gr_type', $type)->max('max_number');
            $gr_number = $branch_code.'-'.sprintf('%06d', $max+1);
            
            return response()->json(['gr_number' => strtoupper($gr_number), 'max_number' => $max+1]);
        }
        else if($type=='PRINTED'){
            // GET THE NEXT NUMBER THAT HAS BEEN ISSUE TO THE BRANCH WITH THE CERTAIN PREFIX
            $max = BookGr::where('prefix',$prefix)->where('gr_type', $type)->where('branch_id', $branch_id)->max('max_number');
            
            // IF MAX IS PRESENT, THEN CHECK IN ISSUE BOOKS IF IT IS VALID
            if( $max ){
                $check = IssueBooks::select('prefix','party_id')->with(['party:id,party_name'])->where('prefix',$prefix)->where('branch_id', $branch_id)->where('book_type','GR')->where('issue_from', '<=', $max+1)->where('issue_to', '>=', $max+1)->first();
                if($max && $check){
                    $gr_number = $prefix ? $prefix.'-'.($max+1) : ($max+1);
                    return response()->json(['gr_number' => strtoupper($gr_number), 'max_number' => $max+1, 'party' => $check->party]);
                }
            }

            // IF MAX IS NOT PRESENT OR NOT VALID, THEN CHECK IN ISSUE BOOKS IF IT IS VALID
            $check = IssueBooks::select('prefix','party_id','issue_from')->with(['party:id,party_name'])->where('prefix',$prefix)->where('branch_id', $branch_id)->where('book_type','GR')->where('issue_from', '>=', $max+1)->first();
            if( $check ){
                $gr_number = $prefix ? $prefix.'-'.$check->issue_from : $check->issue_from;
                return response()->json(['gr_number' => strtoupper($gr_number), 'max_number' => $check->issue_from, 'party' => $check->party]);
            }
            else{
                return response()->json(['errors' => ['gr_number' => ['No Valid GR Number']]],422);
            }
        }
    }

    public function fetch_gr_desc_for_challan(Request $request){
        $gr_selection = $request->gr_selection;
        $branch = $request->branch;

        $selected_branch_ids = $request->selected_branch_ids;
        
        $type = $gr_selection['GRSelectionType'];

        if($type == 'GR WISE'){
            // GET DESCS ALONG WITH THEIR GRS
            $gr_descs = GrDesc::with('material:id,material_name','bookGr:id,branch_id,gr_number,consignor_id,consignee_id,destination_id,destination_name,consignor_name,consignee_name,grand_total,pay_mode','bookGr.consignor:id,party_name','bookGr.consignee:id,party_name','bookGr.destination:id,station_name')
                       ->where('gr_id', $gr_selection['gr_number']['id'])
                       ->where('free_pkgs','>',0)
                       ->whereHas('bookGr', function ($query) {
                        $query->where('is_cancelled', false);
                       })
                       ->get();
        }
        else if( $type == 'STATION WISE' ){
            $gr_descs = GrDesc::with('material:id,material_name','bookGr:id,branch_id,gr_number,consignor_id,consignee_id,destination_id,destination_name,consignor_name,consignee_name,grand_total,pay_mode','bookGr.consignor:id,party_name','bookGr.consignee:id,party_name','bookGr.destination:id,station_name')
                       ->where('destination_id', $gr_selection['station']['id'])
                       ->where('free_pkgs','>',0)
                       ->whereHas('bookGr', function ($query) {
                        $query->where('is_cancelled', false);
                       })
                       ->get();
        }
        else if( $type =='ALL' ){
            // GET ALL GR DESCS WHERE free_pks > 0 
            $gr_descs = GrDesc::with('material:id,material_name','bookGr:id,branch_id,gr_number,consignor_id,consignee_id,destination_id,destination_name,consignor_name,consignee_name,grand_total,pay_mode','bookGr.consignor:id,party_name','bookGr.consignee:id,party_name','bookGr.destination:id,station_name')
                       ->where('free_pkgs','>',0)
                       ->whereHas('bookGr', function ($query) {
                        $query->where('is_cancelled', false);
                       })
                       ->take($gr_selection['to'])
                       ->get();
        }

        $gr_descs->transform(function ($gr_desc) use ($selected_branch_ids) {
            $free = PackagePosition::where('gr_desc_id', $gr_desc->id)->whereIn('position', $selected_branch_ids)->sum('pkg');
            return [
                'branch_id' => $gr_desc->bookGr->branch_id,
                'challan_desc_id' => null,
                'gr_id' => $gr_desc->gr_id,
                'gr_number' => $gr_desc->gr_number,
                'challan_id' => null,
                'challan_number' => null,
                'gr_desc_id' => $gr_desc->id,
                'pkg' => $gr_desc->pkg,
                'wt' => $gr_desc->material_weight,
                'km'=> $gr_desc->km,
                'free_pkgs' => $free,
                'loaded_pkgs' => $free,
                'material' => $gr_desc->material['material_name'],
                'consignor' => $gr_desc->bookGr->consignor_id ? $gr_desc->bookGr->consignor['party_name'] : $gr_desc->bookGr->consignor_name,
                'consignee' => $gr_desc->bookGr->consignee_id ? $gr_desc->bookGr->consignee['party_name'] : $gr_desc->bookGr->consignee_name,
                'destination' => $gr_desc->bookGr->destination_id ?  $gr_desc->bookGr->destination['station_name'] : $gr_desc->bookGr->destination_name,
                'grand_total' => $gr_desc->bookGr->grand_total,
                'pay_mode' => $gr_desc->bookGr->pay_mode, 
                'checked' => true,
            ];
        });
        return response()->json(['gr_descs' => $gr_descs]);
        
    }

    public function fetch_gr_details_for_pod(Request $request, $pk){
        $gr = BookGr::select('id','branch_id', 'gr_number', 'gr_date', 'pay_mode', 'acc_party_id','acc_party_branch_id', 'vehicle_id','vehicle_number', 'delivery_pt', 'source_id', 'destination_id', 'consignor_id', 'consignee_id','consignor_name','consignee_name','source_name','destination_name', 'total_pkg', 'total_material_weight','total_km', 'total_chg_qty', 'total_desc_value', 'total_invoice_value', 'freight','gr_chg','payment_dynamic_fields','gst_paid_by','total_without_gst','igst_amount','cgst_amount','sgst_amount','igst_percent','cgst_percent','sgst_percent', 'grand_total')
              ->with([
                    'branch:id,branch_name,branch_code',
                    'acc_party' => function ($query) {
                        $query->select('id', 'account_name','account_type');
                    },
                    'acc_party.party:id,account_id',
                    'acc_party_branch:id,branch_name',
                    'vehicle:id,vehicle_number,vehicle_type',
                    'consignor:id,party_name',
                    'consignee:id,party_name',
                    'source:id,station_name',
                    'destination:id,station_name'       
              ])
              ->find($pk);
        
        $data = [
                'pod_desc_id' => null,
                'pod_id' => null,
                'gr_id' => $gr->id,
                'gr_number' => ['id'=> $gr->id, 'gr_number'=>$gr->gr_number],
                'gr_date' => $gr->gr_date,
                'source' => $gr->source_id ? $gr->source : ['id'=> null, 'station_name'=>$gr->source_name] ,
                'destination' => $gr->destination_id ? $gr->destination : ['id'=> null, 'station_name'=>$gr->destination_name] ,
                'consignor' => $gr->consignor_id ? $gr->consignor : ['id'=> null, 'party_name'=>$gr->consignor_name] ,
                'consignee' => $gr->consignee_id ? $gr->consignee : ['id'=> null, 'party_name'=>$gr->consignee_name] ,
                'total_pkg' => $gr->total_pkg,
                'total_km' => $gr->total_km, 
                'total_material_weight' => $gr->total_material_weight,
                'chg_unit' => '', // Add your logic for getting the chg_unit here
                'total_chg_qty' => $gr->total_chg_qty,
                'rate' => '', // Add your logic for getting the rate here
                'total_desc_value' => $gr->total_desc_value,
                'total_invc_value' => $gr->total_invoice_value,
                'payment_dynamic_fields' => $gr->payment_dynamic_fields,
                'pay_mode' => $gr->pay_mode,
                'vehicle'=> $gr->vehicle ? $gr->vehicle : ($gr->vehicle_number ? ['id'=> null, 'vehicle_number'=>$gr->vehicle_number, 'vehicle_type'=>$gr->vehicle_type] : null),
                'acc_party'=> $gr->acc_party, 
                'acc_party_branch'=> $gr->acc_party_branch,
                'freight' => $gr->freight,
                'gr_chg' => $gr->gr_chg,
                'gst_paid_by' => $gr->gst_paid_by,
                'total_without_gst' => $gr->total_without_gst,
                'igst_amount' => $gr->igst_amount,
                'cgst_amount' => $gr->cgst_amount,
                'sgst_amount' => $gr->sgst_amount,
                'igst_percent' => $gr->igst_percent,
                'cgst_percent' => $gr->cgst_percent,
                'sgst_percent' => $gr->sgst_percent,
        ];

        return response()->json(['gr' => $data]);
        
    }

    public function get_gr_report(Request $request){
        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $report_data = $request->report_data;
                                                                                                                 
        $query = BookGr::select('id','gr_number','gr_date','consignor_id','consignee_id','acc_party_id','edd','source_id','destination_id','vehicle_id','vehicle_number','pay_mode','total_pkg','total_material_weight','total_invoice_value','igst_amount','cgst_amount','sgst_amount','total_without_rounding','is_cancelled')
                        ->with(
                            ['consignor:id,party_name','consignee:id,party_name','acc_party:id,account_name','acc_party.party:id,account_id,gst_number','source:id,station_name','destination:id,station_name','vehicle:id,vehicle_number','gr_desc','gr_desc.material:id,material_name',
                                'challan_desc' => function ($query) {
                                    $query->select('gr_id','challan_id', 'challan_number')->groupBy('gr_id','challan_id', 'challan_number');
                                },
                                'pod_desc' => function ($query) {
                                    $query->select('gr_id','pod_id', 'pod_number')->groupBy('gr_id', 'pod_id','pod_number');
                                },
                                'bill_desc' => function ($query) {
                                    $query->select('gr_id', 'bill_id','bill_number')->groupBy('gr_id', 'bill_id','bill_number');
                                },
                                'voucher_desc'=>function($query){
                                    $query->select('against_number','voucher_id', 'voucher_number')->where('against','GR')->groupBy('against_number','voucher_id', 'voucher_number');
                                }
                        
                            ]
                        )->where('fin_year',$request->current_year);

        
        if(isset($report_data['branch']) ){
            $query->where('branch_id', $report_data['branch']['id']);
        }

        if(isset($report_data['gr_num_from']) && isset($report_data['gr_num_to'])){
            $query->where('gr_number','>=',$report_data['gr_num_from']['gr_number'])->where('gr_number','<=',$report_data['gr_num_to']['gr_number']);
        }


        if(isset($report_data['from_date']) && isset($report_data['to_date'])){
            $query->where('gr_date','>=',$report_data['from_date'])->where('gr_date','<=',$report_data['to_date']);
        }

        if(isset($report_data['consignor']) ){
            $query->where('consignor_id', $report_data['consignor']['id']);
        }

        if(isset($report_data['consignee']) ){
            $query->where('consignee_id', $report_data['consignee']['id']);
        }

        if(isset($report_data['acc_party']) ){
            $query->where('acc_party_id', $report_data['acc_party']['id']);
        }

        if(isset($report_data['pay_mode']) ){
            $query->where('pay_mode', $report_data['pay_mode']);
        }

        if(isset($report_data['paid_status']) ){
            if($report_data['paid_status']=='Paid')
            $query->where('paid_status', 1);
            else if($report_data['paid_status']=='Outstanding')
            $query->where('paid_status', 0);
        }


        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('gr_number', 'like', $searchValue . '%');
            });
        }

        $data = $query->orderBy($column, $dir)->paginate($length);
          
        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    public function get_gr_stock_report(Request $request){
        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $report_data = $request->report_data;
        
        $branch = $report_data['branch'];
        $from_date = $report_data['from_date'];
        $to_date = $report_data['to_date'];
        $status = $report_data['status'];
                
                // Define the package_position relationship separately
                $packagePositionRelationship = function ($query) use ($status, $branch) {
                    if ($status == 'Pending') {
                        $query->where('position', $branch['id'])->where('pkg', '>', 0)->where('is_delivered', 0);
                    }

                    if ($status == 'Delivered') {
                        $query->where('position', '!=', 'IN TRANSIT')->where('is_delivered', 1);
                    }

                    if ($status == 'In Transit') {
                        $query->where('position', 'IN TRANSIT')->where('pkg', '>', 0)->where('is_delivered', 0);
                    }
                };

                // Use the defined relationship in the with method
                $query = BookGr::with([
                    'branch:id,branch_name',
                    'consignor:id,party_name',
                    'consignee:id,party_name',
                    'source:id,station_name',
                    'destination:id,station_name',
                    'pod_desc' => function ($query) {
                        $query->select('gr_id', 'pod_id', 'pod_number')->groupBy('gr_id', 'pod_id', 'pod_number');
                    },
                    'package_position' => $packagePositionRelationship,
                    'package_position.challan_desc:id,gr_id,challan_id,challan_number,loaded_pkgs',
                    'package_position.gr_desc:id,material_id,pkg',
                    'package_position.gr_desc.material:id,material_name',
                    'package_position.branch:id,branch_code'
                ])
                    ->where('gr_date', '>=', $from_date)
                    ->where('gr_date', '<=', $to_date)
                    ->when($status == 'Delivered' || $status == 'In Transit', function ($query) use ($branch) {
                        $query->where('branch_id', $branch['id']);
                    })
                    ->whereHas('package_position', $packagePositionRelationship);

                
                
    

        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('gr_number', 'like', $searchValue . '%');
            });
        }


        $data = $query->orderBy($column, $dir)->paginate($length);
        return response()->json(['data' => $data, 'draw' => $request->draw]);

    }


    public function get_gr_data_for_eway_partb(Request $request,$pk){
        $eway_numbers = EwayGr::where('gr_id',$pk)->pluck('eway_number')->toArray();

        $gr = BookGr::select('source_id','destination_id')->with(['source:id,station_name', 'destination:id,station_name'])->find($pk);

        $data = [
            'source' => $gr->source_id ? $gr->source : null,
            'destination' => $gr->destination_id ? $gr->destination : null,
            'eway_numbers' => $eway_numbers
        ];

        return response()->json(['data' => $data]);

    }

    protected function get_data_for_obj($pk){
        // FETCH AND SEND
        $gr = BookGr::with([
            'branch:id,branch_name,branch_code','source:id,station_name', 'destination:id,station_name', 'consignor:id,party_name', 'consignee:id,party_name', 'vehicle:id,vehicle_number', 'driver:id,emp_name',
            'gr_desc',
            'gr_desc.material' => function ($query) {
                $query->select('id', 'material_name');
            },
            'eway_details',
            'acc_party' => function ($query) {
                $query->select('id', 'account_name');
            },
            'acc_party_branch:id,branch_name',
            'consignor_branch:id,branch_name',
            'consignee_branch:id,branch_name',
            ])->find($pk);

        return ['gr'=>$gr];
    }

    protected function get_max_number_from_gr_number($input){
        // Extract the integer value using regular expression
        $pattern = '/[-\/](\d+)$/';
        preg_match($pattern, $input, $matches);
        
        if(empty($matches)){
            return intval($input);
        }

        $integerValue = intval(end($matches));
        return $integerValue;
    }

    protected function record_transaction_in_accounts($request, $gr){
            $t_data = [];
            $t_data['t_against'] = 'GR';
            $t_data['t_against_id'] = $gr->id;
            $t_data['fin_year'] = $request->current_year;
            $t_data['t_against_no'] = $gr->gr_number;
            
            $this->delete_transaction_voucher($t_data);
            
            $t_data['branch'] = $request->branch;
            $t_data['voucher_date'] = $request->gr_date;
            $t_data['voucher_time'] = $request->gr_time;
            $t_data['t_against_narration'] = $gr->remarks;
            $t_data['fin_year'] = $request->current_year;

            $t_data['voucher_accounting_details'] = [];
            
            $t_data['cancellable'] = 1;
            if($gr->income_account_id && $request->total_without_rounding>0){
                $t_data['voucher_accounting_details']=[];
                // DEBIT PARTY ACCOUNT
                $t_data['voucher_accounting_details'][] = ['account_id'=>$gr->acc_party_id, 'nature'=>'dr' , 'dr_amount'=>$request->total_without_rounding, 'cr_amount'=>0 ];
                

                // CREDIT SIDE
                if($request->cgst_amount > 0){
                    // CREDIT CGST ACCOUNT
                    $t_data['voucher_accounting_details'][] = ['account_id'=>12, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$request->cgst_amount ];
                }
                if($request->sgst_amount > 0){
                    // CREDIT SGST ACCOUNT
                    $t_data['voucher_accounting_details'][] = ['account_id'=>13, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$request->sgst_amount ];
                }
                if($request->igst_amount > 0){
                    // CREDIT IGST ACCOUNT
                    $t_data['voucher_accounting_details'][] = ['account_id'=>14, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$request->igst_amount ];
                }

                // CREDIT INCOME ACCOUNT
                $t_data['voucher_accounting_details'][] = ['account_id'=>$gr->income_account_id, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$request->total_without_gst ];

                
                //CHECK TOTAL DR AND CR
                $total_dr = 0;
                $total_cr = 0;

                foreach ($t_data['voucher_accounting_details'] as $vad) {
                    $total_dr += $vad['dr_amount'];
                    $total_cr += $vad['cr_amount'];
                }

                $total_dr_rounded = round(floatval($total_dr), 2);
            $total_cr_rounded = round(floatval($total_cr), 2);

            // Check if they are not equal
            if ($total_dr_rounded != $total_cr_rounded) {
                // Output the values for debugging
                dd($total_dr_rounded, $total_cr_rounded);
                
                // Return response with an error message
                return response()->json(['message' => 'Error In Accounting Entries'], 422);
            }


                $t_data['total_dr_amount'] = $total_dr;
                $t_data['total_cr_amount'] = $total_cr; 


                $this->create_transaction_voucher( $t_data );
            }
    }

    


}
