<?php

namespace App\Http\Controllers\FleetOwner;

use App\Http\Controllers\Controller;
use App\Http\Traits\CommonTrait;
use App\Http\Traits\EditLogTrait;
use App\Http\Traits\PermissionsTrait;
use App\Models\Booking\BookGr;
use App\Models\FleetOwner\DieselReceipt;
use App\Models\FleetOwner\LoadingSlip;
use App\Models\GoodsMove\Challan;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class DieselReceiptController extends Controller
{
    use PermissionsTrait;
    use EditLogTrait;
    use CommonTrait;

    public function index(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'DieselReceipt');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        //FETCH AND SEND ALL
        $diesel_receipts = DieselReceipt::with([
                    'branch:id,branch_name,branch_code',
                    'vehicle:id,vehicle_number',
                    'dr_on_account:id,account_name',
                    'filling_station:id,account_name'
                    ])->where('fin_year',$request->current_year)->get();
        return response()->json($diesel_receipts);
    }

    public function show(Request $request, $pk){
        // FETCH AND SEND
        $dr = DieselReceipt::with([
                        'branch:id,branch_name,branch_code',
                        'vehicle:id,vehicle_number',
                        'dr_on_account:id,account_name',
                        'filling_station:id,account_name'
                    ])->find($pk);

        return response()->json(['diesel_receipt' => $dr]);
    }

    public function store(Request $request){

        $test = $this->is_new_permitted($request->user->id, 'DieselReceipt');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $validatedData = $request->validate([
            'branch' => 'required',
            'dr_number' => 'required|unique:diesel_receipts,dr_number',
            'vehicle' => 'required',
            'dr_on_account' => 'required',
            'filling_station' => 'required',
        ], [
            'branch.required' => 'Required',
            'dr_number.required' => 'Required',
            'vehicle.required' => 'Required',
            'dr_on_account.required' => 'Required',
            'filling_station.required' => 'Required',
        ]);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();  

            $max_number = $this->get_max_number_from_dr_number($request->dr_number);   
            
            // SAVING A NEW DR
            $dr = new DieselReceipt();
            $dr->branch_id = $request->branch['id'];
            $dr->max_number = $max_number;
            $dr->dr_number = $request->dr_number;
            $dr->against = $request->against;


            $dr->vehicle_id = $request->vehicle['id'];
            $dr->vehicle_number = $request->vehicle['vehicle_number'];
            $dr->booking_id = $request->booking_number['id'];
            $dr->booking_number = $request->booking_number['against_number'];
            $dr->booking_date = $request->booking_date;
            $dr->bill_number = $request->bill_number;
            $dr->dr_on_account = $request->dr_on_account['id'];
            $dr->filling_station = $request->filling_station['id'];
            $dr->slip_number = $request->slip_number;
            $dr->fuel_type = $request->fuel_type;
            $dr->qty = $request->qty;
            $dr->rate = $request->rate;
            $dr->amount = $request->amount;
            $dr->remarks = $request->remarks;

            $dr->fin_year = $request->current_year; 

            $dr->created_by = $request->user->id;

            // Save the model to the database
            $dr->save();

            
            
            // RECORD THE TRANSACTION IN ACCOUNTS
            $this->record_transaction_in_accounts($request, $dr);
            // TRANSACTION RECORDED IN ACCOUNTS

            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($dr->id);
            $this->createEditLog('created', 'diesel_receipts', $dr->id, $dr->dr_number,null, $newVal, $request->user->id,$request->current_year); // CREATE EDIT LOG
            
        
            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json(['id'=>$dr->id, 'dr_number'=>$dr->dr_number], 200); // RETURNED RESPONSE

        } 
        // IF ANY ERROR WHILE SAVING
        catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function update(Request $request, $pk){   
        $test = $this->is_edit_permitted($request->user->id, 'DieselReceipt');
        $test_edit = $test->edit;
        $edit_valid_till_hours = $test->edit_valid_till;

        if( !$test_edit ){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $dr = DieselReceipt::find($pk);

        if( $edit_valid_till_hours!=null ){
            $validity = Carbon::parse($dr->created_at)->addHours($edit_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }

        $validatedData = $request->validate([
            'vehicle' => 'required',
            'dr_on_account' => 'required',
            'filling_station' => 'required',
        ], [
            'vehicle.required' => 'Required',
            'dr_on_account.required' => 'Required',
            'filling_station.required' => 'Required',
        ]);
          

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();

            // FETCHING OLD VALUE FOR LOG
            $oldValue = $this->get_data_for_obj($pk);

            // SAVING A NEW DR
            $dr->against = $request->against;
            $dr->vehicle_id = $request->vehicle['id'];
            $dr->vehicle_number = $request->vehicle['vehicle_number'];
            $dr->booking_id = $request->booking_number['id'];
            $dr->booking_number = $request->booking_number['against_number'];
            $dr->booking_date = $request->booking_date;
            $dr->bill_number = $request->bill_number;
            $dr->dr_on_account = $request->dr_on_account['id'];
            $dr->filling_station = $request->filling_station['id'];
            $dr->slip_number = $request->slip_number;
            $dr->fuel_type = $request->fuel_type;
            $dr->qty = $request->qty;
            $dr->rate = $request->rate;
            $dr->amount = $request->amount;
            $dr->remarks = $request->remarks;

            $dr->updated_by = $request->user->id;

            $dr->save();
            // NEW LS SAVED

            // RECORD THE TRANSACTION IN ACCOUNTS
            $this->record_transaction_in_accounts($request, $dr);
            // TRANSACTION RECORDED IN ACCOUNTS
            
            
            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($dr->id);
            $this->createEditLog('updated', 'diesel_receipts', $dr->id, $dr->dr_number,$oldValue, $newVal, $request->user->id,$request->current_year); // CREATE EDIT LOG
            

            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function destroy(Request $request, $pk){
        $test = $this->is_delete_permitted($request->user->id, 'DieselReceipt');
        $test_delete = $test->delete;
        $delete_valid_till_hours = $test->delete_valid_till;

        if( !$test_delete ){
            return response()->json(['Permission Denied For The Action'],401);
        }

        // FETCHING OLD VALUE FOR LOG
        $oldValue = $this->get_data_for_obj($pk);
        
        $dr = DieselReceipt::find($pk);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();
            

        if( $delete_valid_till_hours!=null ){
            $validity = Carbon::parse($dr->created_at)->addHours($delete_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }

        // DELETE THE TRANSACTION VOUCHER
        $t_data = [];
        $t_data['t_against'] = 'DIESEL RECEIPT';
        $t_data['t_against_no'] = $dr->dr_number;
        $t_data['fin_year'] = $request->current_year;
        $t_data['t_against_id'] = $dr->id;
        $this->delete_transaction_voucher($t_data);

        //FIND AND DELETE THE LS
        $dr->delete();

        $this->createEditLog('deleted', 'diesel_receipts', $dr->id, $dr->dr_number, $oldValue, null, $request->user->id,$request->current_year); // CREATE EDIT LOG

            // TRANSACTIONS COMMITTED
            DB::commit();
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function autocomplete(Request $request){
        // Initialize the query builder
        $query = DieselReceipt::select('id', 'dr_number')->where('fin_year',$request->current_year);

        $user_branch_ids = $request->user_branch_ids;
        if ( $user_branch_ids ) {
            $query->whereIn('branch_id', $user_branch_ids );
        }

       // Apply any additional filters based on other columns as needed
        if ($request->has('filterby')) {
            $orWhereFilters = isset($request->filterby['orWhere']) ? $request->filterby['orWhere'] : [];
            $whereFilters = isset($request->filterby['where']) ? $request->filterby['where'] : [];

        // Loop through the whereFilters array
        foreach ($whereFilters as $whereFilter) {
            // Apply the filter to the query
            $query->where($whereFilter['key'], $whereFilter['value']);
        }

        // Loop through the orWhereFilters array
        foreach ($orWhereFilters as $orWhereFilter) {
            // Apply the filter to the query
            $query->orWhere($orWhereFilter['key'], $orWhereFilter['value']);
        }
        
        }
        
        // Check if searchVariable is provided and apply the filter
        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('dr_number', 'like', '%'. $searchVariable . '%');
        }

        // Limit the number of results
        $query->take($request->limit)->orderBy('id', 'desc');

        // Retrieve the filtered 
        $diesel_receipts = $query->get();
        return response(['diesel_receipts' => $diesel_receipts, 'draw' => $request->draw]);
    }

    public function against_for_diesel_receipt(Request $request){
        // Initialize the query builder
        $against = $request->against;
        $branch_id = $request->branch_id;
        $searchVariable = $request->searchVariable;

        if($against=='GR'){
            $query = BookGr::select('id', 'gr_number as against_number')->where('is_cancelled',false)->where('gr_number', 'like', '%'. $searchVariable . '%');   
        }
        elseif($against == 'Loading Slip'){
            $query = LoadingSlip::select('id', 'ls_number as against_number')->where('ls_number', 'like', '%'. $searchVariable . '%');
        }
        elseif($against == 'Challan'){
            $query = Challan::select('id', 'challan_number as against_number')->where('challan_number', 'like', '%'. $searchVariable . '%');
        }
        // Check if searchVariable is provided and apply the filter
        if ( $branch_id ) {
            $query->where('branch_id', $branch_id);
        }

        // Limit the number of results
        $query->take($request->limit)->orderBy('id', 'desc');

        // Retrieve the filtered 
        $againsts = $query->get();
        return response(['againsts' => $againsts]);
    }

    public function datatable(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'DieselReceipt');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $query = DieselReceipt::with([
                    'branch:id,branch_name,branch_code',
                    'vehicle:id,vehicle_number',
                    'dr_on_account:id,account_name',
                    'filling_station:id,account_name'
                    ])->where('fin_year',$request->current_year) 
                ->orderBy($column, $dir);

        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('dr_number', 'like', $searchValue . '%');
            });
        }

        $data = $query->paginate($length);

        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    public function check_unique(Request $request){
        $key = $request->key;
        $value = $request->value;

        if(!$value){
            return response()->json(['exists' => false]);
        }

        $exists = DieselReceipt::where($key, $value)->exists();
        if($exists){
            return response()->json(['errors' => [ $key  => ['Already Exists !']]],422);
        }

        return response()->json(['exists' => false]);
    }

    public function get_recent(Request $request) {
        $DieselReceiptsQuery = DieselReceipt::select('id', 'dr_number')->where('fin_year',$request->current_year);
    
        if (!empty($request->user_branch_ids)) {
            $DieselReceiptsQuery->whereIn('branch_id', $request->user_branch_ids);
        }
    
        $DieselReceipts = $DieselReceiptsQuery->orderBy('id', 'desc')->take(10)->get();
    
        return response()->json(['diesel_receipts' => $DieselReceipts]);
    }
    


    public function get_next_dr_number(Request $request){
        $branch_id = $request->branch_id;
        $branch_code = $request->branch_code;
        
        $max = DieselReceipt::where('branch_id',$branch_id)->max('max_number');
        $dr_number = $branch_code.'-'.sprintf('%06d', $max+1);
        return response()->json(['dr_number' => strtoupper($dr_number), 'max_number' => $max+1]);
        
    }


    public function get_diesel_receipt_report(Request $request){
        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $report_data = $request->report_data;
                                                                                                                 
        $query = DieselReceipt::with(['branch:id,branch_name,branch_code','vehicle:id,vehicle_number','dr_on_account:id,account_name','filling_station:id,account_name'])->where('fin_year',$request->current_year);

        
        if(isset($report_data['branch']) ){
            $query->where('branch_id', $report_data['branch']['id']);
        }

        if(isset($report_data['dr_num_from']) && isset($report_data['dr_num_to'])){
            $query->where('dr_number','>=',$report_data['dr_num_from']['dr_number'])->where('dr_number','<=',$report_data['dr_num_to']['dr_number']);
        }


        if(isset($report_data['from_date']) && isset($report_data['to_date'])){
            $query->where('booking_date','>=',$report_data['from_date'])->where('booking_date','<=',$report_data['to_date']);
        }

        if(isset($report_data['vehicle']) ){
            $query->where('vehicle_id', $report_data['vehicle']['id']);
        }

        if(isset($report_data['filling_station']) ){
            $query->where('filling_station', $report_data['filling_station']['id']);
        }


        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('dr_number', 'like','%'. $searchValue . '%');
            });
        }

        $data = $query->orderBy($column, $dir)->paginate($length);
          
        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }


    protected function get_data_for_obj($pk){
        // FETCH AND SEND
        $dr = DieselReceipt::with([
                        'branch:id,branch_name,branch_code',
                        'vehicle:id,vehicle_number',
                        'dr_on_account:id,account_name',
                        'filling_station:id,account_name'
            ])->find($pk);

        return ['diesel_receipt' => $dr];
    }

    protected function get_max_number_from_dr_number($input){
        // Extract the integer value using regular expression
        $pattern = '/[-\/](\d+)$/';
        preg_match($pattern, $input, $matches);
        
        if(empty($matches)){
            return intval($input);
        }

        $integerValue = intval(end($matches));
        return $integerValue;
    }

    protected function record_transaction_in_accounts($request, $dr){
        $t_data = [];
        $t_data['t_against'] = 'DIESEL RECEIPT';
        $t_data['t_against_id'] = $dr->id;
        $t_data['t_against_no'] = $dr->dr_number;
        $t_data['fin_year'] = $request->current_year;
        $this->delete_transaction_voucher($t_data);
        
        $t_data['branch'] = $request->branch;
        $t_data['voucher_date'] = $request->booking_date;
        $t_data['voucher_time'] = null;
        $t_data['t_against_narration'] = $dr->remarks;
        $t_data['fin_year'] = $request->current_year;

        $t_data['voucher_accounting_details'] = [];
        
        $t_data['cancellable'] = 1;
        if($dr->dr_on_account && $dr->filling_station && $request->amount>0){
            $t_data['voucher_accounting_details']=[];
            // DEBIT PARTY ACCOUNT
            $t_data['voucher_accounting_details'][] = ['account_id'=>$dr->dr_on_account, 'nature'=>'dr' , 'dr_amount'=>$request->amount, 'cr_amount'=>0 ];
            

            // CREDIT INCOME ACCOUNT
            $t_data['voucher_accounting_details'][] = ['account_id'=>$dr->filling_station, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$request->amount ];

            
            //CHECK TOTAL DR AND CR
            $total_dr = 0;
            $total_cr = 0;

            foreach ($t_data['voucher_accounting_details'] as $vad) {
                $total_dr += $vad['dr_amount'];
                $total_cr += $vad['cr_amount'];
            }

            $total_dr_rounded = round(floatval($total_dr), 2);
            $total_cr_rounded = round(floatval($total_cr), 2);

            // Check if they are not equal
            if ($total_dr_rounded != $total_cr_rounded) {
                // Output the values for debugging
                dd($total_dr_rounded, $total_cr_rounded);
                
                // Return response with an error message
                return response()->json(['message' => 'Error In Accounting Entries'], 422);
            }


            $t_data['total_dr_amount'] = $total_dr;
            $t_data['total_cr_amount'] = $total_cr; 

            $this->create_transaction_voucher( $t_data );
        }

    }
}
