<?php

namespace App\Http\Controllers\HR;

use App\Http\Controllers\Controller;
use App\Http\Traits\EditLogTrait;
use App\Http\Traits\PermissionsTrait;
use App\Models\HR\Attendance;
use App\Models\HR\Holiday;
use App\Models\Master\Employee;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class HolidayController extends Controller
{
    use PermissionsTrait;
    use EditLogTrait;

    public function index(Request $request)
    {
        $test = $this->is_read_permitted($request->user->id, 'Holiday');
        if (!$test) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        // Fetch and send all Holidays
        $holidays = Holiday::select('id','title', 'date')->where('fin_year',$request->current_year)->get();
        return response()->json($holidays);
    }

    public function show(Request $request, $pk)
    {
        // Fetch and send the Holiday
        $holiday = Holiday::find($pk);
        return response()->json(['holiday' => $holiday]);
    }

    public function store(Request $request)
    {
        $test = $this->is_new_permitted($request->user->id, 'Holiday');
        if (!$test) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        $validatedData = $request->validate([
            'title' => 'required|unique:holidays',
            'date' => 'required|unique:holidays',
        ], [
            'title.required' => 'Required',
            'title.unique' => 'Already Exists!',
            'date.required' => 'Required',
            'date.unique' => 'Already Exists!',
        ]);

        try {
            // Transaction begins
            DB::beginTransaction();

            // Save a new Holiday
            $holiday = new Holiday();
            $holiday->title = $request->title;
            $holiday->date = $request->date;
            $holiday->fin_year = $request->current_year;    
            $holiday->created_by = $request->user->id;
            $holiday->save();

            
            // MARK ATTENDANCE FOR ALL EMPLOYEES
            $employees = Employee::select('id')->get();

            foreach ($employees as $employee) {
                $attendance = Attendance::where('emp_id', $employee->id)->where('attend_date', $request->date)->first();
                if ($attendance) {
                    $attendance->attendance = 'H';
                    $attendance->updated_by = $request->user->id;
                    $attendance->save();
                } else {
                    $attendance = new Attendance();
                    $attendance->emp_id = $employee->id;
                    $attendance->attend_date = $request->date;
                    $attendance->attendance = 'H';
                    $attendance->fin_year = $request->current_year;
                    $attendance->created_by = $request->user->id;
                    $attendance->save();
                }
            }


            // Save the edit log
            $newVal = $this->get_data_for_obj($holiday->id);
            $this->createEditLog('created', 'holidays', $holiday->id, $holiday->title, null, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG


            // Transaction committed
            DB::commit();

            
            return response()->json("Success", 200); // Returned response

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function update(Request $request, $pk)
    {
        $test = $this->is_edit_permitted($request->user->id, 'Holiday');
        $test_edit = $test->edit;
        $edit_valid_till_hours = $test->edit_valid_till;

        if (!$test_edit) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        $validatedData = $request->validate([
            'date' => 'required|unique:holidays',
        ], [
            'date.required' => 'Required',
            'date.unique' => 'Already Exists!',
        ]);

        $holiday = Holiday::find($pk);

        // Fetch the old value for log
        $oldValue = $this->get_data_for_obj($pk);

        if ($edit_valid_till_hours != null) {
            $validity = Carbon::parse($holiday->created_at)->addHours($edit_valid_till_hours);
            if (!$validity->isFuture()) {
                return response()->json(['Permission Timed Out'], 500);
            }
        }

        try {
            // Transaction begins
            DB::beginTransaction();


            // MARK ATTENDANCE FOR ALL EMPLOYEES

            // Delete AND UPDATE the previous attendance
            // IF IN $holiday->date is ahead of current date, then delete, OTHERWISE UPDATE AS 'P'
            if( Carbon::parse($holiday->date)->isFuture() ){
                $attendance = Attendance::where('attend_date', $holiday->date)->where('fin_year',$request->current_year)->delete();
            } else {
                $attendance = Attendance::where('attend_date', $holiday->date)->where('fin_year',$request->current_year)->update(['attendance' => 'P']);
            }

            $employees = Employee::select('id')->get();

            foreach ($employees as $employee) {
                $attendance = Attendance::where('emp_id', $employee->id)->where('attend_date', $request->date)->first();
                if ($attendance) {
                    $attendance->attendance = 'H';
                    $attendance->updated_by = $request->user->id;
                    $attendance->save();
                } else {
                    $attendance = new Attendance();
                    $attendance->emp_id = $employee->id;
                    $attendance->attend_date = $request->date;
                    $attendance->attendance = 'H';
                    $attendance->fin_year = $request->current_year;
                    $attendance->created_by = $request->user->id;
                    $attendance->save();
                }
            }

            // ATTENDANCE MARKED



            // Update the Holiday
            $holiday->date = $request->date;
            $holiday->updated_by = $request->user->id;
            $holiday->save();

            


            $newVal = $this->get_data_for_obj($holiday->id);
            $this->createEditLog('updated', 'holidays', $holiday->id, $holiday->title, $oldValue, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG

            // Transaction committed
            DB::commit();

            
            return response()->json("Success", 200); // Returned response

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function destroy(Request $request, $pk)
    {
        $test = $this->is_delete_permitted($request->user->id, 'Holiday');
        $test_delete = $test->delete;
        $delete_valid_till_hours = $test->delete_valid_till;

        if (!$test_delete) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        $holiday = Holiday::find($pk);

        if ($delete_valid_till_hours != null) {
            $validity = Carbon::parse($holiday->created_at)->addHours($delete_valid_till_hours);
            if (!$validity->isFuture()) {
                return response()->json(['Permission Timed Out'], 500);
            }
        }

        // Fetch the old value for log
        $oldValue = $this->get_data_for_obj($pk);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();

            // Delete AND UPDATE the previous attendance
            // IF IN $holiday->date is ahead of current date, then delete, OTHERWISE UPDATE AS 'P'
            if( Carbon::parse($holiday->date)->isFuture() ){
                $attendance = Attendance::where('attend_date', $holiday->date)->where('fin_year',$request->current_year)->delete();
            } else {
                $attendance = Attendance::where('attend_date', $holiday->date)->where('fin_year',$request->current_year)->update(['attendance' => 'P']);
            }


        // Find and delete the Holiday
        $holiday->delete();

        $this->createEditLog('deleted', 'holidays', $holiday->id, $holiday->title, $oldValue, null, $request->user->id, $request->current_year); // CREATE EDIT LOG

            // TRANSACTIONS COMMITTED
            DB::commit();
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function autocomplete(Request $request)
    {
        // Perform search and return matching holidays
        $query = Holiday::select('id', 'title')->where('fin_year',$request->current_year);

        // Apply any additional filters based on other columns as needed
        if ($request->has('filterby')) {
            $orWhereFilters = isset($request->filterby['orWhere']) ? $request->filterby['orWhere'] : [];
            $whereFilters = isset($request->filterby['where']) ? $request->filterby['where'] : [];

           // Loop through the whereFilters array
           foreach ($whereFilters as $whereFilter) {
               // Apply the filter to the query
               $query->where($whereFilter['key'], $whereFilter['value']);
           }

           // Loop through the orWhereFilters array
           foreach ($orWhereFilters as $orWhereFilter) {
               // Apply the filter to the query
               $query->orWhere($orWhereFilter['key'], $orWhereFilter['value']);
           }
        
        }
        
        // Check if searchVariable is provided and apply the filter
        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('title', 'like', '%'. $searchVariable . '%');
        }

        // Limit the number of results
        $query->take($request->limit)->orderBy('title', 'asc');

        // Retrieve the filtered 
        $holidays = $query->get();
        
        return response(['holidays' => $holidays, 'draw' => $request->draw]);
    }

    public function check_unique(Request $request)
    {
        $key = $request->key;
        $value = $request->value;

        if (!$value) {
            return response()->json(['exists' => false]);
        }

        $exists = Holiday::where($key, $value)->exists();
        if ($exists) {
            return response()->json(['errors' => [$key  => ['Already Exists !']]], 422);
        }

        return response()->json(['exists' => false]);
    }

    protected function get_data_for_obj($pk)
    {
        // Fetch and send the holiday
        $holiday = Holiday::find($pk);
        return ['holiday' => $holiday];
    }
}

