<?php

namespace App\Http\Controllers\Inventory;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Http\Traits\CommonTrait;
use App\Http\Traits\EditLogTrait;
use App\Http\Traits\PermissionsTrait;
use App\Models\Inventory\InventoryBalances;
use App\Models\Inventory\Product;
use App\Models\Inventory\PurchaseDesc;
use Illuminate\Support\Facades\DB;

use Carbon\Carbon;

class ProductController extends Controller
{
    use PermissionsTrait;
    use EditLogTrait;
    use CommonTrait;

    public function index(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'ProductDetails');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        //FETCH AND SEND ALL
        $currentYear = $request->current_year; // Assuming $request is available in the surrounding scope

        $product = Product::with([
            'brand:id,brand_name',
            'product_category:id,product_category',
            'linked_account:id,account_name',
            
            'balances' => function($qr) use ($currentYear) {
                $qr->with(['branch:id,branch_name,branch_code'])->select('id','branch_id', 'product_id', 'opening_bal')->where('fin_year', $currentYear);
            }
        ])->get();

        return response()->json($product);
    }

    public function show(Request $request, $pk){
        // FETCH AND SEND
        $product = Product::with(['brand:id,brand_name','product_category:id,product_category','linked_account:id,account_name'])->find($pk);

        $balances = InventoryBalances::with(['branch:id,branch_name,branch_code'])->where('product_id', $pk)->where('fin_year',$request->current_year)->get();

        return response()->json(['product'=>$product, 'balances'=>$balances]);
    }

    public function store(Request $request){
        
        $test = $this->is_new_permitted($request->user->id, 'ProductDetails');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $validatedData = $request->validate([
            'product_name' => 'required|unique:products,product_name'
        ], [
            'product_name.required' => 'Required',
            'product_name.unique' => 'Already Exist !',
        ]);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();  

            // SAVING A NEW STATION
            $product = new Product();
            $product->product_name = $request->product_name;
            
            $branch_id = $this->add_if_not_in_master('Brand', $request->brand, $request->current_year);
            $product->brand_id = $branch_id;

            $product->linked_account_id = $request->linked_account?$request->linked_account['id']:null;

            $product_category_id = $this->add_if_not_in_master('ProductCategory', $request->product_category, $request->current_year);
            $product->product_category_id = $product_category_id ;
            $product->type = $request->type;
            $product->size = $request->size;
            $product->weight = $request->weight;
            $product->remarks = $request->remarks;

            $product->created_by = $request->user->id;
            $product->save(); 
            
            if($product->type=='Product'){
                // SAVING INVENTORY BALANCE
                $inv_desc = $request->inv_desc;
                foreach ($inv_desc as $inv) {
                    
                    $balance = new InventoryBalances();
                    $balance->branch_id = $inv['branch']['id'];
                    $balance->product_id = $product->id;
                    $balance->opening_bal = $inv['opening_bal'];
                    $balance->opening_bal_date = Carbon::now()->format('Y-m-d');
                    $balance->fin_year = $request->current_year;
                    $balance->save();
                }

            }
            


            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($product->id, $request->current_year);
            $this->createEditLog('created', 'products', $product->id, $product->product_name , null, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG
            

            // TRANSACTIONS COMMITTED
            DB::commit();

            return response()->json("Success", 200); // RETURNED RESPONSE

        } 
        // IF ANY ERROR WHILE SAVING
        catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function update(Request $request, $pk){   
        $test = $this->is_edit_permitted($request->user->id, 'ProductDetails');
        $test_edit = $test->edit;
        $edit_valid_till_hours = $test->edit_valid_till;

        if( !$test_edit ){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $product = Product::find($pk);

        // FETCHING OLD VALUE FOR LOG
        $oldValue = $this->get_data_for_obj($pk,$request->current_year);

        if( $edit_valid_till_hours!=null ){
            $validity = Carbon::parse($product->created_at)->addHours($edit_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }
          

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();

            // SAVING A NEW STATION
            $branch_id = $this->add_if_not_in_master('Brand', $request->brand, $request->current_year);
            $product->brand_id = $branch_id;

            $product->linked_account_id = $request->linked_account?$request->linked_account['id']:null;

            $product_category_id = $this->add_if_not_in_master('ProductCategory', $request->product_category, $request->current_year);
            $product->product_category_id = $product_category_id ;
            $product->type = $request->type;
            $product->size = $request->size;
            $product->weight = $request->weight;
            $product->remarks = $request->remarks;
            $product->linked_account_id = $request->linked_account?$request->linked_account['id']:null;

            $product->updated_by = $request->user->id;
            $product->save();
            // NEW STATION SAVED
            
           $del_inv_desc =  InventoryBalances::where('product_id', $product->id)->where('fin_year',$request->current_year)->delete(); 
           
           if($product->type=='Product'){
            // SAVING INVENTORY BALANCE
            $inv_desc = $request->inv_desc;
            foreach ($inv_desc as $inv) {
                
                $balance = new InventoryBalances();
                $balance->branch_id = $inv['branch']['id'];
                $balance->product_id = $product->id;
                $balance->opening_bal = $inv['opening_bal'];
                $balance->opening_bal_date = Carbon::now()->format('Y-m-d');
                $balance->fin_year = $request->current_year;
                $balance->save();
            }
           }

            $newVal = $this->get_data_for_obj($product->id,$request->current_year);
            $this->createEditLog('updated', 'products', $product->id, $product->product_name, $oldValue, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG

            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function destroy(Request $request, $pk){
        $test = $this->is_delete_permitted($request->user->id, 'ProductDetails');
        $test_delete = $test->delete;
        $delete_valid_till_hours = $test->delete_valid_till;

        if( !$test_delete ){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $product = Product::find($pk);

        if( $delete_valid_till_hours!=null ){
            $validity = Carbon::parse($product->created_at)->addHours($delete_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }

        // FETCHING OLD VALUE FOR LOG
        $oldValue = $this->get_data_for_obj($pk,$request->current_year);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();

        // INVENTORY BALANCES
        $inv = InventoryBalances::where('product_id', $product->id)->delete();

        //FIND AND DELETE THE STATION
        $product->delete();

        $this->createEditLog('deleted', 'products', $product->id, $product->product_name, $oldValue, null, $request->user->id, $request->current_year); // CREATE EDIT LOG

            // TRANSACTIONS COMMITTED
            DB::commit();
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function autocomplete(Request $request){
        // WHILE SEARCHING THIS WILL RUN
        // WHILE SEARCHING THIS WILL RUN
        $query = Product::select('id', 'product_name');

        // Apply any additional filters based on other columns as needed
        if ($request->has('filterby')) {
            $orWhereFilters = isset($request->filterby['orWhere']) ? $request->filterby['orWhere'] : [];
            $whereFilters = isset($request->filterby['where']) ? $request->filterby['where'] : [];

           // Loop through the whereFilters array
           foreach ($whereFilters as $whereFilter) {
               // Apply the filter to the query
               $query->where($whereFilter['key'], $whereFilter['value']);
           }

           // Loop through the orWhereFilters array
           foreach ($orWhereFilters as $orWhereFilter) {
               // Apply the filter to the query
               $query->orWhere($orWhereFilter['key'], $orWhereFilter['value']);
           }
        
        }
        
        // Check if searchVariable is provided and apply the filter
        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('product_name', 'like', '%'. $searchVariable . '%');
        }

        // Limit the number of results
        $query->take($request->limit)->orderBy('product_name');

        // Retrieve the filtered 
        $product = $query->get();
        return response(['product' => $product, 'draw' => $request->draw]);
    }

    public function datatable(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'ProductDetails');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $query = Product::select('id', 'product_name', 'brand_id', 'product_category_id')->with(['brand:id,brand_name','product_category:id,product_category'])->orderBy($column, $dir);

        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('product_name', 'like','%'. $searchValue . '%');
            });
        }

        $data = $query->paginate($length);

        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    public function get_inventory_report(Request $request){
        $currentYear = $request->current_year; // Assuming $request is available in the surrounding scope

        $type='Product';

        $test = $this->is_read_permitted($request->user->id, 'ManageInventory');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        $branch_ids = $request->report_data['branch'] ? $request->report_data['branch']['id'] : null;

        $product = Product::with([
            'brand:id,brand_name',
            'product_category:id,product_category',
            'linked_account:id,account_name',
            'balances' => function($qr) use ($currentYear,$branch_ids) {
                $qr->with(['branch:id,branch_name,branch_code'])
                   ->select('id', 'product_id','branch_id', 'opening_bal')
                   ->when($branch_ids, function($qr2) use ($branch_ids) {
                        return $qr2->where('branch_id', $branch_ids);
                    })
                   ->where('fin_year', $currentYear);
            }
        ])->where('type',$type)->get();

      
        $result = [];
         
        
        foreach ($product as $key) {

            $sum_of_opening_bal = 0;
            foreach($key->balances as $bal){
                $sum_of_opening_bal += $bal->opening_bal; 
            }


            $sum_of_qty_in_purchase = PurchaseDesc::where('product_id', $key->id)
                ->where('vehicle_id', null)
                ->whereHas('purchase', function($qr) use ($branch_ids) {
                    $qr->where('bill_name', '!=', 'Stock Issue')
                    ->when($branch_ids, function($qr2) use ($branch_ids) {
                        return $qr2->where('branch_id', $branch_ids);
                    });
                })
                ->where('fin_year', $request->current_year)
                ->sum('qty');


            $sum_of_qty_in_issue = PurchaseDesc::where('product_id', $key->id)
                ->where('vehicle_id', '!=', null)
                ->whereHas('purchase', function($qr) use ($branch_ids) {
                    $qr->where('bill_name', 'Stock Issue')
                       ->when($branch_ids, function($qr2) use ($branch_ids) {
                           return $qr2->where('branch_id', $branch_ids);
                       });
                })
                ->where('fin_year', $request->current_year)
                ->sum('qty');
            
                                
            $data = [
                'product_id' => $key->id,
                'product_name' => $key->product_name,
                'brand_name' => $key->brand->brand_name,
                'product_category' => $key->product_category->product_category,
                'opening_bal' => $sum_of_opening_bal,
                'total_purchase' => $sum_of_qty_in_purchase,
                'total_issue' => $sum_of_qty_in_issue,
                'current_stock' => $sum_of_opening_bal +  $sum_of_qty_in_purchase - $sum_of_qty_in_issue,
            ];
            
            $result[] = $data;
        }

        return response(['data'=>$result]);
    }

    public function update_balance_of_inventory(Request $request){
        $res = $this->update_balance_for_inventory($request->current_year, $request->product_ids);
        
        return response()->json("Success");
    }

    public function update_openings(Request $request){
        $product_ids = Product::where('type','Product')->pluck('id')->toArray();
        $this->update_balance_for_inventory($request->current_year, $product_ids);
        return response()->json("Success");
    }

    public function calculate_balance_for_inventory(Request $request){
        $pk = $request->product_id;
        $branch_id = $request->branch_id;
        $current_year = $request->current_year;

        $min_max_date = $this->get_min_max_date($current_year);
        $min_date = $min_max_date->min_date;
        $from_date = $min_date;

        // Today's date
        $to_date = Carbon::now()->toDateString();

        $balance_details = $this->calculate_balance_for_inventory_stock($pk,$min_date,$from_date,$to_date,$current_year,$branch_id);
        return response()->json(['balance' => $balance_details]);
    }

    public function get_product_account(Request $request,$pk){
        $product = Product::with(['linked_account:id,account_name'])->find($pk);
        return response()->json(['account' => $product->linked_account]);
    }



    public function check_unique(Request $request){
        $key = $request->key;
        $value = $request->value;

        if(!$value){
            return response()->json(['exists' => false]);
        }

        $exists = Product::where($key, $value)->exists();
        if($exists){
            return response()->json(['errors' => [ $key  => ['Already Exists !']]],422);
        }

        return response()->json(['exists' => false]);
    }

    protected function get_data_for_obj($pk,$current_year){
        $product = Product::with(['brand:id,brand_name','product_category:id,product_category','linked_account:id,account_name'])->find($pk);
        $balances = InventoryBalances::with(['branch:id,branch_name,branch_code'])->where('product_id', $pk)->where('fin_year',$current_year)->get();
        
        return ['product'=>$product, 'balances'=>$balances];
    }

}
