<?php

namespace App\Http\Controllers\Items;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Http\Traits\EditLogTrait;
use App\Http\Traits\PermissionsTrait;
use App\Models\Items\Item;
use Illuminate\Support\Facades\DB;

use Carbon\Carbon;

class ItemController extends Controller
{
    use PermissionsTrait;
    use EditLogTrait;

    public function index(Request $request)
    {
        $test = $this->is_read_permitted($request->user->id, 'item');
        if (!$test) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        // FETCH AND SEND ALL
        $items = Item::with(['category:id,category_name'])->all();
        return response()->json($items);
    }

    public function show(Request $request, $pk)
    {
        // FETCH AND SEND
        $item = Item::select('id','category_id as category', 'icon_img','item_name')->find($pk);
        return response()->json(['item' => $item]);
    }

    public function store(Request $request)
    {

        $test = $this->is_new_permitted($request->user->id, 'item');
        if (!$test) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        $validatedData = $request->validate([
            'item_name' => 'required|min:3|unique:items,item_name',
        ], [
            'item_name.required' => 'Required',
            'item_name.unique' => 'Already Exist!',
            'item_name.min' => 'Minimum 3 Characters Required!'
        ]);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();

            // SAVING A NEW Item
            $item = new Item();
            $item->item_name = $request->item_name;
            $item->icon_img = $request->icon_img;
            $item->category_id = $request->category;
            $item->created_by = $request->user->id;
            $item->save();
            // NEW Item SAVED

            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($item->id);
            $this->createEditLog('created', 'items', $item->id, $item->item_name, null, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG


            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json("Success", 200); // RETURNED RESPONSE

        }
        // IF ANY ERROR WHILE SAVING
        catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function update(Request $request, $pk)
    {
        $test = $this->is_edit_permitted($request->user->id, 'item');
        $test_edit = $test->edit;
        $edit_valid_till_hours = $test->edit_valid_till;

        if (!$test_edit) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        $item = Item::find($pk);

        // FETCHING OLD VALUE FOR LOG
        $oldValue = $this->get_data_for_obj($pk);

        if ($edit_valid_till_hours != null) {
            $validity = Carbon::parse($item->created_at)->addHours($edit_valid_till_hours);
            if (!$validity->isFuture()) {
                return response()->json(['Permission Timed Out'], 500);
            }
        }


        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();

            // UPDATING THE Item
            $item->item_name = $request->item_name;
            $item->icon_img = $request->icon_img;
            $item->category_id = $request->category;
            $item->updated_by = $request->user->id;
            $item->save();
            // Item UPDATED


            $newVal = $this->get_data_for_obj($item->id);
            $this->createEditLog('updated', 'items', $item->id, $item->item_name, $oldValue, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG


            // TRANSACTIONS COMMITTED
            DB::commit();

            

            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function destroy(Request $request, $pk)
    {
        $test = $this->is_delete_permitted($request->user->id, 'item');
        $test_delete = $test->delete;
        $delete_valid_till_hours = $test->delete_valid_till;

        if (!$test_delete) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        $item = Item::find($pk);

        if ($delete_valid_till_hours != null) {
            $validity = Carbon::parse($item->created_at)->addHours($delete_valid_till_hours);
            if (!$validity->isFuture()) {
                return response()->json(['Permission Timed Out'], 500);
            }
        }

        // FETCHING OLD VALUE FOR LOG
        $oldValue = $this->get_data_for_obj($pk);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();
            

        // FIND AND DELETE THE Item
        $item->delete();

        $this->createEditLog('deleted', 'items', $item->id, $item->item_name, $oldValue, null, $request->user->id, $request->current_year); // CREATE EDIT LOG

            // TRANSACTIONS COMMITTED
            DB::commit();
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function autocomplete(Request $request)
    {
        // WHILE SEARCHING THIS WILL RUN
        $query = Item::select('id', 'item_name');

        // Apply any additional filters based on other columns as needed
        if ($request->has('filterby')) {
            $orWhereFilters = isset($request->filterby['orWhere']) ? $request->filterby['orWhere'] : [];
            $whereFilters = isset($request->filterby['where']) ? $request->filterby['where'] : [];

           // Loop through the whereFilters array
           foreach ($whereFilters as $whereFilter) {
               // Apply the filter to the query
               $query->where($whereFilter['key'], $whereFilter['value']);
           }

           // Loop through the orWhereFilters array
           foreach ($orWhereFilters as $orWhereFilter) {
               // Apply the filter to the query
               
               $query->orWhere($orWhereFilter['key'], $orWhereFilter['value']);
           }
        
        }
        
        // Check if searchVariable is provided and apply the filter
        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('item_name', 'like', '%'. $searchVariable . '%');
        }

        // Limit the number of results
        $query->take($request->limit)->orderBy('item_name', 'asc');
        // Retrieve the filtered 
        $items = $query->get();
        return response(['items' => $items, 'draw' => $request->draw]);
    }

    public function item_menu(Request $request)
    {
        // WHILE SEARCHING THIS WILL RUN
        $query = Item::select('id', 'item_name','icon_img');

        // Apply any additional filters based on other columns as needed
        if ($request->has('category_id')) {
            $query->where('category_id', $request->category_id);
        }
        
        // Check if searchVariable is provided and apply the filter
        if ($request->has('searchVariable')) {
            $query->where('item_name', 'like', $request->searchVariable . '%');
        }

        // Retrieve the filtered 
        $items = $query->get();

        return response(['items' => $items]);
    }

    public function datatable(Request $request)
    {
        $test = $this->is_read_permitted($request->user->id, 'item');
        if (!$test) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $query = Item::with(['category:id,category_name'])->orderBy($column, $dir);

        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('item_name', 'like', $searchValue . '%');
            });
        }

        $data = $query->paginate($length);

        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    public function check_unique(Request $request)
    {
        $key = $request->key;
        $value = $request->value;

        if (!$value) {
            return response()->json(['exists' => false]);
        }

        $exists = Item::where($key, $value)->exists();
        if ($exists) {
            return response()->json(['errors' => [$key  => ['Already Exists !']]], 422);
        }

        return response()->json(['exists' => false]);
    }

    protected function get_data_for_obj($pk)
    {
        $item = Item::find($pk);
        return ['item' => $item];
    }
}
