<?php

namespace App\Http\Controllers\Leads;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Http\Traits\EditLogTrait;
use App\Http\Traits\PermissionsTrait;
use App\Models\Business\Company;
use App\Models\Leads\FollowUp;
use App\Models\Leads\Lead;
use App\Models\Leads\LeadDesc;
use Illuminate\Support\Facades\DB;

use Carbon\Carbon;

class LeadController extends Controller
{
    use PermissionsTrait;
    use EditLogTrait;

    public function index(Request $request)
    {
        $test = $this->is_read_permitted($request->user->id, 'Lead');
        if (!$test) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        

        // FETCH AND SEND ALL
        $query = Lead::with(['lead_desc:id,lead_id,item_id,qty','lead_desc.item:id,item_name'])->where('fin_year', $request->current_year);
        
        if($request->user->power < 1){
            $query->where('assigned_to_id', $request->user->id)->orWhere('surveyor_id',$request->user->id);
        }

        $leads = $query->all();

        return response()->json($leads);
    }

    public function show(Request $request, $pk)
    {
        // FETCH AND SEND
        $lead = Lead::with(['lead_desc:id,lead_id,item_id,qty','lead_desc.item:id,item_name','follow_ups','follow_ups.follow_up_by:id,username','assigned_to:id,username','surveyor:id,username'])->find($pk);

        // Map lead_desc like { id: 1, item_id: 1, item_name: 'item1', qty: 1 }
        $lead_desc = $lead->lead_desc->map(function ($item) {
            return [
                'lead_desc_id' => $item->id,
                'lead_id' => $item->lead_id,
                'id' => $item->item_id,
                'item_name' => $item->item->item_name,
                'qty' => $item->qty,
            ];
        });

        return response()->json(['lead' => $lead, 'lead_desc' => $lead_desc]);
    }

    public function store(Request $request)
    {

        $test = $this->is_new_permitted($request->user->id, 'Lead');
        if (!$test) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        $validatedData = $request->validate([
            'lead_number' => 'required|unique:leads,lead_number',
            'source' => 'required',
            'destination' => 'required',
            'name' => 'required',
            'mob_number' => 'required',
        ], [
            'lead_number.required' => 'Required',
            'lead_number.unique' => 'Already Exist!',
            'source.required' => 'Required',
            'destination.required' => 'Required',
            'name.required' => 'Required',
            'mob_number.required' => 'Required',
        ]);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();
            
            $max_number = $this->get_max_number_from_lead_number($request->lead_number);

            // SAVING A NEW Lead
            $lead = new Lead();
            $lead->lead_number = $request->lead_number;

            // Get the current date
            $lead->lead_date = Carbon::today();
            // Get the current time
            $lead->lead_time = Carbon::now()->format('H:i:s');

            $lead->max_number = $max_number;
            $lead->name = $request->name;
            $lead->email = $request->email;
            $lead->mob_number = $request->mob_number;
            $lead->mob_number2 = $request->mob_number2;
            $lead->source = $request->source;
            $lead->destination = $request->destination;
            $lead->pickup_date = $request->pickup_date;
            $lead->drop_date = $request->drop_date;
            $lead->pickup_address1 = $request->pickup_address1;
            $lead->pickup_address2 = $request->pickup_address2;
            $lead->pickup_state = $request->pickup_state;
            $lead->pickup_landmark = $request->pickup_landmark;
            $lead->pickup_packing_service = $request->pickup_packing_service;
            $lead->pickup_loading_service = $request->pickup_loading_service;
            $lead->pickup_parking = $request->pickup_parking;
            $lead->pickup_floor = $request->pickup_floor;
            $lead->pickup_lift = $request->pickup_lift;
            $lead->drop_address1 = $request->drop_address1;
            $lead->drop_address2 = $request->drop_address2;
            $lead->drop_state = $request->drop_state;
            $lead->drop_landmark = $request->drop_landmark;
            $lead->drop_unpacking_service = $request->drop_unpacking_service;
            $lead->drop_unloading_service = $request->drop_unloading_service;
            $lead->drop_parking = $request->drop_parking;
            $lead->drop_floor = $request->drop_floor;
            $lead->drop_lift = $request->drop_lift;

            $lead->lead_status = 'Unassigned';

            $lead->fin_year = $request->current_year;

            $lead->created_by = $request->user->id;
            $lead->save();
            // NEW Lead SAVED
            
            $lead_descs = $request->lead_desc;

            foreach ($lead_descs as $l_desc) {
                if (!$l_desc['item_name'] || $l_desc['qty']<1) {
                    continue;
                }
                
                $l_des = new LeadDesc();
                $l_des->lead_id = $lead->id;
                $l_des->item_id = $l_desc['id'];
                $l_des->qty = $l_desc['qty'];

                $l_des->fin_year = $request->current_year;
                $l_des->save();
            }
            

            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($lead->id);
            $this->createEditLog('created', 'leads', $lead->id, $lead->lead_number, null, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG


            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json("Success", 200); // RETURNED RESPONSE

        }
        // IF ANY ERROR WHILE SAVING
        catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function update(Request $request, $pk)
    {
        $test = $this->is_edit_permitted($request->user->id, 'Lead');
        $test_edit = $test->edit;
        $edit_valid_till_hours = $test->edit_valid_till;

        if (!$test_edit) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        $lead = Lead::find($pk);

        if($request->user->power < 1){
            if($lead->assigned_to_id != $request->user->id && $lead->surveyor_id != $request->user->id){
                return response()->json(['This lead is not assigned to the user'], 500);
            }
        }

        // FETCHING OLD VALUE FOR LOG
        $oldValue = $this->get_data_for_obj($pk);

        if ($edit_valid_till_hours != null) {
            $validity = Carbon::parse($lead->created_at)->addHours($edit_valid_till_hours);
            if (!$validity->isFuture()) {
                return response()->json(['Permission Timed Out'], 500);
            }
        }


        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();

            // UPDATING THE Lead
            $lead->name = $request->name;
            $lead->email = $request->email;
            $lead->mob_number = $request->mob_number;
            $lead->mob_number2 = $request->mob_number2;
            $lead->source = $request->source;
            $lead->destination = $request->destination;
            $lead->pickup_date = $request->pickup_date;
            $lead->drop_date = $request->drop_date;
            $lead->pickup_address1 = $request->pickup_address1;
            $lead->pickup_address2 = $request->pickup_address2;
            $lead->pickup_state = $request->pickup_state;
            $lead->pickup_landmark = $request->pickup_landmark;
            $lead->pickup_packing_service = $request->pickup_packing_service;
            $lead->pickup_loading_service = $request->pickup_loading_service;
            $lead->pickup_parking = $request->pickup_parking;
            $lead->pickup_floor = $request->pickup_floor;
            $lead->pickup_lift = $request->pickup_lift;
            $lead->drop_address1 = $request->drop_address1;
            $lead->drop_address2 = $request->drop_address2;
            $lead->drop_state = $request->drop_state;
            $lead->drop_landmark = $request->drop_landmark;
            $lead->drop_unpacking_service = $request->drop_unpacking_service;
            $lead->drop_unloading_service = $request->drop_unloading_service;
            $lead->drop_parking = $request->drop_parking;
            $lead->drop_floor = $request->drop_floor;
            $lead->drop_lift = $request->drop_lift;

            $lead->assigned_to_id = $request->assigned_to ? $request->assigned_to['id'] : null;
            $lead->surveyor_id = $request->surveyor ? $request->surveyor['id'] : null;

            $lead->updated_by = $request->user->id;
            $lead->save();
            // Lead UPDATED


            $lead_desc_delete = LeadDesc::where('lead_id',$pk)->delete();


            $lead_descs = $request->lead_desc;

            foreach ($lead_descs as $l_desc) {
                if (!$l_desc['item_name'] || $l_desc['qty']<1) {
                    continue;
                }
                
                $l_des = new LeadDesc();
                $l_des->lead_id = $lead->id;
                $l_des->item_id = $l_desc['id'];
                $l_des->qty = $l_desc['qty'];
                $l_des->fin_year = $request->current_year;
                $l_des->save();
            }


            // SAVING NEWLY ADDED FOLLOW UPS
            $newly_added_follow_ups = $request->newly_added_follow_ups;
            foreach ($newly_added_follow_ups as $nafu) {
                if(!$nafu['status'])
                continue;
                
                $fu = new FollowUp();
                $fu->lead_id = $lead->id;
                $fu->status = $nafu['status'];
                $fu->date = $nafu['date'];
                $fu->time = $nafu['time'];
                $fu->remarks = $nafu['remarks'];
                $fu->follow_up_by_id = $request->user->id;
                $fu->against = 'Lead';
                $fu->fin_year = $request->current_year;
                $fu->save();
            }
            // NEWLY ADDED FOLLOW UPS SAVED


            // SAVING UPDATES ON  FollowUPS
            $updated_follow_up_details = $request->updated_follow_up_details;
            if( !empty($updated_follow_up_details) ){
                foreach ($updated_follow_up_details as $ufud) {
                    if(!$ufud['status'])
                       continue;

                    $fu = FollowUp::find($ufud['id']);
                    $fu->status = $ufud['status'];
                    $fu->date = $ufud['date'];
                    $fu->time = $ufud['time'];
                    $fu->remarks = $ufud['remarks'];
                    $fu->save();

                }

            }
            // UPDATES SAVED ON FollowUPS

            // DELETING EXISTING Follow UPS
            $deleted_follow_up_details = $request->deleted_follow_up_details;
            if( !empty($deleted_follow_up_details) ){
                    $fu = FollowUp::whereIn('id', $deleted_follow_up_details);
                    $fu->delete();
            }
            //DELETED EXISTING Follow UPS

            $latest_status = FollowUp::select('status')->where('lead_id',$pk)->orderBy('id','desc')->first();
            if($latest_status){
                $lead->lead_status = $latest_status->status;
                $lead->save();
            }
            

            $newVal = $this->get_data_for_obj($lead->id);
            $this->createEditLog('updated', 'leads', $lead->id, $lead->lead_number, $oldValue, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG


            // TRANSACTIONS COMMITTED
            DB::commit();

            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function destroy(Request $request, $pk)
    {
        $test = $this->is_delete_permitted($request->user->id, 'Lead');
        $test_delete = $test->delete;
        $delete_valid_till_hours = $test->delete_valid_till;

        if (!$test_delete) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        $lead = Lead::find($pk);

        if($request->user->power < 1){
            if($lead->assigned_to_id != $request->user->id){
                return response()->json(['This lead is not assigned to the user'], 500);
            }
        }

        if ($delete_valid_till_hours != null) {
            $validity = Carbon::parse($lead->created_at)->addHours($delete_valid_till_hours);
            if (!$validity->isFuture()) {
                return response()->json(['Permission Timed Out'], 500);
            }
        }

        // FETCHING OLD VALUE FOR LOG
        $oldValue = $this->get_data_for_obj($pk);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();
            

        $lead_desc_delete = LeadDesc::where('lead_id',$pk)->delete();
        $follow_up_delete = FollowUp::where('lead_id',$pk)->delete();

        // FIND AND DELETE THE Lead
        $lead->delete();

        $this->createEditLog('deleted', 'leads', $lead->id, $lead->lead_number, $oldValue, null, $request->user->id, $request->current_year); // CREATE EDIT LOG

            // TRANSACTIONS COMMITTED
            DB::commit();
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function autocomplete(Request $request)
    {
        // WHILE SEARCHING THIS WILL RUN
        $query = Lead::select('id', 'lead_number')->where('fin_year', $request->current_year);

        if($request->user->power < 1){
            $query->where('assigned_to_id', $request->user->id)->orWhere('surveyor_id',$request->user->id);
        }
        

        // Apply any additional filters based on other columns as needed
        if ($request->has('filterby')) {
            $orWhereFilters = isset($request->filterby['orWhere']) ? $request->filterby['orWhere'] : [];
            $whereFilters = isset($request->filterby['where']) ? $request->filterby['where'] : [];

           // Loop through the whereFilters array
           foreach ($whereFilters as $whereFilter) {
               // Apply the filter to the query
               $query->where($whereFilter['key'], $whereFilter['value']);
           }

           // Loop through the orWhereFilters array
           foreach ($orWhereFilters as $orWhereFilter) {
               // Apply the filter to the query
               $query->orWhere($orWhereFilter['key'], $orWhereFilter['value']);
           }
        
        }
        
        // Check if searchVariable is provided and apply the filter
        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('lead_number', 'like', '%'. $searchVariable . '%')->orWhere('mob_number', 'like', '%'. $searchVariable . '%');
        }

        // Limit the number of results
        $query->take($request->limit)->orderBy('lead_number', 'asc');
        // Retrieve the filtered 
        $leads = $query->get();
        return response(['leads' => $leads, 'draw' => $request->draw]);
    }

    public function lead_for_quotation(Request $request) {
        $leads = Lead::select('id', 'lead_number')
            ->whereDoesntHave('quotations')
            ->when($request->user->power < 1, function ($query) use ($request) {
                return $query->where(function ($subQuery) use ($request) {
                    $subQuery->where('assigned_to_id', $request->user->id)
                        ->orWhere('surveyor_id', $request->user->id);
                });
            })
            ->get();
    
        return response()->json(['leads' => $leads]);
    }
    

    public function datatable(Request $request)
    {
        $test = $this->is_read_permitted($request->user->id, 'Lead');
        if (!$test) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;
        $status = $request->lead_status;
        $type = $request->type;

        $query = Lead::select('id','lead_number','source','destination','name','mob_number','pickup_address1','drop_address1','lead_date','lead_time','lead_status','assigned_to_id','surveyor_id', 'pickup_state','drop_state')->with(['lead_desc:id,lead_id,item_id,qty','lead_desc.item:id,item_name','assigned_to:id,username','surveyor:id,username','quotations:id,quotation_number,lead_id'])->where('fin_year', $request->current_year)->orderBy($column, $dir);

        if($request->user->power < 1){
            $query->where('assigned_to_id', $request->user->id)->orWhere('surveyor_id',$request->user->id);
        }

        if ($type) {
            if ($type == 'Local') {
                $query->whereColumn('pickup_state', '=', 'drop_state');
            } elseif ($type == 'Intercity') {
                $query->whereColumn('pickup_state', '!=', 'drop_state');
            }
        }

        if($status){
            $query->where(function ($query) use ($status) {
                $query->where('lead_status', $status);
            });
        }
        
        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('lead_number', 'like', $searchValue . '%')->orWhere('mob_number', 'like', $searchValue . '%');
            });
        }



        $data = $query->paginate($length);

        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    public function get_lead_info_for_quotation($pk){
        $lead = Lead::with(['lead_desc:id,lead_id,item_id,qty','lead_desc.item:id,item_name,category_id','lead_desc.item.category:id,category_name'])->find($pk);
        
        $lead_desc = $lead->lead_desc->map(function ($item) {
            return [
                'lead_desc_id' => $item->id,
                'lead_id' => $item->lead_id,
                'quotation_id' => null,
                'category_name' => $item->item->category->category_name,
                'id' => $item->item_id,
                'item_name' => $item->item->item_name,
                'qty' => $item->qty,
            ];
        });

        return response()->json(['lead' => $lead, 'lead_desc' => $lead_desc]);
    }

    public function get_next_lead_number(Request $request){
        $type='SYSTEM';

        if($type=='SYSTEM'){
            $company_code = Company::select('company_code')->find($request->company['id'])['company_code'];

            $max = Lead::max('max_number');
            //$lead_number = sprintf('%06d', $max+1);
            $lead_number = $company_code.'-'.sprintf('%06d',$max+1);
            
            return response()->json(['lead_number' => strtoupper($lead_number), 'max_number' => $max+1]);
        }
    }

    public function assign_user_to_lead(Request $request){
        if($request->user->power < 1){
            return response()->json(['Only Admin Can Assign The Lead'], 500);
        }

        $lead = Lead::find($request->lead_id);
        $status = $request->user_id ? 'Yet To Be Contacted' : 'Unassigned';
        $lead->assigned_to_id = $request->user_id;
        $lead->lead_status = $status;
        $lead->save();
    }

    public function assign_surveyor_to_lead(Request $request){
        $lead = Lead::find($request->lead_id);

        if($request->user->power < 1 && $lead->assigned_to_id!=$request->user->id){
            return response()->json(['Only Admin Or Assigned User Can Set A Surveyor'], 500);
        }

        $lead->surveyor_id = $request->user_id;
        $lead->save();
    }

    public function check_unique(Request $request)
    {
        $key = $request->key;
        $value = $request->value;

        if (!$value) {
            return response()->json(['exists' => false]);
        }

        $exists = Lead::where($key, $value)->exists();
        if ($exists) {
            return response()->json(['errors' => [$key  => ['Already Exists !']]], 422);
        }

        return response()->json(['exists' => false]);
    }

    protected function get_data_for_obj($pk)
    {
        $lead = Lead::with(['lead_desc:id,lead_id,item_id,qty','lead_desc.item:id,item_name','follow_ups','follow_ups.follow_up_by:id,username','assigned_to:id,username','surveyor:id,username'])->find($pk);
        return ['lead' => $lead];
    }

    protected function get_max_number_from_lead_number($input){
        // Extract the integer value using regular expression
        $pattern = '/[-\/](\d+)$/';
        preg_match($pattern, $input, $matches);
        
        if(empty($matches)){
            return intval($input);
        }

        $integerValue = intval(end($matches));
        return $integerValue;
    }
}
