<?php

namespace App\Http\Controllers\Master;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Http\Traits\EditLogTrait;
use App\Http\Traits\PermissionsTrait;
use App\Models\Account\Account;
use App\Models\Account\Balances;
use App\Models\Account\Group;
use App\Models\Master\Branch;
use App\Models\Master\BranchCommission;
use Illuminate\Support\Facades\DB;

use Carbon\Carbon;

class BranchController extends Controller
{
    use PermissionsTrait;
    use EditLogTrait;

    public function index(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'BranchDetails');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        //FETCH AND SEND ALL
        $branches = Branch::with(['commissions', 'branch_station'=> function ($query) {
                            $query->select('id', 'station_name', 'branch_code');
                    }])->get();
        return response()->json($branches);
    }

    public function show(Request $request, $pk){
        // FETCH AND SEND
        $branch = Branch::with(['commissions',
                   'branch_station'=> function ($query) {
                        $query->select('id', 'station_name');
                  }])->find($pk);
        $balance = Balances::where('account_id', $branch->account_id)->where('fin_year',$request->current_year)->first();
        return response()->json(['branch'=>$branch, 'balance'=>$balance]);
    }

    public function store(Request $request){
        
        $test = $this->is_new_permitted($request->user->id, 'BranchDetails');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $validatedData = $request->validate([
            'branch_name' => 'required|unique:branches,branch_name',
            'branch_code' => 'required|unique:branches,branch_code',
        ], [
            'branch_name.required' => 'Required',
            'branch_code.required' => 'Required',
            'branch_name.unique' => 'Already Exist !',
            'branch_code.unique' => 'Already Exist !',
        ]);

        if (!isFalsy($request->opening_bal_dr) && !isFalsy($request->opening_bal_cr)) {
            return response()->json("Please Provide A Single Value For Opening Balances", 500);
        }

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();  

            // SAVING A NEW BRANCH
            $branch = new Branch;
            $branch->branch_name = $request->branch_name ;
            $branch->branch_code = $request->branch_code ;
            $branch->branch_type = $request->branch_type ;
            $branch->is_agency = $request->is_agency ;
            $branch->contact_person = $request->contact_person ;
            $branch->tele_number = $request->tele_number ;
            $branch->mob_number = $request->mob_number ;
            $branch->email = $request->email ;
            $branch->gst_number = $request->gst_number ;
            $branch->pan_number = $request->pan_number ;
            $branch->pan_number = $request->pan_number ;

            if($request->branch_station){
                $branch->branch_station = $request->branch_station['id'] ;
            }
            else{
                $branch->branch_station = null ;
            }
            
            $branch->address1 = $request->address1 ;
            $branch->address2 = $request->address2 ;
            $branch->city = $request->city ;
            $branch->state = $request->state ;
            $branch->pincode = $request->pincode ;
            $branch->printer_type = $request->printer_type ;
            

            $is_agency = $request->is_agency;
            $gp = $is_agency ? 'Sundry Creditors' : 'Branch / Divisions';
            $grp = Group::where('group_name', $gp)->first();
            
            $new_account = new Account();
            $new_account->group_id = $grp->id ;
            $new_account->account_name = $branch->branch_name;
            $new_account->account_type = 'Branch';
            $new_account->created_by = $request->user->id;
            $new_account->save();

            $bal = new Balances();
            $bal->account_id = $new_account->id;
            $bal->opening_bal_dr = $request->opening_bal_dr;
            $bal->opening_bal_cr = $request->opening_bal_cr;
            $bal->opening_bal_date = Carbon::now();
            $bal->fin_year = $request->current_year;
            $bal->save();


            $branch->account_id = $new_account->id;
            $branch->created_by = $request->user->id;
            $branch->save(); 
            // NEW BRANCH SAVED

            // SAVING COMMISSIONS
            $commissionData = $request->commissions;
            foreach ($commissionData as $cd) {
                if(!$cd['comm_rate'] || !$cd['comm_type'])
                continue;
                
                $commission = new BranchCommission();
                $commission->branch_id = $branch->id;
                $commission->comm_type  = $cd['comm_type'];
                $commission->comm_unit = $cd['comm_unit'];
                $commission->comm_qty = $cd['comm_qty'];
                $commission->comm_rate = $cd['comm_rate'];
                $commission->dynamic_options = json_encode($cd['dynamic_options']);
                $commission->active = $cd['active'];
                $commission->save();
            }
            // COMMISSIONS SAVED

            $newVal = $this->get_data_for_obj($branch->id,$request->current_year);
            $this->createEditLog('created', 'branches', $branch->id, $branch->branch_name , null, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG
            
            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json("Success", 200); // RETURNED RESPONSE

        } 
        // IF ANY ERROR WHILE SAVING
        catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function update(Request $request, $pk){   
        $test = $this->is_edit_permitted($request->user->id, 'BranchDetails');
        $test_edit = $test->edit;
        $edit_valid_till_hours = $test->edit_valid_till;

        if( !$test_edit ){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $branch = Branch::find($pk);

        if( $edit_valid_till_hours!=null ){
            $validity = Carbon::parse($branch->created_at)->addHours($edit_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }

        if (!isFalsy($request->opening_bal_dr) && !isFalsy($request->opening_bal_cr)) {
            return response()->json("Please Provide A Single Value For Opening Balances", 500);
        }
          

        try {
            // FETCH OLD VALUE FOR LOG
            $oldValue = $this->get_data_for_obj($pk, $request->current_year);

            // TRANSACTION BEGINS
            DB::beginTransaction();

            // SAVING A NEW BRANCH
            $branch->branch_type = $request->branch_type ;
            $branch->is_agency = $request->is_agency ;
            $branch->contact_person = $request->contact_person ;
            $branch->tele_number = $request->tele_number ;
            $branch->mob_number = $request->mob_number ;
            $branch->email = $request->email ;
            $branch->gst_number = $request->gst_number ;
            $branch->pan_number = $request->pan_number ;

            if($request->branch_station){
                $branch->branch_station = $request->branch_station['id'] ;
            }
            else{
                $branch->branch_station = null ;
            }

            $branch->address1 = $request->address1 ;
            $branch->address2 = $request->address2 ;
            $branch->city = $request->city ;
            $branch->state = $request->state ;
            $branch->pincode = $request->pincode ;
            $branch->printer_type = $request->printer_type ;
            $branch->updated_by = $request->user->id;
            $branch->save();
            // NEW BRANCH SAVED
            
            // SAVING NEWLY ADDED COMMISSIONS
            $newly_added_commissions = $request->newly_added_commissions;
            if( !empty($newly_added_commissions) ){
                foreach ($newly_added_commissions as $nac) {
                    if(!$nac['comm_rate'] || !$nac['comm_type'])
                       continue;

                        $commission = new BranchCommission();
                        $commission->branch_id = $branch->id;
                        $commission->comm_type  = $nac['comm_type'];
                        $commission->comm_unit = $nac['comm_unit'];
                        $commission->comm_qty = $nac['comm_qty'];
                        $commission->comm_rate = $nac['comm_rate'];
                        $commission->dynamic_options = json_encode($nac['dynamic_options']);
                        $commission->active = $nac['active'];
                        $commission->save();
                }
            }
            // NEWLY ADDED COMMISSIONS SAVED

            // SAVING UPDATES ON EXISTING COMMISSIONS
            $updated_commissions = $request->updated_commissions;
            if( !empty($updated_commissions) ){
                foreach ($updated_commissions as $uc) {
                    if(!$uc['comm_rate'] || !$uc['comm_type'])
                       continue;
                       
                        $commission = BranchCommission::find( $uc['id'] );
                        $commission->branch_id = $branch->id;
                        $commission->comm_type  = $uc['comm_type'];
                        $commission->comm_unit = $uc['comm_unit'];
                        $commission->comm_qty = $uc['comm_qty'];
                        $commission->comm_rate = $uc['comm_rate'];
                        $commission->dynamic_options = json_encode($uc['dynamic_options']);
                        $commission->active = $uc['active'];
                        $commission->save();
                }
            }
            // UPDATES SAVED

            // DELETING EXISTING COMMISSION
            $deleted_commissions = $request->deleted_commissions;
            if( !empty($deleted_commissions) ){
                    $commission = BranchCommission::whereIn('id', $deleted_commissions);
                    $commission->delete();
            }
            //DELETED EXISTING COMMISSION

            
            $is_agency = $request->is_agency;
            $gp = $is_agency ? 'Sundry Creditors' : 'Branch / Divisions';
            $grp = Group::where('group_name', $gp)->first();

            $account = Account::find($branch->account_id);
            $account->group_id = $grp->id;
            $account->updated_by = $request->user->id;
            $account->save();
            
            $bal = Balances::where('account_id', $account->id)->where('fin_year', $request->current_year)->first();
            $bal->opening_bal_dr = $request->opening_bal_dr;
            $bal->opening_bal_cr = $request->opening_bal_cr;
            $bal->save();

            $newVal = $this->get_data_for_obj($branch->id, $request->current_year);
            $this->createEditLog('updated', 'branches', $branch->id, $branch->branch_name, $oldValue, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG
            

            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function destroy(Request $request, $pk){
        $test = $this->is_delete_permitted($request->user->id, 'BranchDetails');
        $test_delete = $test->delete;
        $delete_valid_till_hours = $test->delete_valid_till;

        if( !$test_delete ){
            return response()->json(['Permission Denied For The Action'],401);
        }

        
        $branch = Branch::find($pk);
        
        // FETCH OLD VALUE FOR LOG
        $oldValue = $this->get_data_for_obj($pk, $request->current_year);

        if( $delete_valid_till_hours!=null ){
            $validity = Carbon::parse($branch->created_at)->addHours($delete_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();
            
            
        //FIND AND DELETE THE BRANCH
        $branch->delete();

        $account = Account::where('id',$branch->account_id)->first();
            $balances = Balances::where('account_id', $account->id)->delete();
            $account->delete();
        // COMMISSIONS WILL BE CASCADED AUTOMATICALLY

        $this->createEditLog('deleted', 'branches', $branch->id,  $branch->branch_name,$oldValue, null, $request->user->id, $request->current_year); // CREATE EDIT LOG

            // TRANSACTIONS COMMITTED
            DB::commit();
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function autocomplete(Request $request){
        // Initialize the query builder
        $fields = ['id', 'branch_name', 'branch_code'];
        
        if ($request->fields) {
            $fields = array_merge($fields, $request->fields);
        }

        // WHILE SEARCHING THIS WILL RUN
        $query = Branch::select($fields);

        if(!$request->show_all){
            $user_branch_ids = $request->user_branch_ids;
            if ( $user_branch_ids ) {
                $query->whereIn('id', $user_branch_ids );
            }

        }
        

        // Apply any additional filters based on other columns as needed
        if ($request->has('filterby')) {
            $orWhereFilters = isset($request->filterby['orWhere']) ? $request->filterby['orWhere'] : [];
            $whereFilters = isset($request->filterby['where']) ? $request->filterby['where'] : [];

           // Loop through the whereFilters array
           foreach ($whereFilters as $whereFilter) {
               // Apply the filter to the query
               $query->where($whereFilter['key'], $whereFilter['value']);
           }

           // Loop through the orWhereFilters array
           foreach ($orWhereFilters as $orWhereFilter) {
               // Apply the filter to the query
               $query->orWhere($orWhereFilter['key'], $orWhereFilter['value']);
           }
        
        }
        
        // Check if searchVariable is provided and apply the filter
        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('branch_name', 'like', '%'. $searchVariable . '%');
        }

        // Limit the number of results
        $query->take($request->limit)->orderBy('branch_name', 'asc');

        // Retrieve the filtered 
        $branch = $query->get();

        return response(['branch' => $branch, 'draw' => $request->draw]);
    }

    public function datatable(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'BranchDetails');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $query = Branch::select('id', 'branch_name', 'branch_code', 'contact_person', 'address1', 'city', 'tele_number')->orderBy($column, $dir);

        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('branch_name', 'like', $searchValue . '%')
                    ->orWhere('branch_code', 'like', $searchValue . '%')
                    ->orWhere('tele_number', 'like', $searchValue . '%');
            });
        }

        $user_branch_ids = $request->user_branch_ids;
        if ( $user_branch_ids ) {
            $query->whereIn('id', $user_branch_ids );
        }

        $data = $query->paginate($length);

        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    public function get_branch_station($pk){
        $branch = Branch::find($pk);
        $station = $branch->branch_station()->select('id', 'station_name')->first();
        return response()->json(['branch_station'=>$station]);
    }

    public function delivery_commission_details($pk){
        $branch_id = Branch::where('account_id', $pk)->first()['id'];
        $branch_comm = BranchCommission::where('branch_id', $branch_id)->where('active',true)->where('comm_type','Delivery')->first();
        return response()->json(['commission'=>$branch_comm]);
    }

    public function get_custom_fields(Request $request){
        // Get only the fields present in $request->fields
        $fields = $request->fields;
        
        $query = Branch::select($fields);

        if($request->has('with_relations'))
        {
            $with_relations = $request->with_relations;
            $query->with($with_relations);
        }
          
        
        $res = $query->find($request->branch_id);
        return response()->json(['branch' => $res]);
    }

    public function check_unique(Request $request){
        $key = $request->key;
        $value = $request->value;

        if(!$value){
            return response()->json(['exists' => false]);
        }

        $exists = Branch::where($key, $value)->exists();
        if($exists){
            return response()->json(['errors' => [ $key  => ['Already Exists !']]],422);
        }

        return response()->json(['exists' => false]);
    }

    public function get_branch_report(Request $request){
        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $report_data = $request->report_data;

        $query = Branch::select('id','branch_name','branch_code','branch_type','is_agency','contact_person','tele_number','address1','city','email','gst_number');

        if(isset($report_data['branch'])){
            $query->where('id', $report_data['branch']['id']);
        }

        if($report_data['is_agency']!='Both'){
            $query->where('is_agency', $report_data['is_agency']);
        }

        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('branch_name', 'like', $searchValue . '%')
                    ->orWhere('branch_code', 'like', $searchValue . '%')
                    ->orWhere('tele_number', 'like', $searchValue . '%');
            });
        }

        $user_branch_ids = $request->user_branch_ids;
        if ( $user_branch_ids ) {
            $query->whereIn('id', $user_branch_ids );
        }


        $data = $query->orderBy($column, $dir)->paginate($length);
          
        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    protected function get_data_for_obj($pk, $current_year){
        $branch = Branch::with('commissions')->find($pk);
        $balance = Balances::where('account_id', $branch->account_id)->where('fin_year', $current_year)->first();
        return ['branch'=>$branch, 'balance'=>$balance];
    }
}
