<?php

namespace App\Http\Controllers\Master;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Http\Traits\CommonTrait;
use App\Http\Traits\EditLogTrait;
use App\Http\Traits\PermissionsTrait;
use App\Models\Account\Account;
use App\Models\Account\Balances;
use App\Models\Account\Group;
use App\Models\Business\Company;
use App\Models\FleetOwner\LoadingSlip;
use App\Models\HR\Attendance;
use App\Models\Master\Employee;
use App\Models\Master\EmployeeVehicle;
use App\Models\Master\Vehicle;
use Illuminate\Support\Facades\DB;

use Carbon\Carbon;

class EmployeeController extends Controller
{
    use PermissionsTrait;
    use EditLogTrait;
    use CommonTrait;

    public function index(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'EmployeeDetails');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        //FETCH AND SEND ALL
        $employee = Employee::with(['branch' => function ($query) {
            $query->select('id', 'branch_name','branch_code');
        }])->get();

        return response()->json(['employee'=>$employee]);
    }

    public function show(Request $request, $pk){
        // FETCH AND SEND
        $employee = Employee::with(['branch' => function ($query) {
            $query->select('id', 'branch_name','branch_code');
         },
         'vehicles'=> function ($query) {
            $query->wherePivot('active', true)
            ->select('vehicles.id', 'vehicles.vehicle_number', 'vehicles.vehicle_type')->withPivot('id', 'auth_date');
         },
         
         ])->find($pk);

        // Count Casual, Paid, Sick Leaves

        $minMaxDate = $this->get_min_max_date($request->current_year);

        $attendances = Attendance::where('emp_id', $employee->id)
            ->where('attend_date', '>=', $minMaxDate->min_date)
            ->where('attend_date', '<=', $minMaxDate->max_date)
            ->whereIn('attendance', ['CL', 'PL', 'SL'])
            ->where('fin_year', $request->current_year)
            ->select('attendance', DB::raw('count(*) as count'))
            ->groupBy('attendance')
            ->get();
        
        $data = $employee->toArray();
        
        foreach ($attendances as $attendance) {
            $data[strtolower($attendance->attendance)] = $attendance->count;
        }
        
        // If you want to ensure that keys 'cl', 'pl', and 'sl' are set even if there are no records for them, you can add the following lines:
        $data['cl'] = $data['cl'] ?? 0;
        $data['pl'] = $data['pl'] ?? 0;
        $data['sl'] = $data['sl'] ?? 0;
        

        $data['vehicles'] = $employee->vehicles->map(function ($vehicle) {
            return [
                'id' => $vehicle->pivot->id,
                'auth_date' => $vehicle->pivot->auth_date,
                'vehicle' => [
                    'id' => $vehicle->id,
                    'vehicle_number' => $vehicle->vehicle_number,
                    'vehicle_type' => $vehicle->vehicle_type,
                ],
            ];
        });
         
        $balance = Balances::where('account_id', $employee->account_id)->where('fin_year',$request->current_year)->first();

        return response()->json(['employee'=>$data, 'balance'=>$balance]);
    }

    public function store(Request $request){
        
        $test = $this->is_new_permitted($request->user->id, 'EmployeeDetails');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $validatedData = $request->validate([
            'emp_id'=>'unique:employees,emp_id',
            'emp_name' => 'required|min:3|unique:employees,emp_name'
        ], [
            'emp_name.required' => 'Required',
            'emp_id.unique' => 'Already Exist !',
            'emp_name.min' => 'Minimum 3 Characters Required !'
        ]);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();  

            $max_number = $this->get_max_number_from_emp_id($request->emp_id);

            // SAVING A NEW Employee
            $employee = new Employee();
            // Set the values from the request to the employee object
            $employee->emp_id = $request->emp_id;
            $employee->emp_name = $request->emp_name;
            $employee->category = $request->category;

            $employee->max_number = $max_number;

            if($request->branch){
                $employee->branch_id = $request->branch['id'];
            }
            else{
                $employee->branch_id = null;
            }

            $employee->designation = $request->designation;
            $employee->dob = $request->dob;
            $employee->joining_date = $request->joining_date;
            $employee->resignation_date = $request->resignation_date;
            $employee->address = $request->address;
            $employee->contact_number = $request->contact_number;
            $employee->father_name = $request->father_name;
            $employee->father_contact = $request->father_contact;
            $employee->guarantor_name = $request->guarantor_name;
            $employee->guarantor_contact = $request->guarantor_contact;
            $employee->pan_number = $request->pan_number;
            
            $res =  $this->check_for_blacklisted_employee($request->aadhaar_number, $request->pan_number, $request->contact_number, $request->license_number);
            if($res){
             return response()->json(['error'=>'Black Listed', 'emp_name'=>$res->emp_name],500);
            }

            $employee->aadhaar_number = $request->aadhaar_number;
            $employee->pf_number = $request->pf_number;
            $employee->esi_number = $request->esi_number;

            $employee->black_list = false;

            if($request->resignation_date){
                $employee->status = 'Inactive';
            }else{
                $employee->status = 'Active'; 
            }

            
            $employee->employee_pic = $request->employee_pic;
            $employee->employee_addr_proof_pic = $request->employee_addr_proof_pic;
            $employee->employee_aadhar_pic = $request->employee_aadhar_pic;
            
            
            $employee->account_number = $request->account_number;
            $employee->holder_name = $request->holder_name;
            $employee->bank_name = $request->bank_name;
            $employee->bank_branch = $request->bank_branch;
            $employee->ifsc = $request->ifsc;

            $employee->casual_leaves = $request->casual_leaves;
            $employee->earning_leaves = $request->earning_leaves;
            $employee->sick_leaves = $request->sick_leaves;

            if($request->payment_detail_edited){
                $employee->fixed_basic = $request->fixed_basic;
                $employee->fixed_spl_pf = $request->fixed_spl_pf;
                $employee->fixed_house_rent = $request->fixed_house_rent;
                $employee->fixed_conveyance = $request->fixed_conveyance;
                $employee->fixed_other = $request->fixed_other;
                $employee->fixed_medical = $request->fixed_medical;
                $employee->fixed_statutory_bonus = $request->fixed_statutory_bonus;
                $employee->fixed_gross_salary = $request->fixed_gross_salary;
                $employee->pf_employee_percent = $request->pf_employee_percent;
                $employee->pf_employee = $request->pf_employee;
                $employee->esic_employee_percent = $request->esic_employee_percent;
                $employee->esic_employee = $request->esic_employee;
                $employee->pt = $request->pt;
                $employee->lwf_employee = $request->lwf_employee;
                $employee->employee_medical = $request->employee_medical;
                $employee->total_employee_deduction = $request->total_employee_deduction;
                $employee->net_take_home_salary = $request->net_take_home_salary;
                $employee->pf_employer_percent = $request->pf_employer_percent;
                $employee->pf_employer = $request->pf_employer;
                $employee->esic_employer_percent = $request->esic_employer_percent;
                $employee->esic_employer = $request->esic_employer;
                $employee->lwf_employer = $request->lwf_employer;
                $employee->total_employer_contribution = $request->total_employer_contribution;
                $employee->total_ctc = $request->total_ctc;
            }

            if($request->category == 'Driver'){
                $employee->license_number = $request->license_number;
                $employee->license_issued_from = $request->license_issued_from;
                $employee->license_valid_till = $request->license_valid_till;
                $employee->driver_license_pic = $request->driver_license_pic;   
            }

            // Saving New Account
            if($request->category == 'Staff')
            $grp = Group::where('group_name', 'Current Liabilities')->first();
            else if($request->category == 'Driver')
            $grp = Group::where('group_name', 'Loans & Advances (Asset)')->first();

            $new_account = new Account();
            $new_account->group_id = $grp->id;
            $new_account->account_name = $request->emp_name;
            $new_account->account_type = $request->category;
            $new_account->created_by = $request->user->id;
            $new_account->save();
            // New Account Saved
           
            // Saving the Balances
            $bal = new Balances();
            $bal->account_id = $new_account->id;
            $bal->opening_bal_dr = $request->opening_bal_dr;
            $bal->opening_bal_cr = $request->opening_bal_cr;
            $bal->opening_bal_date = Carbon::now();
            $bal->fin_year = $request->current_year;
            $bal->save();
            // New Balances Saved

            $employee->account_id = $new_account->id;
            $employee->created_by = $request->user->id;    
            $employee->save(); 
            // NEW Employee SAVED
              
            if($request->category == 'Driver'){
                // If $request->newly_added_vehicle, then add the vehicle to the employee
                if($request->newly_added_vehicle){
                    $vehicle_id = $request->newly_added_vehicle['vehicle']['id'];

                    // Check if vehicle is already assigned to any other driver
                    $check = Employee::select('emp_name','vehicle_id')->where('vehicle_id', $vehicle_id)->first();
                    if($check){
                        return response()->json(['errors' => [ 'vehicle'  => ['Vehicle Already Assigned To Another Driver = '.$check['emp_name']]]],422);
                    }

                    $emp_vehicle = new EmployeeVehicle();
                    $emp_vehicle->employee_id = $employee->id;
                    $emp_vehicle->vehicle_id = $vehicle_id;
                    $emp_vehicle->auth_date = $request->newly_added_vehicle['auth_date'];
                    $emp_vehicle->active = true;
                    
                    $emp_vehicle->save();
                     
                    $vehicle = Vehicle::find($vehicle_id);
                    $vehicle->employee_id = $employee->id;
                    $vehicle->save();

                    $employee->vehicle_id = $vehicle_id;
                    $employee->save();

                }
            }

            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($employee->id, $request->current_year);
            $this->createEditLog('created', 'employees', $employee->id, $employee->emp_name , null, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG
            

            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json("Success", 200); // RETURNED RESPONSE

        } 
        // IF ANY ERROR WHILE SAVING
        catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function update(Request $request, $pk){   
        $test = $this->is_edit_permitted($request->user->id, 'EmployeeDetails');
        $test_edit = $test->edit;
        $edit_valid_till_hours = $test->edit_valid_till;

        if( !$test_edit ){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $employee = Employee::find($pk);

        if( $edit_valid_till_hours!=null ){
            $validity = Carbon::parse($employee->created_at)->addHours($edit_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }
          

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();

            // FETCHING OLD VALUE FOR LOG
            $oldValue = $this->get_data_for_obj($pk, $request->current_year);

            // UPDATING THE Employee
            if($request->branch){
                $employee->branch_id = $request->branch['id'];
            }
            else{
                $employee->branch_id = null;
            }
            $employee->designation = $request->designation;
            $employee->dob = $request->dob;
            $employee->joining_date = $request->joining_date;
            $employee->resignation_date = $request->resignation_date;
            $employee->address = $request->address;
            $employee->contact_number = $request->contact_number;
            $employee->father_name = $request->father_name;
            $employee->father_contact = $request->father_contact;
            $employee->guarantor_name = $request->guarantor_name;
            $employee->guarantor_contact = $request->guarantor_contact;
            $employee->pan_number = $request->pan_number;


            $res =  $this->check_for_blacklisted_employee($request->aadhaar_number, $request->pan_number, $request->contact_number, $request->license_number);
            if($res){
             return response()->json(['error'=>'Black Listed', 'emp_name'=>$res->emp_name],500);
            }

            $employee->aadhaar_number = $request->aadhaar_number;
            $employee->pf_number = $request->pf_number;
            $employee->esi_number = $request->esi_number;
             
            $employee->black_list = $request->black_list;

            if($request->black_list){
                $employee->status = 'Blacklisted';
            }
            else{
                if($request->resignation_date){
                    $employee->status = 'Inactive';
                }else{
                    $employee->status = 'Active'; 
                }
            }
            
               
            $employee->employee_pic = $request->employee_pic;
            $employee->employee_addr_proof_pic = $request->employee_addr_proof_pic;
            $employee->employee_aadhar_pic = $request->employee_aadhar_pic;
            
            $employee->account_number = $request->account_number;
            $employee->holder_name = $request->holder_name;
            $employee->bank_name = $request->bank_name;
            $employee->bank_branch = $request->bank_branch;
            $employee->ifsc = $request->ifsc;

            $employee->casual_leaves = $request->casual_leaves;
            $employee->earning_leaves = $request->earning_leaves;
            $employee->sick_leaves = $request->sick_leaves;

            if($request->payment_detail_edited){
                $employee->fixed_basic = $request->fixed_basic;
                $employee->fixed_spl_pf = $request->fixed_spl_pf;
                $employee->fixed_house_rent = $request->fixed_house_rent;
                $employee->fixed_conveyance = $request->fixed_conveyance;
                $employee->fixed_other = $request->fixed_other;
                $employee->fixed_medical = $request->fixed_medical;
                $employee->fixed_statutory_bonus = $request->fixed_statutory_bonus;
                $employee->fixed_gross_salary = $request->fixed_gross_salary;
                $employee->pf_employee_percent = $request->pf_employee_percent;
                $employee->pf_employee = $request->pf_employee;
                $employee->esic_employee_percent = $request->esic_employee_percent;
                $employee->esic_employee = $request->esic_employee;
                $employee->pt = $request->pt;
                $employee->lwf_employee = $request->lwf_employee;
                $employee->employee_medical = $request->employee_medical;
                $employee->total_employee_deduction = $request->total_employee_deduction;
                $employee->net_take_home_salary = $request->net_take_home_salary;
                $employee->pf_employer_percent = $request->pf_employer_percent;
                $employee->pf_employer = $request->pf_employer;
                $employee->esic_employer_percent = $request->esic_employer_percent;
                $employee->esic_employer = $request->esic_employer;
                $employee->lwf_employer = $request->lwf_employer;
                $employee->total_employer_contribution = $request->total_employer_contribution;
                $employee->total_ctc = $request->total_ctc;
            }
            
            if($employee->category == 'Driver'){
                $employee->license_number = $request->license_number;
                $employee->license_issued_from = $request->license_issued_from;
                $employee->license_valid_till = $request->license_valid_till;
                $employee->driver_license_pic = $request->driver_license_pic;
                 
                // IF $request->deleted_vehicle, then delete the vehicle from the employee
                if($request->deleted_vehicle){
                    $emp_vehicle = EmployeeVehicle::find($request->deleted_vehicle['id']);
                    $emp_vehicle->deauth_date = now();
                    $emp_vehicle->active = 0;
                    $emp_vehicle->save();

                    $vehicle = Vehicle::find($request->deleted_vehicle['vehicle']['id']);
                    $vehicle->employee_id = null;
                    $vehicle->save();

                    $employee->vehicle_id = null;
                }

                // If $request->newly_added_vehicle, then add the vehicle to the employee
                if($request->newly_added_vehicle){
                    $vehicle_id = $request->newly_added_vehicle['vehicle']['id'];

                    // Check if vehicle is already assigned to any other driver
                    $check = Employee::select('emp_name','vehicle_id')->where('vehicle_id', $vehicle_id)->first();
                    if($check){
                        return response()->json(['errors' => [ 'vehicle'  => ['Vehicle Already Assigned To Another Driver = '.$check['emp_name']]]],422);
                    }

                    $emp_vehicle = new EmployeeVehicle();
                    $emp_vehicle->employee_id = $employee->id;
                    $emp_vehicle->vehicle_id = $vehicle_id;
                    $emp_vehicle->auth_date = $request->newly_added_vehicle['auth_date'];
                    $emp_vehicle->active = 1;
                    $emp_vehicle->save();

                    $vehicle = Vehicle::find($vehicle_id);
                    $vehicle->employee_id = $employee->id;
                    $vehicle->save();

                    $employee->vehicle_id = $vehicle_id;
                }
            }
            
            $employee->updated_by = $request->user->id;
            $employee->save();
            // NEW Employee SAVED


            $account = Account::find($employee->account_id);
            $account->updated_by = $request->user->id;
            $account->save();

            $bal = Balances::where('account_id', $account->id)->where('fin_year', $request->current_year)->first();
            $bal->opening_bal_dr = $request->opening_bal_dr;
            $bal->opening_bal_cr = $request->opening_bal_cr;
            $bal->save();


            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($employee->id, $request->current_year);
            $this->createEditLog('updated', 'employees', $employee->id,$employee->emp_name, $oldValue, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG
            
            

            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function destroy(Request $request, $pk){
        $test = $this->is_delete_permitted($request->user->id, 'EmployeeDetails');
        $test_delete = $test->delete;
        $delete_valid_till_hours = $test->delete_valid_till;

        if( !$test_delete ){
            return response()->json(['Permission Denied For The Action'],401);
        }

         // FETCHING OLD VALUE FOR LOG
         $oldValue = $this->get_data_for_obj($pk, $request->current_year);
        
        $employee = Employee::find($pk);

        if( $delete_valid_till_hours!=null ){
            $validity = Carbon::parse($employee->created_at)->addHours($delete_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();

            

        //FIND AND DELETE THE Employee
        $employee->delete();

        $account = Account::where('id',$employee->account_id)->first();
            $balances = Balances::where('account_id', $account->id)->delete();
            $account->delete();

        // SAVING THE EDIT LOG
        $this->createEditLog('deleted', 'employees', $employee->id,$employee->emp_name, $oldValue, null, $request->user->id, $request->current_year); // CREATE EDIT LOG

            // TRANSACTIONS COMMITTED
            DB::commit();
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function autocomplete(Request $request){
        // Initialize the query builder
        $fields = ['id', 'emp_name'];
        
        if ($request->fields) {
            $fields = array_merge($fields, $request->fields);
        }

        $query = Employee::select($fields);

        // Apply any additional filters based on other columns as needed
        if ($request->has('filterby')) {
            $orWhereFilters = isset($request->filterby['orWhere']) ? $request->filterby['orWhere'] : [];
            $whereFilters = isset($request->filterby['where']) ? $request->filterby['where'] : [];

           // Loop through the whereFilters array
           foreach ($whereFilters as $whereFilter) {
               // Apply the filter to the query
               $query->where($whereFilter['key'], $whereFilter['value']);
           }

           // Loop through the orWhereFilters array
           foreach ($orWhereFilters as $orWhereFilter) {
               // Apply the filter to the query
               $query->orWhere($orWhereFilter['key'], $orWhereFilter['value']);
           }
        
        }
        
        // Check if searchVariable is provided and apply the filter
        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('emp_name', 'like', '%'. $searchVariable . '%')->orWhere('emp_id', 'like', '%'. $searchVariable . '%')->orWhere('contact_number', 'like', '%'. $searchVariable . '%')->orWhere('aadhaar_number', 'like', '%'. $searchVariable . '%')->orWhere('pan_number', 'like', '%'. $searchVariable . '%');
        }

        


        // Limit the number of results
        $query->take($request->limit)->orderBy('emp_name', 'asc');

        // Retrieve the filtered employees
        $employees = $query->get();
        return response(['employee' => $employees, 'draw' => $request->draw]);
    }

    public function autocomplete_for_driver_stl(Request $request){
        $vehicle_id = $request->vehicle_id;

        // Pick driver_ids from bookings where vehicle_id = $vehicle_id
        $driver_ids = LoadingSlip::where('vehicle_id', $vehicle_id)
                        ->whereDoesntHave('d_settlement_desc', function ($query) {
                            $query->where('against', 'Loading Slip');
                        })
                       ->pluck('driver_id');

        $query = Employee::select('id','emp_name','category','account_id')->whereIn('account_id', $driver_ids);

        // Check if searchVariable is provided and apply the filter
        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('emp_name', 'like', '%'. $searchVariable . '%')->orWhere('emp_id', 'like', '%'. $searchVariable . '%');
        }

        
        // Limit the number of results
        $query->take(20)->orderBy('emp_name', 'asc');

        // Retrieve the filtered employees
        $employees = $query->get();
        return response(['employee' => $employees]);

    }

    public function datatable(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'EmployeeDetails');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $query = Employee::select('id', 'emp_id', 'emp_name', 'category', 'designation','branch_id')->with(['branch' => function ($query) {
                            $query->select('id', 'branch_name','branch_code');
                        }])->orderBy($column, $dir);

        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('emp_name', 'like', $searchValue . '%')
                        ->orWhere('emp_id', 'like', $searchValue . '%')
                        ->orWhereHas('branch', function ($query) use ($searchValue) {
                            $query->where('branch_name', 'like', $searchValue . '%');
                        });
            });
        }

        $data = $query->paginate($length);

        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    public function check_unique(Request $request){
        $key = $request->key;
        $value = $request->value;

        if(!$value){
            return response()->json(['exists' => false]);
        }

        $exists = Employee::where($key, $value)->exists();
        if($exists){
            return response()->json(['errors' => [ $key  => ['Already Exists !']]],422);
        }

        return response()->json(['exists' => false]);
    }

    public function check_black_list(Request $request){
        $res =  $this->check_for_blacklisted_employee($request->aadhaar_number, $request->pan_number, $request->contact_number, $request->license_number);
            if($res){
             return response()->json(['error'=>'Black Listed', 'emp_name'=>$res->emp_name],500);
            }
    }

    public function get_next_emp_id(Request  $request){
        $type='SYSTEM';

        $category = $request->category;

        if($type=='SYSTEM'){
            $company_code = Company::select('company_code')->find($request->company['id'])['company_code'];

            $max = Employee::where('category',$category)->max('max_number');
            
            $pre = $category=='Staff'?'S':'D';

            $emp_id = $company_code.'-'.$pre.'-'.sprintf('%05d', $max+1);
            
            return response()->json(['emp_id' => strtoupper($emp_id), 'max_number' => $max+1]);
        }
    }

    public function get_employee_report(Request $request){
        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $report_data = $request->report_data;


        $query = Employee::select('id','emp_name','category','joining_date','resignation_date','address','contact_number','license_number','license_valid_till','pan_number','aadhaar_number','account_number','bank_name','ifsc','bank_branch','status')->with(['branch' => function ($query) {
                            $query->select('id', 'branch_name','branch_code');
                        }]);

        if(isset($report_data['employee'])){
            $query->where('id', $report_data['employee']['id']);
        }

        if(isset($report_data['branch'])){
            $query->where('branch_id', $report_data['branch']['id']);
        }

        if(isset($report_data['category'])){
            $query->where('category', $report_data['category']);
        }

        if(isset($report_data['status'])){
            $query->where('status', $report_data['status']);
        }


        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('emp_name', 'like', $searchValue . '%')
                    ->orWhere('emp_id', 'like', $searchValue . '%')
                    ->orWhere('contact_number', 'like', $searchValue . '%');
            });
        }

        $user_branch_ids = $request->user_branch_ids;
        if ( $user_branch_ids ) {
            $query->whereIn('branch_id', $user_branch_ids );
        }


        $data = $query->orderBy($column, $dir)->paginate($length);
          
        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    protected function check_for_blacklisted_employee($aadhaar, $pan, $contact, $license){
        $query = Employee::select('id','emp_name')->where('black_list',1);

        if($aadhaar)
            $query->where('aadhaar_number', $aadhaar);
        if($pan)
            $query->where('pan_number', $pan);
        if($contact)
            $query->where('contact_number', $contact);
        if($license)
            $query->where('license_number', $license);
                
         
        $check_for_black_list = $query->first();
        return $check_for_black_list;
    }

    protected function get_data_for_obj($pk, $current_year){
        $employee = Employee::with(['branch' => function ($query) {
            $query->select('id', 'branch_name','branch_code');
         },
         'vehicles'=> function ($query) {
            $query->wherePivot('active', true)
            ->select('vehicles.id', 'vehicles.vehicle_number', 'vehicles.vehicle_type')->withPivot('id', 'auth_date');
         }
         ])->find($pk);

        $data = $employee->toArray();

        $data['vehicles'] = $employee->vehicles->map(function ($vehicle) {
            return [
                'id' => $vehicle->pivot->id,
                'auth_date' => $vehicle->pivot->auth_date,
                'vehicle' => [
                    'id' => $vehicle->id,
                    'vehicle_number' => $vehicle->vehicle_number,
                    'vehicle_type' => $vehicle->vehicle_type,
                ],
            ];
        });

        $balance = Balances::where('account_id', $employee->account_id)->where('fin_year',$current_year)->first();
        
        return ['employee'=>$data, 'balance'=>$balance];
    }

    protected function get_max_number_from_emp_id($input){
        // Extract the integer value using regular expression
        $pattern = '/[-\/](\d+)$/';
        preg_match($pattern, $input, $matches);
        
        if(empty($matches)){
            return intval($input);
        }

        $integerValue = intval(end($matches));
        return $integerValue;
    }

}
