<?php

namespace App\Http\Controllers\Master;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Http\Traits\EditLogTrait;
use App\Http\Traits\PermissionsTrait;
use App\Models\Bill\Bill;
use App\Models\Booking\BookGr;
use App\Models\Master\IssueBooks;
use Illuminate\Support\Facades\DB;

use Carbon\Carbon;

class IssueBooksController extends Controller
{
    use PermissionsTrait;
    use EditLogTrait;

    public function index(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'IssueBooks');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        //FETCH AND SEND ALL
        $issue_book = IssueBooks::with([
                       'branch' => function ($query) {
                        $query->select('id', 'branch_name','branch_code');
                        },
                        'party' => function ($query) {
                            $query->select('id', 'party_name','party_code');
                        }
                    ])->get();
        return response()->json($issue_book);
    }

    public function show(Request $request, $pk){
        // FETCH AND SEND
        $issue_book = IssueBooks::with([
            'branch' => function ($query) {
                $query->select('id', 'branch_name','branch_code');
            },
            'party' => function ($query) {
                $query->select('id', 'party_name','party_code');
            }
        ])->find($pk);
        return response()->json(['issue_books'=>$issue_book]);
    }

    public function store(Request $request){
        
        $test = $this->is_new_permitted($request->user->id, 'IssueBooks');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $validatedData = $request->validate([
            'book_type' => 'required',
            'branch' => 'required',
            'issue_from' => 'required|numeric',
            'issue_to' => 'required|numeric',
        ], [
            'book_type.required' => 'Required',
            'branch.required' => 'Please Select Branch',
            'issue_from.required' => 'Required',
            'issue_to.required' => 'Required',
        ]);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();  

            // SAVING A NEW IssueBooks
            $book_type = $request->book_type;
            $issue_from = $request->issue_from;
            $issue_to = $request->issue_to;
            $prefix = $request->prefix;
            $search_from = $request->prefix ? $request->prefix.'-'.$request->issue_from : $request->issue_from;
            $search_to = $request->prefix ? $request->prefix.'-'.$request->issue_to : $request->issue_to;

           


            $check = IssueBooks::with(['branch' => function ($query) {
                $query->select('id', 'branch_name','branch_code');
            }])
            ->where('book_type', $book_type)
            ->where('prefix', $prefix)
            ->where(function ($query) use ($issue_from, $issue_to) {
                $query->whereBetween('issue_from', [$issue_from, $issue_to])
                      ->orWhereBetween('issue_to', [$issue_from, $issue_to])
                      ->orWhere(function ($query) use ($issue_from, $issue_to) {
                          $query->where('issue_from', '<', $issue_from)
                                ->where('issue_to', '>', $issue_to);
                      });
            })
            ->first();
            
            if ($check) {
                $range = $prefix? $prefix.'-'.$check->issue_from.' to '.$prefix.'-'.$check->issue_to : $check->issue_from.' to '.$check->issue_to;
                return response()->json(['errors' => [
                    'issue_from' => ['Found at ' . $check->branch['branch_name'] . ' with range ' . $range],
                    'issue_to' => ['Found at ' . $check->branch['branch_name']],
                    'range'=> $range,
                    'prefix'=> $prefix,
                    'search_from'=> $check['search_from'],
                ]], 422);
            }
                        
            $book=new IssueBooks;
            $book->book_type=$book_type;
            $book->branch_id=$request->branch['id'];

            if($request->party){
                $book->party_id = $request->party['id'];
            }
            else{
                $book->party_id = null;
            }

            $book->issue_date=$request->issue_date;
            $book->prefix=$request->prefix;
            $book->issue_from=$issue_from;
            $book->issue_to=$issue_to;
            $book->search_from=$search_from;
            $book->search_to=$search_to;

            if($request->valid_for_current_year){
                $book->valid_for = $request->current_year;
            }
            else{
                $book->valid_for = null;
            }

            $book->created_by = $request->user->id;
            $book->save();
            // NEW IssueBooks SAVED

            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($book->id);
            $this->createEditLog('created', 'issue_books', $book->id, $book->prefix.' '.$book->issue_from.'-'.$book->issue_to , null, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG
            

            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json("Success", 200); // RETURNED RESPONSE

        } 
        // IF ANY ERROR WHILE SAVING
        catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function destroy(Request $request, $pk){
        $test = $this->is_delete_permitted($request->user->id, 'IssueBooks');
        $test_delete = $test->delete;
        $delete_valid_till_hours = $test->delete_valid_till;

        if( !$test_delete ){
            return response()->json(['Permission Denied For The Action'],401);
        }

        // FETCHING OLD VALUE FOR LOG
        $oldValue = $this->get_data_for_obj($pk);
        
        $issue_book = IssueBooks::find($pk);

        if( $delete_valid_till_hours!=null ){
            $validity = Carbon::parse($issue_book->created_at)->addHours($delete_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();


        //FIND AND DELETE THE IssueBooks
        $issue_book->delete();

        $this->createEditLog('deleted', 'issue_books', $issue_book->id, $issue_book->prefix.' '.$issue_book->issue_from.'-'.$issue_book->issue_to , $oldValue, null, $request->user->id, $request->current_year); // CREATE EDIT LOG

            // TRANSACTIONS COMMITTED
            DB::commit();
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function datatable(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'IssueBooks');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $query = IssueBooks::select('id', 'book_type', 'prefix', 'issue_date' , 'issue_from','issue_to','search_from','search_to','valid_for', 'branch_id', 'party_id')->with([
                        'branch' => function ($query) {
                            $query->select('id', 'branch_name','branch_code');
                        },
                        'party'=> function ($query) {
                            $query->select('id', 'party_name','party_code');
                        }
                        ])->orderBy($column, $dir);
        
        $query->where(function ($query) use ($request) {
            $query->where('valid_for', $request->current_year)->orWhere('valid_for',null);
        });
                        
        if ($searchValue) {
                $query->where(function ($query) use ($searchValue) {
                    $query->where('issue_from', '<=', $searchValue)
                          ->where('issue_to', '>=', $searchValue)
                          ->orWhere('prefix', $searchValue)
                          ->orWhere('search_from', 'like', '%'.$searchValue.'%')
                          ->orWhere('search_to',  'like', '%'.$searchValue.'%')
                          ->orWhereHas('branch', function ($query) use ($searchValue) {
                            $query->where('branch_name', 'like', $searchValue . '%');
                          })
                          ->get();
                });
        }

        $data = $query->paginate($length);

        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    public function check_unique(Request $request){
        $key = $request->key;
        $value = $request->value;

        if(!$value){
            return response()->json(['exists' => false]);
        }

        $exists = IssueBooks::where($key, $value)->exists();
        if($exists){
            return response()->json(['errors' => [ $key  => ['Already Exists !']]],422);
        }

        return response()->json(['exists' => false]);
    }

    public function get_prefixes(Request $request, $book_type){
        // GET ALL THE PREFIXES WHERE BOOKTYPE AND BRANCH ID MATCH
        $query = IssueBooks::where('book_type', $book_type)
                    ->where('prefix', '!=', null);

        $query->where(function ($query) use ($request) {
            $query->where('valid_for', $request->current_year)->orWhere('valid_for',null);
        });

        if(!$request->branch_id){
            $user_branch_ids = $request->user_branch_ids;
            if ( $user_branch_ids ) {
                $query->whereIn('branch_id', $user_branch_ids );
            }
        }
        else{
            $query->where('branch_id', $request->branch_id);
        }            
        

        $prefixes = $query->distinct()->pluck('prefix');

        return response()->json(['prefixes' => $prefixes]);
    }

    public function get_pending_books_report(Request $request){
        $report_data = $request->report_data;


        $book_type = $report_data['book_type'];
        $prefix = $report_data['prefix'];
        $issue_from = $report_data['issue_from'];
        $issue_to = $report_data['issue_to'];

        // Loop prefix-issue_from to prefix-issue_to

        $data = [];
        if($book_type=='GR'){
            
            for ($i=$issue_from; $i <$issue_to ; $i++) { 

               $book_num = $prefix ? $prefix.'-'.$i : $i;

               $gr = BookGr::where('gr_number',$book_num)->exists();

               if(!$gr){
                $data[] = $book_num;
               }

            }


        }
        else{

            for ($i=$issue_from; $i <$issue_to ; $i++) { 
                $book_num = $prefix ? $prefix.'-'.$i : $i;
 
                $bill = Bill::where('bill_number',$book_num)->exists();
 
                if(!$bill){
                 $data[] = $book_num;
                }
 
             }

        }
          
        return response()->json(['data' => $data]);
    }


    protected function get_data_for_obj($pk){
        $issue_book = IssueBooks::with([
            'branch' => function ($query) {
                $query->select('id', 'branch_name','branch_code');
            },
            'party' => function ($query) {
                $query->select('id', 'party_name','party_code');
            }
        ])->find($pk);
        return ['issue_books'=>$issue_book];  
    }

}
