<?php

namespace App\Http\Controllers\Master;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Http\Traits\EditLogTrait;
use App\Http\Traits\PermissionsTrait;
use App\Models\Master\Material;
use Illuminate\Support\Facades\DB;

use Carbon\Carbon;

class MaterialController extends Controller
{
    use PermissionsTrait;
    use EditLogTrait;

    public function index(Request $request)
    {
        $test = $this->is_read_permitted($request->user->id, 'MaterialDetails');
        if (!$test) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        // FETCH AND SEND ALL
        $materials = Material::all();
        return response()->json($materials);
    }

    public function show(Request $request, $pk)
    {
        // FETCH AND SEND
        $material = Material::find($pk);
        return response()->json(['material' => $material]);
    }

    public function store(Request $request)
    {

        $test = $this->is_new_permitted($request->user->id, 'MaterialDetails');
        if (!$test) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        $validatedData = $request->validate([
            'material_name' => 'required|min:3|unique:materials,material_name',
        ], [
            'material_name.required' => 'Required',
            'material_name.unique' => 'Already Exist!',
            'material_name.min' => 'Minimum 3 Characters Required!'
        ]);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();

            // SAVING A NEW MATERIAL
            $material = new Material();
            $material->material_name = $request->material_name;
            $material->desc = $request->desc;
            $material->gst_exempted = $request->gst_exempted;
            $material->created_by = $request->user->id;
            $material->save();
            // NEW MATERIAL SAVED

            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($material->id);
            $this->createEditLog('created', 'materials', $material->id, $material->material_name, null, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG


            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json("Success", 200); // RETURNED RESPONSE

        }
        // IF ANY ERROR WHILE SAVING
        catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function update(Request $request, $pk)
    {
        $test = $this->is_edit_permitted($request->user->id, 'MaterialDetails');
        $test_edit = $test->edit;
        $edit_valid_till_hours = $test->edit_valid_till;

        if (!$test_edit) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        $material = Material::find($pk);

        // FETCHING OLD VALUE FOR LOG
        $oldValue = $this->get_data_for_obj($pk);

        if ($edit_valid_till_hours != null) {
            $validity = Carbon::parse($material->created_at)->addHours($edit_valid_till_hours);
            if (!$validity->isFuture()) {
                return response()->json(['Permission Timed Out'], 500);
            }
        }


        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();

            // UPDATING THE MATERIAL
            $material->material_name = $request->material_name;
            $material->desc = $request->desc;
            $material->gst_exempted = $request->gst_exempted;
            $material->updated_by = $request->user->id;
            $material->save();
            // MATERIAL UPDATED


            $newVal = $this->get_data_for_obj($material->id);
            $this->createEditLog('updated', 'materials', $material->id, $material->material_name, $oldValue, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG


            // TRANSACTIONS COMMITTED
            DB::commit();

            

            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function destroy(Request $request, $pk)
    {
        $test = $this->is_delete_permitted($request->user->id, 'MaterialDetails');
        $test_delete = $test->delete;
        $delete_valid_till_hours = $test->delete_valid_till;

        if (!$test_delete) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        $material = Material::find($pk);

        if ($delete_valid_till_hours != null) {
            $validity = Carbon::parse($material->created_at)->addHours($delete_valid_till_hours);
            if (!$validity->isFuture()) {
                return response()->json(['Permission Timed Out'], 500);
            }
        }

        // FETCHING OLD VALUE FOR LOG
        $oldValue = $this->get_data_for_obj($pk);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();


        // FIND AND DELETE THE MATERIAL
        $material->delete();

        $this->createEditLog('deleted', 'materials', $material->id, $material->material_name, $oldValue, null, $request->user->id, $request->current_year); // CREATE EDIT LOG

            // TRANSACTIONS COMMITTED
            DB::commit();
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function autocomplete(Request $request)
    {
         // Initialize the query builder
         $fields = ['id', 'material_name'];
        
         if ($request->fields) {
             $fields = array_merge($fields, $request->fields);
         }
 
        $query = Material::select($fields);


        // Apply any additional filters based on other columns as needed
        if ($request->has('filterby')) {
            $orWhereFilters = isset($request->filterby['orWhere']) ? $request->filterby['orWhere'] : [];
            $whereFilters = isset($request->filterby['where']) ? $request->filterby['where'] : [];

           // Loop through the whereFilters array
           foreach ($whereFilters as $whereFilter) {
               // Apply the filter to the query
               $query->where($whereFilter['key'], $whereFilter['value']);
           }

           // Loop through the orWhereFilters array
           foreach ($orWhereFilters as $orWhereFilter) {
               // Apply the filter to the query
               $query->orWhere($orWhereFilter['key'], $orWhereFilter['value']);
           }
        
        }
        
        // Check if searchVariable is provided and apply the filter
        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('material_name', 'like', '%'. $searchVariable . '%');
        }

        // Limit the number of results
        $query->take($request->limit)->orderBy('material_name', 'asc');
        // Retrieve the filtered 
        $materials = $query->get();
        return response(['materials' => $materials, 'draw' => $request->draw]);
    }

    public function datatable(Request $request)
    {
        $test = $this->is_read_permitted($request->user->id, 'MaterialDetails');
        if (!$test) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $query = Material::orderBy($column, $dir);

        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('material_name', 'like', $searchValue . '%');
            });
        }

        $data = $query->paginate($length);

        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    public function check_unique(Request $request)
    {
        $key = $request->key;
        $value = $request->value;

        if (!$value) {
            return response()->json(['exists' => false]);
        }

        $exists = Material::where($key, $value)->exists();
        if ($exists) {
            return response()->json(['errors' => [$key  => ['Already Exists !']]], 422);
        }

        return response()->json(['exists' => false]);
    }

    protected function get_data_for_obj($pk)
    {
        $material = Material::find($pk);
        return ['material' => $material];
    }
}
