<?php

namespace App\Http\Controllers\Master;

use App\Http\Controllers\Controller;
use App\Http\Traits\EditLogTrait;
use App\Http\Traits\PermissionsTrait;
use App\Models\Master\PackageMethod;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class PackageMethodController extends Controller
{
    use PermissionsTrait;
    use EditLogTrait;

    public function index(Request $request)
    {
        $test = $this->is_read_permitted($request->user->id, 'PackageMethodDetails');
        if (!$test) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        // Fetch and send all PackageMethods
        $package_methods = PackageMethod::select('id','package_method','desc')->get();
        return response()->json($package_methods);
    }

    public function show(Request $request, $pk)
    {
        // Fetch and send the PackageMethod
        $package_method = PackageMethod::find($pk);
        return response()->json(['package_method' => $package_method]);
    }

    public function store(Request $request)
    {
        $test = $this->is_new_permitted($request->user->id, 'PackageMethodDetails');
        if (!$test) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        $validatedData = $request->validate([
            'package_method' => 'required|unique:package_methods',
        ], [
            'package_method.required' => 'Required',
            'package_method.unique' => 'Already Exists!',
        ]);

        try {
            // Transaction begins
            DB::beginTransaction();

            // Save a new PackageMethod
            $package_method = new PackageMethod();
            $package_method->package_method = $request->package_method;
            $package_method->desc = $request->desc;
            $package_method->created_by = $request->user->id;
            $package_method->save();

            // Save the edit log
            $newVal = $this->get_data_for_obj($package_method->id);
            $this->createEditLog('created', 'package_methods', $package_method->id, $package_method->package_method, null, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG


            // Transaction committed
            DB::commit();

            
            return response()->json("Success", 200); // Returned response

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function update(Request $request, $pk)
    {
        $test = $this->is_edit_permitted($request->user->id, 'PackageMethodDetails');
        $test_edit = $test->edit;
        $edit_valid_till_hours = $test->edit_valid_till;

        if (!$test_edit) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        $package_method = PackageMethod::find($pk);

        // Fetch the old value for log
        $oldValue = $this->get_data_for_obj($pk);

        if ($edit_valid_till_hours != null) {
            $validity = Carbon::parse($package_method->created_at)->addHours($edit_valid_till_hours);
            if (!$validity->isFuture()) {
                return response()->json(['Permission Timed Out'], 500);
            }
        }

        try {
            // Transaction begins
            DB::beginTransaction();

            // Update the PackageMethod
            $package_method->package_method = $request->package_method;
            $package_method->desc = $request->desc;
            $package_method->updated_by = $request->user->id;
            $package_method->save();

            $newVal = $this->get_data_for_obj($package_method->id);
            $this->createEditLog('updated', 'package_methods', $package_method->id, $package_method->package_method, $oldValue, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG

            // Transaction committed
            DB::commit();

            
            return response()->json("Success", 200); // Returned response

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function destroy(Request $request, $pk)
    {
        $test = $this->is_delete_permitted($request->user->id, 'PackageMethodDetails');
        $test_delete = $test->delete;
        $delete_valid_till_hours = $test->delete_valid_till;

        if (!$test_delete) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        $package_method = PackageMethod::find($pk);

        if ($delete_valid_till_hours != null) {
            $validity = Carbon::parse($package_method->created_at)->addHours($delete_valid_till_hours);
            if (!$validity->isFuture()) {
                return response()->json(['Permission Timed Out'], 500);
            }
        }

        // Fetch the old value for log
        $oldValue = $this->get_data_for_obj($pk);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();


        // Find and delete the PackageMethod
        $package_method->delete();

        $this->createEditLog('deleted', 'package_methods', $package_method->id, $package_method->package_method, $oldValue, null, $request->user->id, $request->current_year); // CREATE EDIT LOG

            // TRANSACTIONS COMMITTED
            DB::commit();
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function autocomplete(Request $request)
    {
        // Perform search and return matching package_methods
        // WHILE SEARCHING THIS WILL RUN
        $query = PackageMethod::select('id', 'package_method');

        // Apply any additional filters based on other columns as needed
        if ($request->has('filterby')) {
            $orWhereFilters = isset($request->filterby['orWhere']) ? $request->filterby['orWhere'] : [];
            $whereFilters = isset($request->filterby['where']) ? $request->filterby['where'] : [];

           // Loop through the whereFilters array
           foreach ($whereFilters as $whereFilter) {
               // Apply the filter to the query
               $query->where($whereFilter['key'], $whereFilter['value']);
           }

           // Loop through the orWhereFilters array
           foreach ($orWhereFilters as $orWhereFilter) {
               // Apply the filter to the query
               $query->orWhere($orWhereFilter['key'], $orWhereFilter['value']);
           }
        
        }
        
        // Check if searchVariable is provided and apply the filter
        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('package_method', 'like', '%'. $searchVariable . '%');
        }

        // Limit the number of results
        $query->take($request->limit)->orderBy('package_method', 'asc');

        // Retrieve the filtered 
        $package_methods = $query->get();
        return response(['package_methods' => $package_methods, 'draw' => $request->draw]);
    }

    public function check_unique(Request $request)
    {
        $key = $request->key;
        $value = $request->value;

        if (!$value) {
            return response()->json(['exists' => false]);
        }

        $exists = PackageMethod::where($key, $value)->exists();
        if ($exists) {
            return response()->json(['errors' => [$key  => ['Already Exists !']]], 422);
        }

        return response()->json(['exists' => false]);
    }

    protected function get_data_for_obj($pk)
    {
        // Fetch and send the package_method
        $package_method = PackageMethod::find($pk);
        return ['package_method' => $package_method];
    }
}

