<?php

namespace App\Http\Controllers\Master;

use App\Http\Controllers\Controller;
use App\Http\Traits\EditLogTrait;
use App\Http\Traits\PermissionsTrait;
use App\Models\Account\Account;
use App\Models\Account\Balances;
use App\Models\Account\Group;
use App\Models\Master\Party;
use App\Models\Master\PartyBranches;
use App\Models\Master\PartyOpeningDetails;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class PartyController extends Controller
{
    use PermissionsTrait;
    use EditLogTrait;

    public function index(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'PartyDetails');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        $current_year = $request->current_year; 

        //FETCH AND SEND ALL
        $parties = Party::with(['party_branches' => function ($query) {
                        $query->select('id', 'branch_name', 'address', 'state','city', 'pincode', 'gst_number', 'contact_person', 'tele_number', 'party_id');
                    }, 
                    'material'=>function($query){
                        $query->select('id','material_name');
                    },
                    'preset_on_account'=>function($query){
                        $query->select('id','account_name','account_type');
                    },
                    'party_opening_details'=>function($query) use ($current_year){
                        $query->where('fin_year',$current_year);
                    }
                    ])->get();
        return response()->json($parties);
    }

    public function show(Request $request, $pk){
        $current_year = $request->current_year;

        // FETCH AND SEND
        $party = Party::with(['party_branches' => function ($query) {
                            $query->select('id', 'branch_name', 'address', 'state','city', 'pincode', 'gst_number', 'contact_person', 'tele_number', 'party_id');
                        }, 
                        'party_opening_details'=>function($query) use ($current_year){
                            $query->where('fin_year',$current_year);
                        },
                        'material'=>function($query){
                            $query->select('id','material_name');
                        },
                        'preset_on_account'=>function($query){
                            $query->select('id','account_name','account_type');
                        },
                        'branch' => function ($query) {
                            $query->select('id', 'branch_name','branch_code');
                         }])->find($pk);

        $balance = $party->account_id ? Balances::select('id','opening_bal_dr', 'opening_bal_cr', 'opening_bal_date')->where('fin_Year',$request->current_year)->where('account_id', $party->account_id)->first() : null;

        return response()->json(['party'=>$party,'balance'=>$balance]);
    }

    public function store(Request $request){

        $test = $this->is_new_permitted($request->user->id, 'PartyDetails');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $validatedData = $request->validate([
            'party_name' => 'required|unique:parties,party_name',
            'party_code' => 'required|unique:parties,party_code',
        ], [
            'party_name.required' => 'Required',
            'party_code.required' => 'Required',
            'party_name.unique' => 'Already Exist !',
            'party_code.unique' => 'Already Exist !',
        ]);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();  


            $check_uniqueness = $this->check_unique_filters($request->gst_number,$request->pan_number,$request->aadhar_number,null);
            if($check_uniqueness['status']){
                return response()->json(['message'=>'Matching Party - '.$check_uniqueness['party_name'],  'index'=>$check_uniqueness['party_name']],500);
            }
            
            // SAVING A NEW PARTY
            $party = new Party();
            $party->party_name = $request->party_name;
            $party->party_code = $request->party_code;
            $party->party_type = $request->party_type;

            if($request->branch){
                $party->branch_id = $request->branch['id'];
            }
            else{
                $party->branch_id = null;
            }

            $party->contact_person = $request->contact_person;
            $party->tele_number = $request->tele_number;
            $party->mob_number  = $request->mob_number;
            $party->email = $request->email;
            $party->pan_number = $request->pan_number;
            $party->aadhar_number = $request->aadhar_number;
            $party->gst_number = $request->gst_number;

            $party->add_to_accounts = $request->add_to_accounts;

            $party->vendor_code = $request->vendor_code;
            $party->limit = $request->limit;
            $party->account_head = $request->account_head;

            $party->address1 = $request->address1;
            $party->address2 = $request->address2;
            $party->city = $request->city; 
            $party->state = $request->state;
            $party->pincode = $request->pincode;

            $party->account_number = $request->account_number;
            $party->holder_name = $request->holder_name;
            $party->bank_name = $request->bank_name;
            $party->bank_branch = $request->bank_branch;
            $party->ifsc = $request->ifsc;

            $party->gst_paid_by = $request->gst_paid_by;

            if($request->material){
                $party->material_id = $request->material['id'];
            }
            else{
                $party->material_id = null;
            }

            $party->unit = $request->unit;
            $party->pay_mode = $request->pay_mode;
            $party->gr_chg = $request->gr_chg;
            $party->preset_dynamics = $request->preset_dynamics;
            $party->preset_on_account_id = $request->preset_on_account ? $request->preset_on_account['id'] :null;

            $party->pod_req = $request->pod_req;
            $party->door_del = $request->door_del;
            $party->gst_percent = $request->gst_percent;

            $party->cheque_img = $request->cheque_img;
            $party->pan_img = $request->pan_img;
            $party->tds_certificate_img = $request->tds_certificate_img;
            $party->gst_certificate_img = $request->gst_certificate_img;
            $party->other_img = $request->other_img;
            
            $party->created_by = $request->user->id;
            // Save the model to the database
            $party->save();

            // SAVING PARTY BRANCHES
            $party_branches_data = $request->party_branches;
            foreach ($party_branches_data as $pbd) {
                if(!$pbd['branch_name'])
                continue;
                
                // Check if branch already exist
                $branch = PartyBranches::where('branch_name', $pbd['branch_name'])->where('party_id', $party->id)->exists();
                if($branch){
                    return response()->json("Party Branch Name Already Exist", 500);
                }
                
                $pb = new PartyBranches();
                $pb->party_id = $party->id;
                $pb->branch_name = $pbd['branch_name'];
                $pb->address = $pbd['address'];
                $pb->state = $pbd['state'];
                $pb->city = $pbd['city'];
                $pb->pincode = $pbd['pincode'];
                $pb->gst_number = $pbd['gst_number'];
                $pb->contact_person = $pbd['contact_person'];
                $pb->tele_number = $pbd['tele_number'];
                $pb->save();
            }
            // PARTY BRANCHES SAVED
             

            if($request->add_to_accounts){
                if (!isFalsy($request->opening_bal_dr) && !isFalsy($request->opening_bal_cr)) {
                    return response()->json("Please Provide A Single Value For Opening Balances", 500);
                }

                // SAVING OPENING BALANCES
                $opening_details_data = $request->opening_details;
                foreach ($opening_details_data as $odd) {
                    if(!$odd['against'] || !$odd['against_number'] || ( !$odd['dr'] && !$odd['cr'] ) || ( $odd['dr'] && $odd['cr'] ) )
                    continue;
                    
                    $pod = new PartyOpeningDetails();
                    $pod->party_id = $party->id;
                    $pod->against = $odd['against'];
                    $pod->against_number = $odd['against_number'];
                    $pod->date = $odd['date'];
                    $pod->desc = $odd['desc'];
                    $pod->rate = $odd['rate'];
                    $pod->dr = $odd['dr'];
                    $pod->cr = $odd['cr'];
                    $pod->paid_status = false;
                    $pod->fin_year = $request->current_year;
                    $pod->save();
                }
                // OPENING BALANCES SAVED
                 
                // Saving New Account
                $grp = Group::where('group_name', $request->account_head)->first();

                $new_account = new Account();
                $new_account->group_id = $grp->id;
                $new_account->account_name = $request->party_name;
                $new_account->account_type = 'Party';
                $new_account->created_by = $request->user->id;
                $new_account->save();
                // New Account Saved
               
                // Saving the Balances
                $bal = new Balances();
                $bal->account_id = $new_account->id;
                $bal->opening_bal_dr = $request->opening_bal_dr;
                $bal->opening_bal_cr = $request->opening_bal_cr;
                $bal->opening_bal_date = Carbon::now();
                $bal->fin_year = $request->current_year;
                $bal->save();
                // New Balances Saved

                $party->account_id = $new_account->id;
                $party->save(); 
            }
             
            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($party->id,$request->current_year);
            $this->createEditLog('created', 'parties', $party->id, $party->party_name,null, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG
            
        
            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json("Success", 200); // RETURNED RESPONSE

        } 
        // IF ANY ERROR WHILE SAVING
        catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function update(Request $request, $pk){   
        $test = $this->is_edit_permitted($request->user->id, 'PartyDetails');
        $test_edit = $test->edit;
        $edit_valid_till_hours = $test->edit_valid_till;

        if( !$test_edit ){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $party = Party::find($pk);

        $check_uniqueness = $this->check_unique_filters($request->gst_number,$request->pan_number,$request->aadhar_number, $pk);
        if($check_uniqueness['status']){
            return response()->json(['message'=>'Matching Party - '.$check_uniqueness['party_name'],  'index'=>$check_uniqueness['party_name']],500);
        }


        if( $edit_valid_till_hours!=null ){
            $validity = Carbon::parse($party->created_at)->addHours($edit_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }
          

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();

            // FETCHING OLD VALUE FOR LOG
            $oldValue = $this->get_data_for_obj($pk, $request->current_year);

            // SAVING A NEW PARTY
            $party->party_type = $request->party_type;
            
            if($request->branch){
                $party->branch_id = $request->branch['id'];
            }
            else{
                $party->branch_id = null;
            }

            $party->contact_person = $request->contact_person;
            $party->tele_number = $request->tele_number;
            $party->mob_number  = $request->mob_number;
            $party->email = $request->email;
            $party->pan_number = $request->pan_number;
            $party->aadhar_number = $request->aadhar_number;
            $party->gst_number = $request->gst_number;
             
            $party->vendor_code = $request->vendor_code;
            $party->limit = $request->limit;
            $party->account_head = $request->account_head;

            $party->address1 = $request->address1;
            $party->address2 = $request->address2;
            $party->city = $request->city; 
            $party->state = $request->state;
            $party->pincode = $request->pincode;
            
            

            $party->account_number = $request->account_number;
            $party->holder_name = $request->holder_name;
            $party->bank_name = $request->bank_name;
            $party->bank_branch = $request->bank_branch;
            $party->ifsc = $request->ifsc;

            $party->gst_paid_by = $request->gst_paid_by;

            if($request->material){
                $party->material_id = $request->material['id'];
            }
            else{
                $party->material_id = null;
            }

            $party->unit = $request->unit;
            $party->pay_mode = $request->pay_mode;
            $party->gr_chg = $request->gr_chg;
            $party->preset_dynamics = $request->preset_dynamics;
            $party->preset_on_account_id = $request->preset_on_account ? $request->preset_on_account['id'] :null;

            $party->pod_req = $request->pod_req;
            $party->door_del = $request->door_del;
            $party->gst_percent = $request->gst_percent;

            $party->cheque_img = $request->cheque_img;
            $party->pan_img = $request->pan_img;
            $party->tds_certificate_img = $request->tds_certificate_img;
            $party->gst_certificate_img = $request->gst_certificate_img;
            $party->other_img = $request->other_img;
            $party->updated_by = $request->user->id;
            $party->save();
            // NEW PARTY SAVED
            
            // SAVING NEWLY ADDED PARTY BRANCHES
            $newly_added_party_branches = $request->newly_added_party_branches;
            foreach ($newly_added_party_branches as $napb) {
                if(!$napb['branch_name'])
                continue;

                // Check if branch already exist
                $branch = PartyBranches::where('branch_name', $napb['branch_name'])->where('party_id', $party->id)->exists();
                if($branch){
                    return response()->json("Party Branch Name Already Exist", 500);
                }
                
                $pb = new PartyBranches();
                $pb->party_id = $party->id;
                $pb->branch_name = $napb['branch_name'];
                $pb->address = $napb['address'];
                $pb->state = $napb['state'];
                $pb->city = $napb['city'];
                $pb->pincode = $napb['pincode'];
                $pb->gst_number = $napb['gst_number'];
                $pb->contact_person = $napb['contact_person'];
                $pb->tele_number = $napb['tele_number'];
                $pb->save();
            }
            // NEWLY ADDED PARTY BRANCHES SAVED

            // SAVING UPDATES ON EXISTING PARTY BRANCHES 
            $updated_party_branches = $request->updated_party_branches;
            if( !empty($updated_party_branches) ){
                foreach ($updated_party_branches as $upb) {
                    if(!$upb['branch_name'])
                       continue;

                    // Check if branch already exist
                    $branch = PartyBranches::where('branch_name', $upb['branch_name'])->where('party_id', $party->id)->first();
                    if($branch && $branch->id != $upb['id']){
                        return response()->json("Party Branch Name Already Exist", 500);
                    }
                       
                        $pb = PartyBranches::find( $upb['id'] );
                        $pb->branch_name = $upb['branch_name'];
                        $pb->address = $upb['address'];
                        $pb->state = $upb['state'];
                        $pb->city = $upb['city'];
                        $pb->pincode = $upb['pincode'];
                        $pb->gst_number = $upb['gst_number'];
                        $pb->contact_person = $upb['contact_person'];
                        $pb->tele_number = $upb['tele_number'];
                        $pb->save();
                }
            }
            // UPDATES SAVED ON EXISTING PARTY BRANCHES

            // DELETING EXISTING PARTY BRANCHES
            $deleted_party_branches = $request->deleted_party_branches;
            if( !empty($deleted_party_branches) ){
                    $pb = PartyBranches::whereIn('id', $deleted_party_branches);
                    $pb->delete();
            }
            //DELETED EXISTING PARTY BRANCHES
            
            
            // IF ADD TO ACCOUNTS IS ALREADY TRUE IN DB
            if($party->add_to_accounts){
                if (!isFalsy($request->opening_bal_dr) && !isFalsy($request->opening_bal_cr)) {
                    return response()->json("Please Provide A Single Value For Opening Balances", 500);
                }
                if(!$request->add_to_accounts){
                    return response()->json("Cannot Remove The Account From Master", 500);
                }
                
                // SAVING NEWLY ADDED OPENING DETAILS
                $newly_added_opening_details = $request->newly_added_opening_details;
                foreach ($newly_added_opening_details as $naod) {
                    if(!$naod['against'] || !$naod['against_number'] || ( !$naod['dr'] && !$naod['cr'] ) )
                    continue;
                    
                    $pod = new PartyOpeningDetails();
                    $pod->party_id = $party->id;
                    $pod->against = $naod['against'];
                    $pod->against_number = $naod['against_number'];
                    $pod->date = $naod['date'];
                    $pod->desc = $naod['desc'];
                    $pod->rate = $naod['rate'];
                    $pod->dr = $naod['dr'];
                    $pod->cr = $naod['cr'];
                    $pod->paid_status = false;
                    $pod->fin_year = $request->current_year;
                    $pod->save();
                }
                // NEWLY ADDED OPENING DETAILS SAVED

                // SAVING UPDATES ON EXISTING OPENING DETAILS 
                $updated_opening_details = $request->updated_opening_details;
                if( !empty($updated_opening_details) ){
                    foreach ($updated_opening_details as $uod) {
                        if(!$uod['against'] || !$uod['against_number'] || ( !$uod['dr'] && !$uod['cr'] ) )
                        continue;
                        
                            $pb = PartyOpeningDetails::find( $uod['id'] );
                            $pb->date = $uod['date'];
                            $pb->desc = $uod['desc'];
                            $pb->rate = $uod['rate'];
                            $pb->dr = $uod['dr'];
                            $pb->cr = $uod['cr'];
                            $pb->save();
                    }
                }
                
                // UPDATES SAVED ON EXISTING OPENING DETAILS

                // DELETING EXISTING OPENING DETAILS
                $deleted_opening_details = $request->deleted_opening_details;
                if( !empty($deleted_opening_details) ){
                        $pb = PartyOpeningDetails::whereIn('id', $deleted_opening_details);
                        $pb->delete();
                }
                //DELETED EXISTING OPENING DETAILS
                 
                $grp = Group::where('group_name', $request->account_head)->first();

                $acc = Account::find($party->account_id);
                $acc->group_id = $grp->id;
                $acc->updated_by = $request->user->id;
                $acc->save();
                
                // UPDATING Balances
                $bal = Balances::where('account_id', $acc->id)->where('fin_year',$request->current_year)->first();
                $bal->opening_bal_dr = $request->opening_bal_dr;
                $bal->opening_bal_cr = $request->opening_bal_cr;
                $bal->save();
                // UPDATED BALANCES
            }
            // IF ADD TO ACCOUNTS IS FALSE IN DATABASE BUT TRUE IN REQUEST
            else if(!$party->add_to_accounts && $request->add_to_accounts) {
                if (!isFalsy($request->opening_bal_dr) && !isFalsy($request->opening_bal_cr)) {
                    return response()->json("Please Provide A Single Value For Opening Balances", 500);
                }

                // SAVING NEWLY ADDED OPENING DETAILS
                $newly_added_opening_details = $request->newly_added_opening_details;
                foreach ($newly_added_opening_details as $naod) {
                    if(!$naod['against'] || !$naod['against_number'] || ( !$naod['dr'] && !$naod['cr'] ) )
                    continue;
                    
                    $pod = new PartyOpeningDetails();
                    $pod->party_id = $party->id;
                    $pod->against = $naod['against'];
                    $pod->against_number = $naod['against_number'];
                    $pod->date = $naod['date'];
                    $pod->desc = $naod['desc'];
                    $pod->rate = $naod['rate'];
                    $pod->dr = $naod['dr'];
                    $pod->cr = $naod['cr'];
                    $pod->paid_status = false;
                    $pod->fin_year = $request->current_year;
                    $pod->save();
                }
                // NEWLY ADDED OPENING DETAILS SAVED

               
                // Saving New Account
                $grp = Group::where('group_name', $request->account_head)->first();

                // Saving New Account
                $new_account = new Account();
                $new_account->group_id = $grp->id;
                $new_account->account_name = $request->party_name;
                $new_account->account_type = 'Party';
                $new_account->created_by = $request->user->id;
                $new_account->save();
                // New Account Saved
               
                // Saving the Balances
                $bal = new Balances();
                $bal->account_id = $new_account->id;
                $bal->opening_bal_dr = $request->opening_bal_dr;
                $bal->opening_bal_cr = $request->opening_bal_cr;
                $bal->opening_bal_date = Carbon::now();
                $bal->fin_year = $request->current_year;
                $bal->save();
                // New Balances Saved
               
                $party->add_to_accounts = true;
                $party->account_id = $new_account->id;
                $party->save();
            }        

            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($party->id, $request->current_year);
            $this->createEditLog('updated', 'parties', $party->id, $party->party_name,$oldValue, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG
            

            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function destroy(Request $request, $pk){
        $test = $this->is_delete_permitted($request->user->id, 'PartyDetails');
        $test_delete = $test->delete;
        $delete_valid_till_hours = $test->delete_valid_till;

        if( !$test_delete ){
            return response()->json(['Permission Denied For The Action'],401);
        }

        // FETCHING OLD VALUE FOR LOG
        $oldValue = $this->get_data_for_obj($pk, $request->current_year);
        
        $party = Party::find($pk);

        if( $delete_valid_till_hours!=null ){
            $validity = Carbon::parse($party->created_at)->addHours($delete_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();

            $account = Account::where('id',$party->account_id)->first();
            $balances = Balances::where('account_id', $account->id)->delete();
            $account->delete();

        //FIND AND DELETE THE BRANCH
        $party->delete();
        // COMMISSIONS WILL BE CASCADED AUTOMATICALLY

        $this->createEditLog('deleted', 'parties', $party->id, $party->party_name, $oldValue, null, $request->user->id, $request->current_year); // CREATE EDIT LOG

                // TRANSACTIONS COMMITTED
                DB::commit();
                return response()->json("Success", 200); // RETURNED RESPONSE
    
            } catch (\Exception $e) {
                DB::rollback();
                return response()->json(['error' => $e->getMessage()], 500);
            }
    }

    public function get_party_from_gst_number(Request $request)
    {
        $gst_number = $request->gst_number;

        $partyExist = Party::select('id','party_name','party_code','address1','city','state','pincode','gst_number','tele_number','account_id')->where('gst_number', $gst_number)->first();

        return response()->json($partyExist);
        
    }

    public function autocomplete(Request $request){
        // Initialize the query builder
        $fields = ['id', 'party_name', 'party_code'];
        
        if ($request->fields) {
            $fields = array_merge($fields, $request->fields);
        }

        $query = Party::select($fields);

        // Apply any additional filters based on other columns as needed
        if ($request->has('filterby')) {
            $orWhereFilters = isset($request->filterby['orWhere']) ? $request->filterby['orWhere'] : [];
            $whereFilters = isset($request->filterby['where']) ? $request->filterby['where'] : [];
            $whereInFilters = isset($request->filterby['whereIn']) ? $request->filterby['whereIn'] : [];

           // Loop through the whereFilters array
           foreach ($whereFilters as $whereFilter) {
               // Apply the filter to the query
               $query->where($whereFilter['key'], $whereFilter['value']);
           }

           // Loop through the orWhereFilters array
           $query->where(function($query) use ($orWhereFilters) {
                foreach ($orWhereFilters as $orWhereFilter) {
                    // Apply the filter to the query
                    $query->orWhere($orWhereFilter['key'], $orWhereFilter['value']);
                }
            });

           foreach ($whereInFilters as $whereInFilter) {
                if(isset($whereInFilter['key']) && isset($whereInFilter['values']))
                $query->whereIn($whereInFilter['key'], $whereInFilter['values']);
           } 
        
        }
        
        // Check if searchVariable is provided and apply the filter
        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('party_name', 'like', '%'. $searchVariable . '%');
        }

        // Limit the number of results
        $query->take($request->limit)->orderBy('party_name', 'asc');

        // Retrieve the filtered 
        $parties = $query->get();
        return response(['party' => $parties, 'draw' => $request->draw]);
    }


    public function autocomplete_for_party_branches(Request $request){
        // Initialize the query builder
        $query = PartyBranches::select('id', 'branch_name')->where('party_id', $request->party_id);

        // Apply any additional filters based on other columns as needed
        if ($request->has('filterby')) {
            $orWhereFilters = isset($request->filterby['orWhere']) ? $request->filterby['orWhere'] : [];
            $whereFilters = isset($request->filterby['where']) ? $request->filterby['where'] : [];

           // Loop through the whereFilters array
           foreach ($whereFilters as $whereFilter) {
               // Apply the filter to the query
               $query->where($whereFilter['key'], $whereFilter['value']);
           }

           // Loop through the orWhereFilters array
           foreach ($orWhereFilters as $orWhereFilter) {
               // Apply the filter to the query
               $query->orWhere($orWhereFilter['key'], $orWhereFilter['value']);
           }
        
        }
        
        // Check if searchVariable is provided and apply the filter
        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('branch_name', 'like', '%'. $searchVariable . '%');
        }

        // Limit the number of results
        $query->take($request->limit)->orderBy('branch_name', 'asc');

        // Retrieve the filtered 
        $party_branches = $query->get();
        return response(['party_branches' => $party_branches, 'draw' => $request->draw]);
    }

    public function datatable(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'PartyDetails');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $query = Party::select('id', 'party_name', 'party_code', 'party_type', 'gst_number', 'contact_person', 'address1','state','city','tele_number','email')->orderBy($column, $dir);

        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('party_name', 'like', $searchValue . '%')
                    ->orWhere('party_code', 'like', $searchValue . '%')
                    ->orWhere('party_type', 'like', $searchValue . '%');
            });
        }

        $data = $query->paginate($length);

        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    public function check_unique(Request $request){
        $key = $request->key;
        $value = $request->value;

        if(!$value){
            return response()->json(['exists' => false]);
        }

        $exists = Party::where($key, $value)->exists();
        if($exists){
            return response()->json(['errors' => [ $key  => ['Already Exists !']]],422);
        }

        return response()->json(['exists' => false]);
    }

    public function get_custom_fields(Request $request){
        // Get only the fields present in $request->fields
        $fields = $request->fields;
        
        $query = Party::select($fields);

        if($request->has('with_relations'))
        {
            $with_relations = $request->with_relations;
            $query->with($with_relations);
        }
          
        
        
        $res = $query->find($request->party_id);
        return response()->json(['party' => $res]);
    }

    public function get_custom_fields_for_party_branches(Request $request){
        // Get only the fields present in $request->fields
        $fields = $request->fields;
        $party_branch = PartyBranches::select($fields)->find($request->party_branch_id);
        return response()->json(['party_branch' => $party_branch]);
    }

    public function get_party_report(Request $request){
        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $report_data = $request->report_data;

        $query = Party::select('id','party_name','party_code','party_type','contact_person','tele_number','address1','city','email','gst_number');

        if(isset($report_data['party'])){
            $query->where('id', $report_data['party']['id']);
        }

        if(isset($report_data['party_type']) && $report_data['party_type']!='All'){
            $query->where('party_type', $report_data['party_type']);
        }

        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('party_name', 'like', $searchValue . '%')
                    ->orWhere('party_code', 'like', $searchValue . '%')
                    ->orWhere('tele_number', 'like', $searchValue . '%');
            });
        }

        $data = $query->orderBy($column, $dir)->paginate($length);
          
        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }
    

    protected function get_data_for_obj($pk, $current_year){
        // FETCH AND SEND
        $party = Party::with(['party_branches' => function ($query) {
            $query->select('id', 'branch_name', 'address','state' ,'city', 'pincode', 'gst_number', 'contact_person', 'tele_number', 'party_id');
        },
        'material'=>function($query){
            $query->select('id','material_name');
        },
        'preset_on_account'=>function($query){
            $query->select('id','account_name','account_type');
        }, 
        'party_opening_details'=>function($query) use ($current_year){
            $query->where('fin_year', $current_year);
        },
        'branch:id,branch_name,branch_code'
        ])->find($pk);

        $balance = $party->account_id ? Balances::select('id','opening_bal_dr', 'opening_bal_cr', 'opening_bal_date')->where('fin_Year',$current_year)->where('account_id', $party->account_id)->first() : null;

        return ['party'=>$party,'balance'=>$balance];
    }

    protected function check_unique_filters($gst_number,$pan_number,$aadhar_number,$ignore_id){
        if( empty($gst_number) && empty($pan_number) && empty($aadhar_number) ){
            return ['status'=>false];
        }
        
        $query = Party::select('party_name');

        $query->where(function ($query) use ($gst_number, $pan_number, $aadhar_number) {
            if (!empty($gst_number)) {
                $query->orWhere('gst_number', $gst_number);
            }
    
            if (!empty($pan_number)) {
                $query->orWhere('pan_number', $pan_number);
            }
    
            if (!empty($aadhar_number)) {
                $query->orWhere('aadhar_number', $aadhar_number);
            }
        });

        if (!empty($ignore_id)) {
            $query->where('id', '!=', $ignore_id);
        }

        $exists = $query->first();


        if($exists){
            return ['status'=>true, 'party_name'=>$exists->party_name];
        }

        
        return ['status'=>false];
    }

}
