<?php

namespace App\Http\Controllers\Master;

use App\Http\Controllers\Controller;
use App\Http\Traits\EditLogTrait;
use App\Http\Traits\PermissionsTrait;
use App\Models\Master\PriceList;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class PriceListController extends Controller
{
    use PermissionsTrait;
    use EditLogTrait;

    public function index(Request $request){
        
        // Fetch all the price lists with the relationships on acc_party, source, destination, consignor, consignee
        $price_list = PriceList::with([
            'acc_party' => function ($query) {
                $query->select('id', 'account_name');
            },
            'source' => function ($query) {
                $query->select('id', 'station_name');
            },
            'destination' => function ($query) {
                $query->select('id', 'station_name');
            },
            'consignor' => function ($query) {
                $query->select('id', 'party_name', 'party_code');
            },
            'consignee' => function ($query) {
                $query->select('id', 'party_name', 'party_code');
            },
            'material' => function ($query) {
                $query->select('id', 'material_name');
            },
        ])
        ->when($request->price_number, function ($query) use ($request) {
            $query->where('id', $request->price_number);
        })
        ->when($request->acc_party, function ($query) use ($request) {
            $query->where('acc_party_id', $request->acc_party['id']);
        })
        ->when($request->destination, function ($query) use ($request) {
            $query->where('destination_id', $request->destination['id']);
        })
        ->when($request->material, function ($query) use ($request) {
            $query->where('material_id', $request->material['id']);
        })
        ->when($request->consignor, function ($query) use ($request) {
            $query->where('consignor_id', $request->consignor['id']);
        })
        ->when($request->consignee, function ($query) use ($request) {
            $query->where('consignee_id', $request->consignor['id']);
        })
        ->where(function ($query) use ($request) {
            // Check if the dates fall within the range
            if ($request->eff_from && $request->eff_till) {
                $query->whereDate('eff_from', '<=', $request->eff_till)
                    ->whereDate('eff_till', '>=', $request->eff_from);
            } elseif ($request->eff_from) {
                $query->whereDate('eff_till', '>=', $request->eff_from);
            } elseif ($request->eff_till) {
                $query->whereDate('eff_from', '<=', $request->eff_till);
            }
        })
        ->get();
        

        return response()->json(['price_list' => $price_list]);

    }

    public function show(Request $request, $pk){
            $price_list = PriceList::with([
                'acc_party' => function ($query) {
                    $query->select('id', 'account_name');
                },
                'source' => function ($query) {
                    $query->select('id', 'station_name');
                },
                'destination' => function ($query) {
                    $query->select('id', 'station_name');
                },
                'consignor' => function ($query) {
                    $query->select('id', 'party_name', 'party_code');
                },
                'consignee' => function ($query) {
                    $query->select('id', 'party_name', 'party_code');
                },
                'material' => function ($query) {
                    $query->select('id', 'material_name');
                },
            ])->find($pk);

            
            return response()->json(['price_list' => $price_list]);
    }

    public function store(Request $request){
        

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();  

            // Loop through $request->price_lists and create new PriceList for each of them.
            foreach ($request->price_list as $index => $pl) {
                if( !$pl['acc_party'] || !$pl['eff_from'] || !$pl['eff_till'] || !$pl['source'] || !$pl['destination'] || !$pl['rate'] || !$pl['min_wt_unit'])
                continue;
                 
                // Check if a similar price list already exists
                $exists = $this->find_duplicate($pl);
                $ind = $index + 1;
                if ($exists) {
                    return response()->json(['errors' => ['type'=>'Duplicate Entry', 'msg'=>"Mathing Record At Price Number ".$exists['id'], 'index'=>$ind, 'id'=>$exists['id']]], 422);
                }

                $pri_lis = new PriceList();

                $pri_lis->acc_party_id = $pl['acc_party'] ? $pl['acc_party']['id'] : null;
                
                $pri_lis->eff_from = $pl['eff_from'];
                $pri_lis->eff_till = $pl['eff_till'];

                $pri_lis->source_id = $pl['source'] ? $pl['source']['id'] : null;
                $pri_lis->destination_id = $pl['destination'] ? $pl['destination']['id'] : null;
                
        
                $pri_lis->consignor_id = $pl['consignor'] ? $pl['consignor']['id'] : null;
                $pri_lis->consignee_id = $pl['consignee'] ? $pl['consignee']['id'] : null;

                $pri_lis->material_id = $pl['material'] ? $pl['material']['id'] : null;

                $pri_lis->vehicle_type = $pl['vehicle_type'];
                $pri_lis->min_wt = $pl['min_wt'];
                $pri_lis->min_freight = $pl['min_freight'];
                $pri_lis->rate = $pl['rate'];
                $pri_lis->min_wt_unit = $pl['min_wt_unit'];
                $pri_lis->dynamic_options = json_encode($pl['dynamic_options']);
                
                $pri_lis->created_by = $request->user->id;
                $pri_lis->save();

                $newVal = $this->get_data_for_obj($pri_lis->id);
                $this->createEditLog('created', 'price_lists', $pri_lis->id, $pl['acc_party']['account_name'].' '.$pl['source']['station_name'].' '.$pl['destination']['station_name'] , null, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG
            }
            
            DB::commit();
            
            return response()->json(['success' => 'Price List Created Successfully']);

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['errors' => ['price_list' => [$e->getMessage()]]], 422);
        }    



    }

    public function update(Request $request){

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();  
            
            // Loop through $request->price_lists and create new PriceList for each of them.
            foreach ($request->newly_added_price_list as $index => $napl) {
                if( !$napl['acc_party'] || !$napl['eff_from'] || !$napl['eff_till'] || !$napl['source'] || !$napl['destination'] || !$napl['rate'] || !$napl['min_wt_unit'])
                continue;

                // Check if a similar price list already exists
                $exists = $this->find_duplicate($napl);
                $ind = $index + 1;
                if ($exists) {
                    return response()->json(['errors' => ['type'=>'Duplicate Entry', 'msg'=>"Mathing Record At Price Number ".$exists['id'], 'index'=>$ind, 'id'=>$exists['id']]], 422);
                }


                $pri_lis = new PriceList();
                $pri_lis->acc_party_id = $napl['acc_party'] ? $napl['acc_party']['id'] : null;

                $pri_lis->eff_from = $napl['eff_from'];
                $pri_lis->eff_till = $napl['eff_till'];

                $pri_lis->source_id = $napl['source'] ? $napl['source']['id'] : null;
                $pri_lis->destination_id = $napl['destination'] ? $napl['destination']['id'] : null;

                $pri_lis->consignor_id = $napl['consignor'] ? $napl['consignor']['id'] : null;
                $pri_lis->consignee_id = $napl['consignee'] ? $napl['consignee']['id'] : null;

                $pri_lis->material_id = $napl['material'] ? $napl['material']['id'] : null;

                $pri_lis->vehicle_type = $napl['vehicle_type'];
                $pri_lis->min_wt = $napl['min_wt'];
                $pri_lis->min_freight = $napl['min_freight'];
                $pri_lis->rate = $napl['rate'];
                $pri_lis->min_wt_unit = $napl['min_wt_unit'];
                $pri_lis->dynamic_options = json_encode($napl['dynamic_options']);
                $pri_lis->created_by = $request->user->id;
                $pri_lis->save();

                $newVal = $this->get_data_for_obj($pri_lis->id);
                $this->createEditLog('created', 'price_lists', $pri_lis->id, $napl['acc_party']['account_name'].' '.$napl['source']['station_name'].' '.$napl['destination']['station_name'] , null, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG
            }
             
            // Loop through $request->updated_price_list and update PriceList for each of them.
            foreach ($request->updated_price_list as $index => $upl) {
                if( !$upl['acc_party'] || !$upl['eff_from'] || !$upl['eff_till'] || !$upl['source'] || !$upl['destination'] || !$upl['rate'] || !$upl['min_wt_unit'])
                continue;

                // Check if a similar price list already exists
                $exists = $this->find_duplicate($upl);
                $ind = $index + 1;
                if ($exists) {
                    if($exists['id'] != $upl['id'])
                    return response()->json(['errors' => ['type'=>'Duplicate Entry', 'msg'=>"Mathing Record At Price Number ".$exists['id'], 'index'=>$ind, 'id'=>$exists['id']]], 422);
                }

                // FETCH OLD VALUE FOR LOG
                $oldValue = $this->get_data_for_obj($upl['id']);

                $pri_lis = PriceList::find($upl['id']);
                $pri_lis->eff_from = $upl['eff_from'];
                $pri_lis->eff_till = $upl['eff_till'];

                $pri_lis->source_id = $upl['source'] ? $upl['source']['id'] : null;
                $pri_lis->destination_id = $upl['destination'] ? $upl['destination']['id'] : null;

                $pri_lis->consignor_id = $upl['consignor'] ? $upl['consignor']['id'] : null;
                $pri_lis->consignee_id = $upl['consignee'] ? $upl['consignee']['id'] : null;

                $pri_lis->material_id = $upl['material'] ? $upl['material']['id'] : null;

                $pri_lis->vehicle_type = $upl['vehicle_type'];
                $pri_lis->min_wt = $upl['min_wt'];
                $pri_lis->min_freight = $upl['min_freight'];
                $pri_lis->rate = $upl['rate'];
                $pri_lis->min_wt_unit = $upl['min_wt_unit'];
                $pri_lis->dynamic_options = json_encode($upl['dynamic_options']);
                $pri_lis->updated_by = $request->user->id;
                $pri_lis->save();

                // Save Edit Log
                $newVal = $this->get_data_for_obj($pri_lis->id);
                $this->createEditLog('updated', 'price_lists', $pri_lis->id, $upl['acc_party']['account_name'].' '.$upl['source']['station_name'].' '.$upl['destination']['station_name'] , $oldValue, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG
            }

            // Loop through $request->deleted_price_list and delete PriceList for each of them.
            foreach ($request->deleted_price_list as $dpl) {
                // FETCH OLD VALUE FOR LOG
                $oldValue = $this->get_data_for_obj($dpl);


                $pri_lis = PriceList::find($dpl);
                $pri_lis->delete();

                // Save Edit Log
                $this->createEditLog('deleted', 'price_lists', $pri_lis->id, $oldValue['price_list']['acc_party']['account_name'].' '.$oldValue['price_list']['source']['station_name'].' '.$oldValue['price_list']['destination']['station_name'] , $oldValue, null, $request->user->id, $request->current_year); // CREATE EDIT LOG
            }

            DB::commit();

            return response()->json(['success' => 'Price List Updated Successfully']);
        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['errors' => ['price_list' => [$e->getMessage()]]], 422);
        }
    
    }

    public function destroy(Request $request){
        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();  
            
            // Loop through $request->deleted_price_list and delete PriceList for each of them.
            foreach ($request->deleted_price_list as $dpl) {

                // FETCH OLD VALUE FOR LOG
                $oldValue = $this->get_data_for_obj($dpl);

                $pri_lis = PriceList::find($dpl);
                $pri_lis->delete();

                // Save Edit Log
                $this->createEditLog('deleted', 'price_lists', $pri_lis->id, $oldValue['price_list']['acc_party']['account_name'].' '.$oldValue['price_list']['source']['station_name'].' '.$oldValue['price_list']['destination']['station_name'] , $oldValue, null, $request->user->id, $request->current_year); // CREATE EDIT LOG
            }

            DB::commit();
            return response()->json(['success' => 'Price List Deleted Successfully']);
            
        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['errors' => ['price_list' => [$e->getMessage()]]], 422);
        }
    }

    public function check_unique(Request $request){
        // Check if a similar price list already exists
        $exists = $this->find_duplicate($request->data);
        if ($exists) {
            return response()->json(['errors' => ['type'=>'Duplicate Entry', 'msg'=>"Mathing Record At Price Number ".$exists['id'], 'id'=>$exists['id']]], 422);
        }
        return response("Checked");
    }

    public function fetch_rate_for_gr_desc(Request $request){
        // Start building the query with the selected columns
        $query = PriceList::select('id','dynamic_options','rate')
        ->where('acc_party_id', $request->acc_party_id)
        ->where('source_id', $request->source_id)
        ->where('destination_id', $request->destination_id)
        ->where('min_wt_unit', $request->min_wt_unit);

        $consignor_id = $request->consignor_id;
        $consignee_id = $request->consignee_id;
        $material_id = $request->material_id;
        $vehicle_type = $request->vehicle_type;
        $gr_date = $request->gr_date;
        $min_wt = $request->min_wt;

        // Apply optional conditions using nested where clauses
        $query->where(function ($query) use ($consignor_id) {
            $query->where('consignor_id', $consignor_id)
                ->orWhereNull('consignor_id');
        });

        $query->where(function ($query) use ($consignee_id) {
            $query->where('consignee_id', $consignee_id)
                ->orWhereNull('consignee_id');
        });

        $query->where(function ($query) use ($material_id) {
            $query->where('material_id', $material_id)
                ->orWhereNull('material_id');
        });

        $query->where(function ($query) use ($vehicle_type) {
            $query->where('vehicle_type', $vehicle_type)
                ->orWhereNull('vehicle_type');
        });

        // Add the date range condition
        if ($gr_date) {
            $query->where('eff_from', '<=', $gr_date)
                ->where('eff_till', '>=', $gr_date);
        }


        if ($query->count() > 1) {
            if ($min_wt) {
                $query->where('min_wt', '>', $min_wt)
                    ->orWhereNull('min_wt');
            }
            $query->orderBy('min_wt', 'asc');
        }
        

        // Execute the query and get the results
        $pri_lis = $query->first();

        return response()->json(['pri_lis'=>$pri_lis]);

    }

    public function get_price_list_report(Request $request){
        $report_data = $request->report_data;
        $price_list = PriceList::with([
            'acc_party' => function ($query) {
                $query->select('id', 'account_name');
            },
            'source' => function ($query) {
                $query->select('id', 'station_name');
            },
            'destination' => function ($query) {
                $query->select('id', 'station_name');
            },
            'consignor' => function ($query) {
                $query->select('id', 'party_name', 'party_code');
            },
            'consignee' => function ($query) {
                $query->select('id', 'party_name', 'party_code');
            },
            'material' => function ($query) {
                $query->select('id', 'material_name');
            },
        ])
        ->when(isset($report_data['acc_party']), function ($query) use ($report_data) {
            $query->where('acc_party_id', $report_data['acc_party']['id']);
        })
        ->when(isset($report_data['consignor']), function ($query) use ($report_data) {
            $query->where('consignor_id', $report_data['consignor']['id']);
        })
        ->where(function ($query) use ($report_data) {
            // Check if the dates fall within the range
            if (isset($report_data['eff_from']) && isset($report_data['eff_till'])) {
                $query->whereDate('eff_from', '<=', $report_data['eff_till'])
                    ->whereDate('eff_till', '>=', $report_data['eff_from']);
            } elseif (isset($report_data['eff_from'])) {
                $query->whereDate('eff_till', '>=', $report_data['eff_from']);
            } elseif (isset($report_data['eff_till'])) {
                $query->whereDate('eff_from', '<=', $report_data['eff_till']);
            }
        })
        ->get();
        

        return response()->json(['data' => $price_list]);
    }

    protected function get_data_for_obj($pk){
        $price_list = PriceList::with([
            'acc_party' => function ($query) {
                $query->select('id', 'account_name');
            },
            'source' => function ($query) {
                $query->select('id', 'station_name');
            },
            'destination' => function ($query) {
                $query->select('id', 'station_name');
            },
            'consignor' => function ($query) {
                $query->select('id', 'party_name', 'party_code');
            },
            'consignee' => function ($query) {
                $query->select('id', 'party_name', 'party_code');
            },
            'material' => function ($query) {
                $query->select('id', 'material_name');
            },
        ])->find($pk);


            
        return ['price_list' => $price_list];
    }

    protected function find_duplicate($item){
        $eff_from = $item['eff_from'];
        $eff_till = $item['eff_till'];
        
        $exists = PriceList::where('acc_party_id', $item['acc_party']['id'])
                        ->where('source_id', $item['source']['id'])
                        ->where('destination_id', $item['destination']['id'])
                        ->where('consignor_id', $item['consignor'] ? $item['consignor']['id'] : null)
                        ->where('consignee_id', $item['consignee'] ? $item['consignee']['id'] : null)
                        ->where('material_id', $item['material'] ? $item['material']['id'] : null)
                        ->where('vehicle_type', $item['vehicle_type'] ? $item['vehicle_type'] : null)
                        ->where('min_wt', $item['min_wt'])
                        ->where('min_wt_unit', $item['min_wt_unit'])
                        ->where(function ($query) use ($eff_from, $eff_till) {
                            $query->whereBetween('eff_from', [$eff_from, $eff_till])
                                  ->orWhereBetween('eff_till', [$eff_from, $eff_till])
                                  ->orWhere(function ($query) use ($eff_from, $eff_till) {
                                      $query->where('eff_from', '<=', $eff_from)
                                            ->where('eff_till', '>=', $eff_till);
                                  });
                        })
                        ->when(isset($item['id']), function ($query) use ($item) {
                            $query->where('id', '!=', $item['id']);
                        })
                        ->first();
        return $exists;
    }


}
