<?php

namespace App\Http\Controllers\Master;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Http\Traits\EditLogTrait;
use App\Http\Traits\PermissionsTrait;
use App\Models\Master\Station;
use Illuminate\Support\Facades\DB;

use Carbon\Carbon;

class StationController extends Controller
{
    use PermissionsTrait;
    use EditLogTrait;

    public function index(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'StationDetails');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        //FETCH AND SEND ALL
        $station = Station::with(['branch' => function ($query) {
                        $query->select('id', 'branch_name','branch_code');
                    }])->get();
        return response()->json($station);
    }

    public function show(Request $request, $pk){
        // FETCH AND SEND
        $station = Station::with(['delivery_branch' => function ($query) {
                   $query->select('id', 'branch_name','branch_code');
                }])->find($pk);
        return response()->json(['station'=>$station]);
    }

    public function store(Request $request){
        
        $test = $this->is_new_permitted($request->user->id, 'StationDetails');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $validatedData = $request->validate([
            'station_name' => 'required|min:3|unique:stations,station_name'
        ], [
            'station_name.required' => 'Required',
            'station_name.unique' => 'Already Exist !',
            'station_name.min' => 'Minimum 3 Characters Required !'
        ]);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();  

            // SAVING A NEW STATION
            $station = new Station();
            $station->station_name = $request->station_name;
            $station->state = $request->state;
            $station->district = $request->district;
            $station->pincode = $request->pincode;

            if($request->delivery_branch){
                $station->branch_id = $request->delivery_branch['id'];
            }
            else{
                $station->branch_id = null;
            }
            
            $station->created_by = $request->user->id;
            $station->save(); 
            // NEW STATION SAVED

            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($station->id);
            $this->createEditLog('created', 'stations', $station->id, $station->station_name , null, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG
            

            // TRANSACTIONS COMMITTED
            DB::commit();

            return response()->json("Success", 200); // RETURNED RESPONSE

        } 
        // IF ANY ERROR WHILE SAVING
        catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function update(Request $request, $pk){   
        $test = $this->is_edit_permitted($request->user->id, 'StationDetails');
        $test_edit = $test->edit;
        $edit_valid_till_hours = $test->edit_valid_till;

        if( !$test_edit ){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $station = Station::find($pk);

        // FETCHING OLD VALUE FOR LOG
        $oldValue = $this->get_data_for_obj($pk);

        if( $edit_valid_till_hours!=null ){
            $validity = Carbon::parse($station->created_at)->addHours($edit_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }
          

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();

            // SAVING A NEW STATION
            $station->state = $request->state;
            $station->district = $request->district;
            $station->pincode = $request->pincode;

            if($request->delivery_branch){
                $station->branch_id = $request->delivery_branch['id'];
            }
            else{
                $station->branch_id = null;
            }
            $station->updated_by = $request->user->id;
            $station->save();
            // NEW STATION SAVED

            $newVal = $this->get_data_for_obj($station->id);
            $this->createEditLog('updated', 'stations', $station->id, $station->station_name, $oldValue, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG

            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function destroy(Request $request, $pk){
        $test = $this->is_delete_permitted($request->user->id, 'StationDetails');
        $test_delete = $test->delete;
        $delete_valid_till_hours = $test->delete_valid_till;

        if( !$test_delete ){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $station = Station::find($pk);

        if( $delete_valid_till_hours!=null ){
            $validity = Carbon::parse($station->created_at)->addHours($delete_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }

        // FETCHING OLD VALUE FOR LOG
        $oldValue = $this->get_data_for_obj($pk);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();


        //FIND AND DELETE THE STATION
        $station->delete();

        $this->createEditLog('deleted', 'stations', $station->id, $station->station_name, $oldValue, null, $request->user->id, $request->current_year); // CREATE EDIT LOG

            // TRANSACTIONS COMMITTED
            DB::commit();
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function autocomplete(Request $request){
        // WHILE SEARCHING THIS WILL RUN
        $fields = ['id', 'station_name'];
        
        if ($request->fields) {
            $fields = array_merge($fields, $request->fields);
        }

        $query = Station::select($fields);

        // Apply any additional filters based on other columns as needed
        if ($request->has('filterby')) {
            $orWhereFilters = isset($request->filterby['orWhere']) ? $request->filterby['orWhere'] : [];
            $whereFilters = isset($request->filterby['where']) ? $request->filterby['where'] : [];

           // Loop through the whereFilters array
           foreach ($whereFilters as $whereFilter) {
               // Apply the filter to the query
               $query->where($whereFilter['key'], $whereFilter['value']);
           }

           // Loop through the orWhereFilters array
           foreach ($orWhereFilters as $orWhereFilter) {
               // Apply the filter to the query
               $query->orWhere($orWhereFilter['key'], $orWhereFilter['value']);
           }
        
        }
        
        // Check if searchVariable is provided and apply the filter
        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('station_name', 'like', '%'. $searchVariable . '%');
        }

        // Limit the number of results
        $query->take($request->limit)->orderBy('station_name');

        // Retrieve the filtered 
        $station = $query->get();
        return response(['station' => $station, 'draw' => $request->draw]);
    }

    public function datatable(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'StationDetails');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $query = Station::select('id', 'station_name', 'state', 'district', 'pincode','branch_id')->with(['delivery_branch' => function ($query) {
                            $query->select('id', 'branch_name','branch_code');
                        }])->orderBy($column, $dir);

        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('station_name', 'like', $searchValue . '%');
            });
        }

        $data = $query->paginate($length);

        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    public function check_unique(Request $request){
        $key = $request->key;
        $value = $request->value;

        if(!$value){
            return response()->json(['exists' => false]);
        }

        $exists = Station::where($key, $value)->exists();
        if($exists){
            return response()->json(['errors' => [ $key  => ['Already Exists !']]],422);
        }

        return response()->json(['exists' => false]);
    }

    protected function get_data_for_obj($pk){
        $station = Station::with(['delivery_branch' => function ($query) {
            $query->select('id', 'branch_name','branch_code');
         }])->find($pk);
        return ['station'=>$station];
    }

}
