<?php

namespace App\Http\Controllers\Master;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Http\Traits\EditLogTrait;
use App\Http\Traits\PermissionsTrait;
use App\Models\Master\Unit;
use Illuminate\Support\Facades\DB;
use Carbon\Carbon;

class UnitController extends Controller
{
    use PermissionsTrait;
    use EditLogTrait;

    public function index(Request $request)
    {
        $test = $this->is_read_permitted($request->user->id, 'UnitDetails');
        if (!$test) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        // Fetch and send all units
        $units = Unit::select('id','unit_name','unit_symbol','desc')->get();
        return response()->json($units);
    }

    public function show(Request $request, $pk)
    {
        // Fetch and send the unit
        $unit = Unit::find($pk);
        return response()->json(['unit' => $unit]);
    }

    public function store(Request $request)
    {
        $test = $this->is_new_permitted($request->user->id, 'UnitDetails');
        if (!$test) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        $validatedData = $request->validate([
            'unit_name' => 'required|unique:units,unit_name',
            'unit_symbol' => 'required|unique:units,unit_symbol',
        ], [
            'unit_name.required' => 'Required',
            'unit_name.unique' => 'Already Exists!',
            'unit_symbol.required' => 'Required',
            'unit_symbol.unique' => 'Already Exists!',
        ]);

        try {
            // Transaction begins
            DB::beginTransaction();

            // Save a new unit
            $unit = new Unit();
            $unit->unit_name = $request->unit_name;
            $unit->unit_symbol = $request->unit_symbol;
            $unit->desc = $request->desc;
            $unit->created_by = $request->user->id;
            $unit->save();

            // Save the edit log
            $newVal = $this->get_data_for_obj($unit->id);
            $this->createEditLog('created', 'units', $unit->id, $unit->unit_name, null, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG


            // Transaction committed
            DB::commit();

            
            return response()->json("Success", 200); // Returned response

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function update(Request $request, $pk)
    {
        $test = $this->is_edit_permitted($request->user->id, 'UnitDetails');
        $test_edit = $test->edit;
        $edit_valid_till_hours = $test->edit_valid_till;

        if (!$test_edit) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        $unit = Unit::find($pk);

        // Fetch the old value for log
        $oldValue = $this->get_data_for_obj($pk);

        if ($edit_valid_till_hours != null) {
            $validity = Carbon::parse($unit->created_at)->addHours($edit_valid_till_hours);
            if (!$validity->isFuture()) {
                return response()->json(['Permission Timed Out'], 500);
            }
        }

        try {
            // Transaction begins
            DB::beginTransaction();

            // Update the unit
            $unit->unit_name = $request->unit_name;
            $unit->unit_symbol = $request->unit_symbol;
            $unit->desc = $request->desc;
            $unit->updated_by = $request->user->id;
            $unit->save();

            $newVal = $this->get_data_for_obj($unit->id);
            $this->createEditLog('updated', 'units', $unit->id, $unit->unit_name, $oldValue, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG

            // Transaction committed
            DB::commit();

            
            return response()->json("Success", 200); // Returned response

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function destroy(Request $request, $pk)
    {
        $test = $this->is_delete_permitted($request->user->id, 'UnitDetails');
        $test_delete = $test->delete;
        $delete_valid_till_hours = $test->delete_valid_till;

        if (!$test_delete) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        $unit = Unit::find($pk);

        if ($delete_valid_till_hours != null) {
            $validity = Carbon::parse($unit->created_at)->addHours($delete_valid_till_hours);
            if (!$validity->isFuture()) {
                return response()->json(['Permission Timed Out'], 500);
            }
        }

        // Fetch the old value for log
        $oldValue = $this->get_data_for_obj($pk);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();


        // Find and delete the unit
        $unit->delete();

        $this->createEditLog('deleted', 'units', $unit->id, $unit->unit_name, $oldValue, null, $request->user->id, $request->current_year); // CREATE EDIT LOG

            // TRANSACTIONS COMMITTED
            DB::commit();
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function autocomplete(Request $request)
    {
        // Perform search and return matching units
        // WHILE SEARCHING THIS WILL RUN
        $query = Unit::select('id', 'unit_name','unit_symbol');

        // Apply any additional filters based on other columns as needed
        if ($request->has('filterby')) {
            $orWhereFilters = isset($request->filterby['orWhere']) ? $request->filterby['orWhere'] : [];
            $whereFilters = isset($request->filterby['where']) ? $request->filterby['where'] : [];

           // Loop through the whereFilters array
           foreach ($whereFilters as $whereFilter) {
               // Apply the filter to the query
               $query->where($whereFilter['key'], $whereFilter['value']);
           }

           // Loop through the orWhereFilters array
           foreach ($orWhereFilters as $orWhereFilter) {
               // Apply the filter to the query
               $query->orWhere($orWhereFilter['key'], $orWhereFilter['value']);
           }
        
        }
        
        // Check if searchVariable is provided and apply the filter
        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('unit_name', 'like', '%'. $searchVariable . '%')
            ->orWhere('unit_symbol', 'like', '%'. $searchVariable . '%');
        }

        // Limit the number of results
        $query->take($request->limit)->orderBy('unit_symbol', 'asc');

        // Retrieve the filtered 
        $units = $query->get();
        return response(['units' => $units, 'draw' => $request->draw]);
    }

    public function check_unique(Request $request)
    {
        $key = $request->key;
        $value = $request->value;

        if (!$value) {
            return response()->json(['exists' => false]);
        }

        $exists = Unit::where($key, $value)->exists();
        if ($exists) {
            return response()->json(['errors' => [$key  => ['Already Exists !']]], 422);
        }

        return response()->json(['exists' => false]);
    }

    protected function get_data_for_obj($pk)
    {
        // Fetch and send the unit
        $unit = Unit::find($pk);
        return ['unit' => $unit];
    }
}
