<?php

namespace App\Http\Controllers\Master;

use App\Http\Controllers\Controller;
use App\Http\Traits\EditLogTrait;
use App\Http\Traits\PermissionsTrait;
use App\Models\Master\VehicleType;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class VehicleTypeController extends Controller
{
    use PermissionsTrait;
    use EditLogTrait;

    public function index(Request $request)
    {
        // $test = $this->is_read_permitted($request->user->id, 'VehicleTypeDetails');
        // if (!$test) {
        //     return response()->json(['Permission Denied For The Action'], 401);
        // }

        // Fetch and send all VehicleTypes
        $vehicle_types = VehicleType::select('id','vehicle_type')->get();
        return response()->json($vehicle_types);
    }

    public function show(Request $request, $pk)
    {
        // Fetch and send the VehicleType
        $vehicle_type = VehicleType::find($pk);
        return response()->json(['vehicle_type' => $vehicle_type]);
    }

    public function store(Request $request)
    {
        $test = $this->is_new_permitted($request->user->id, 'VehicleTypeDetails');
        if (!$test) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        $validatedData = $request->validate([
            'vehicle_type' => 'required|unique:vehicle_types',
        ], [
            'vehicle_type.required' => 'Required',
            'vehicle_type.unique' => 'Already Exists!',
        ]);

        try {
            // Transaction begins
            DB::beginTransaction();

            // Save a new VehicleType
            $vehicle_type = new VehicleType();
            $vehicle_type->vehicle_type = $request->vehicle_type;
            $vehicle_type->created_by = $request->user->id;
            $vehicle_type->save();

            // Save the edit log
            $newVal = $this->get_data_for_obj($vehicle_type->id);
            $this->createEditLog('created', 'vehicle_types', $vehicle_type->id, $vehicle_type->vehicle_type, null, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG


            // Transaction committed
            DB::commit();

            
            return response()->json("Success", 200); // Returned response

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function update(Request $request, $pk)
    {
        $test = $this->is_edit_permitted($request->user->id, 'VehicleTypeDetails');
        $test_edit = $test->edit;
        $edit_valid_till_hours = $test->edit_valid_till;

        if (!$test_edit) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        $vehicle_type = VehicleType::find($pk);

        // Fetch the old value for log
        $oldValue = $this->get_data_for_obj($pk);

        if ($edit_valid_till_hours != null) {
            $validity = Carbon::parse($vehicle_type->created_at)->addHours($edit_valid_till_hours);
            if (!$validity->isFuture()) {
                return response()->json(['Permission Timed Out'], 500);
            }
        }

        try {
            // Transaction begins
            DB::beginTransaction();

            // Update the VehicleType
            $vehicle_type->vehicle_type = $request->vehicle_type;
            $vehicle_type->updated_by = $request->user->id;
            $vehicle_type->save();

            $newVal = $this->get_data_for_obj($vehicle_type->id);
            $this->createEditLog('updated', 'vehicle_types', $vehicle_type->id, $vehicle_type->vehicle_type, $oldValue, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG

            // Transaction committed
            DB::commit();

            
            return response()->json("Success", 200); // Returned response

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function destroy(Request $request, $pk)
    {
        $test = $this->is_delete_permitted($request->user->id, 'VehicleTypeDetails');
        $test_delete = $test->delete;
        $delete_valid_till_hours = $test->delete_valid_till;

        if (!$test_delete) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        $vehicle_type = VehicleType::find($pk);

        if ($delete_valid_till_hours != null) {
            $validity = Carbon::parse($vehicle_type->created_at)->addHours($delete_valid_till_hours);
            if (!$validity->isFuture()) {
                return response()->json(['Permission Timed Out'], 500);
            }
        }

        // Fetch the old value for log
        $oldValue = $this->get_data_for_obj($pk);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();


        // Find and delete the VehicleType
        $vehicle_type->delete();

        $this->createEditLog('deleted', 'vehicle_types', $vehicle_type->id, $vehicle_type->vehicle_type, $oldValue, null, $request->user->id, $request->current_year); // CREATE EDIT LOG

            // TRANSACTIONS COMMITTED
            DB::commit();
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function autocomplete(Request $request)
    {
        // Perform search and return matching vehicle_types
        $query = VehicleType::select('id', 'vehicle_type');

        // Apply any additional filters based on other columns as needed
        if ($request->has('filterby')) {
            $orWhereFilters = isset($request->filterby['orWhere']) ? $request->filterby['orWhere'] : [];
            $whereFilters = isset($request->filterby['where']) ? $request->filterby['where'] : [];

           // Loop through the whereFilters array
           foreach ($whereFilters as $whereFilter) {
               // Apply the filter to the query
               $query->where($whereFilter['key'], $whereFilter['value']);
           }

           // Loop through the orWhereFilters array
           foreach ($orWhereFilters as $orWhereFilter) {
               // Apply the filter to the query
               $query->orWhere($orWhereFilter['key'], $orWhereFilter['value']);
           }
        
        }
        
        // Check if searchVariable is provided and apply the filter
        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('vehicle_type', 'like', '%'. $searchVariable . '%');
        }

        // Limit the number of results
        $query->take($request->limit)->orderBy('vehicle_type', 'asc');

        // Retrieve the filtered 
        $vehicle_types = $query->get();
        
        return response(['vehicle_types' => $vehicle_types, 'draw' => $request->draw]);
    }

    public function check_unique(Request $request)
    {
        $key = $request->key;
        $value = $request->value;

        if (!$value) {
            return response()->json(['exists' => false]);
        }

        $exists = VehicleType::where($key, $value)->exists();
        if ($exists) {
            return response()->json(['errors' => [$key  => ['Already Exists !']]], 422);
        }

        return response()->json(['exists' => false]);
    }

    protected function get_data_for_obj($pk)
    {
        // Fetch and send the vehicle_type
        $vehicle_type = VehicleType::find($pk);
        return ['vehicle_type' => $vehicle_type];
    }
}

